@@ -1,3 +1,8 @@
+/**
+ * @file
+ * @brief adjust directed graphs to improve layout aspect ratio
+ */
+
 /*************************************************************************
  * Copyright (c) 2011 AT&T Intellectual Property 
  * All rights reserved. This program and the accompanying materials
@@ -13,19 +18,18 @@
  * Written by Stephen North
  * Updated by Emden Gansner
  */
-#include "config.h"
 
 #include    <stdbool.h>
 #include    <stdio.h>
 #include    <stdlib.h>
 #include    <string.h>
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
 #include    <cgraph/cgraph.h>
+#include    <cgraph/exit.h>
+#include    <cgraph/unreachable.h>
 #include    <ingraphs/ingraphs.h>
 
 #include <getopt.h>
+#include "openFile.h"
 
 static bool Do_fans = false;
 static int MaxMinlen = 0;
@@ -142,19 +146,7 @@ static char *useString =
 static void usage(int v)
 {
     fprintf(stderr, useString, cmd);
-    exit(v);
-}
-
-static FILE *openFile(const char *name)
-{
-    FILE *fp;
-
-    fp = fopen(name, "w");
-    if (!fp) {
-	fprintf(stderr, "%s: could not open file %s for writing\n", cmd, name);
-	exit(-1);
-    }
-    return fp;
+    graphviz_exit(v);
 }
 
 static char **scanargs(int argc, char **argv)
@@ -181,7 +173,7 @@ static char **scanargs(int argc, char **argv)
 	case 'o':
 	    if (outFile != NULL)
 		fclose(outFile);
-	    outFile = openFile(optarg);
+	    outFile = openFile(cmd, optarg, "w");
 	    break;
 	case '?':
 	    if (optopt == '?')
@@ -197,6 +189,8 @@ static char **scanargs(int argc, char **argv)
 		    cmd, optopt);
 	    usage(-1);
 	    break;
+	default:
+	    UNREACHABLE();
 	}
     }
     if (Do_fans && MaxMinlen < 1)
@@ -229,5 +223,5 @@ int main(int argc, char **argv)
 	transform(g);
 	agwrite(g, outFile);
     }
-    return 0;
+    graphviz_exit(0);
 }
