@@ -75,12 +75,13 @@
 #define yychar          aagchar
 
 /* First part of user prologue.  */
-#line 19 "/cygdrive/d/a/scallywag/graphviz/graphviz-2.50.0-1.x86_64/src/graphviz-2.50.0/lib/cgraph/grammar.y"
+#line 19 "/cygdrive/d/a/scallywag/graphviz/graphviz-8.0.5-1.x86_64/src/graphviz-8.0.5/lib/cgraph/grammar.y"
 
 
 #include <stdbool.h>
 #include <stdio.h>
 #include <cghdr.h>
+#include <cgraph/alloc.h>
 #include <cgraph/unreachable.h>
 #include <stddef.h>
 extern void aagerror(const char*);
@@ -142,7 +143,7 @@ static	Agdisc_t	*Disc;		/* discipline passed to agread or agconcat */
 static gstack_t *S;
 
 
-#line 146 "grammar.c"
+#line 147 "grammar.c"
 
 # ifndef YY_CAST
 #  ifdef __cplusplus
@@ -192,43 +193,40 @@ enum yysymbol_kind_t
   YYSYMBOL_19_ = 19,                       /* '='  */
   YYSYMBOL_20_ = 20,                       /* '['  */
   YYSYMBOL_21_ = 21,                       /* ']'  */
-  YYSYMBOL_22_ = 22,                       /* '@'  */
-  YYSYMBOL_23_ = 23,                       /* '+'  */
-  YYSYMBOL_YYACCEPT = 24,                  /* $accept  */
-  YYSYMBOL_graph = 25,                     /* graph  */
-  YYSYMBOL_body = 26,                      /* body  */
-  YYSYMBOL_hdr = 27,                       /* hdr  */
-  YYSYMBOL_optgraphname = 28,              /* optgraphname  */
-  YYSYMBOL_optstrict = 29,                 /* optstrict  */
-  YYSYMBOL_graphtype = 30,                 /* graphtype  */
-  YYSYMBOL_optstmtlist = 31,               /* optstmtlist  */
-  YYSYMBOL_stmtlist = 32,                  /* stmtlist  */
-  YYSYMBOL_optsemi = 33,                   /* optsemi  */
-  YYSYMBOL_stmt = 34,                      /* stmt  */
-  YYSYMBOL_compound = 35,                  /* compound  */
-  YYSYMBOL_simple = 36,                    /* simple  */
-  YYSYMBOL_rcompound = 37,                 /* rcompound  */
-  YYSYMBOL_38_1 = 38,                      /* $@1  */
-  YYSYMBOL_39_2 = 39,                      /* $@2  */
-  YYSYMBOL_nodelist = 40,                  /* nodelist  */
-  YYSYMBOL_node = 41,                      /* node  */
-  YYSYMBOL_attrstmt = 42,                  /* attrstmt  */
-  YYSYMBOL_attrtype = 43,                  /* attrtype  */
-  YYSYMBOL_optmacroname = 44,              /* optmacroname  */
-  YYSYMBOL_optattr = 45,                   /* optattr  */
-  YYSYMBOL_attrlist = 46,                  /* attrlist  */
-  YYSYMBOL_optattrdefs = 47,               /* optattrdefs  */
-  YYSYMBOL_attrdefs = 48,                  /* attrdefs  */
-  YYSYMBOL_attritem = 49,                  /* attritem  */
-  YYSYMBOL_attrassignment = 50,            /* attrassignment  */
-  YYSYMBOL_attrmacro = 51,                 /* attrmacro  */
-  YYSYMBOL_graphattrdefs = 52,             /* graphattrdefs  */
-  YYSYMBOL_subgraph = 53,                  /* subgraph  */
-  YYSYMBOL_54_3 = 54,                      /* $@3  */
-  YYSYMBOL_optsubghdr = 55,                /* optsubghdr  */
-  YYSYMBOL_optseparator = 56,              /* optseparator  */
-  YYSYMBOL_atom = 57,                      /* atom  */
-  YYSYMBOL_qatom = 58                      /* qatom  */
+  YYSYMBOL_22_ = 22,                       /* '+'  */
+  YYSYMBOL_YYACCEPT = 23,                  /* $accept  */
+  YYSYMBOL_graph = 24,                     /* graph  */
+  YYSYMBOL_body = 25,                      /* body  */
+  YYSYMBOL_hdr = 26,                       /* hdr  */
+  YYSYMBOL_optgraphname = 27,              /* optgraphname  */
+  YYSYMBOL_optstrict = 28,                 /* optstrict  */
+  YYSYMBOL_graphtype = 29,                 /* graphtype  */
+  YYSYMBOL_optstmtlist = 30,               /* optstmtlist  */
+  YYSYMBOL_stmtlist = 31,                  /* stmtlist  */
+  YYSYMBOL_optsemi = 32,                   /* optsemi  */
+  YYSYMBOL_stmt = 33,                      /* stmt  */
+  YYSYMBOL_compound = 34,                  /* compound  */
+  YYSYMBOL_simple = 35,                    /* simple  */
+  YYSYMBOL_rcompound = 36,                 /* rcompound  */
+  YYSYMBOL_37_1 = 37,                      /* $@1  */
+  YYSYMBOL_38_2 = 38,                      /* $@2  */
+  YYSYMBOL_nodelist = 39,                  /* nodelist  */
+  YYSYMBOL_node = 40,                      /* node  */
+  YYSYMBOL_attrstmt = 41,                  /* attrstmt  */
+  YYSYMBOL_attrtype = 42,                  /* attrtype  */
+  YYSYMBOL_optmacroname = 43,              /* optmacroname  */
+  YYSYMBOL_optattr = 44,                   /* optattr  */
+  YYSYMBOL_attrlist = 45,                  /* attrlist  */
+  YYSYMBOL_optattrdefs = 46,               /* optattrdefs  */
+  YYSYMBOL_attrdefs = 47,                  /* attrdefs  */
+  YYSYMBOL_attrassignment = 48,            /* attrassignment  */
+  YYSYMBOL_graphattrdefs = 49,             /* graphattrdefs  */
+  YYSYMBOL_subgraph = 50,                  /* subgraph  */
+  YYSYMBOL_51_3 = 51,                      /* $@3  */
+  YYSYMBOL_optsubghdr = 52,                /* optsubghdr  */
+  YYSYMBOL_optseparator = 53,              /* optseparator  */
+  YYSYMBOL_atom = 54,                      /* atom  */
+  YYSYMBOL_qatom = 55                      /* qatom  */
 };
 typedef enum yysymbol_kind_t yysymbol_kind_t;
 
@@ -559,13 +557,13 @@ union yyalloc
 #define YYLAST   59
 
 /* YYNTOKENS -- Number of terminals.  */
-#define YYNTOKENS  24
+#define YYNTOKENS  23
 /* YYNNTS -- Number of nonterminals.  */
-#define YYNNTS  35
+#define YYNNTS  33
 /* YYNRULES -- Number of rules.  */
-#define YYNRULES  62
+#define YYNRULES  59
 /* YYNSTATES -- Number of states.  */
-#define YYNSTATES  80
+#define YYNSTATES  76
 
 /* YYMAXUTOK -- Last valid token kind.  */
 #define YYMAXUTOK   268
@@ -586,9 +584,9 @@ static const yytype_int8 yytranslate[] =
        2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
        2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
        2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
-       2,     2,     2,    23,    17,     2,     2,     2,     2,     2,
+       2,     2,     2,    22,    17,     2,     2,     2,     2,     2,
        2,     2,     2,     2,     2,     2,     2,     2,    18,    16,
-       2,    19,     2,     2,    22,     2,     2,     2,     2,     2,
+       2,    19,     2,     2,     2,     2,     2,     2,     2,     2,
        2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
        2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
        2,    20,     2,    21,     2,     2,     2,     2,     2,     2,
@@ -615,13 +613,12 @@ static const yytype_int8 yytranslate[] =
 /* YYRLINE[YYN] -- Source line where rule number YYN was defined.  */
 static const yytype_uint8 yyrline[] =
 {
-       0,   103,   103,   104,   105,   108,   110,   113,   113,   115,
-     115,   117,   117,   119,   119,   121,   121,   123,   123,   125,
-     126,   129,   133,   133,   135,   135,   135,   136,   140,   140,
-     142,   143,   144,   147,   148,   151,   152,   153,   156,   157,
-     160,   160,   162,   164,   165,   167,   170,   170,   172,   175,
-     178,   181,   181,   184,   185,   186,   189,   189,   189,   191,
-     192,   195,   196
+       0,   104,   104,   105,   106,   109,   111,   114,   114,   116,
+     116,   118,   118,   120,   120,   122,   122,   124,   124,   126,
+     127,   130,   134,   134,   136,   136,   136,   137,   141,   141,
+     143,   144,   145,   148,   149,   152,   153,   154,   157,   158,
+     161,   161,   163,   165,   166,   168,   171,   174,   177,   177,
+     180,   181,   182,   185,   185,   185,   187,   188,   191,   192
 };
 #endif
 
@@ -640,13 +637,12 @@ static const char *const yytname[] =
   "\"end of file\"", "error", "\"invalid token\"", "T_graph", "T_node",
   "T_edge", "T_digraph", "T_subgraph", "T_strict", "T_edgeop", "T_list",
   "T_attr", "T_atom", "T_qatom", "'{'", "'}'", "';'", "','", "':'", "'='",
-  "'['", "']'", "'@'", "'+'", "$accept", "graph", "body", "hdr",
-  "optgraphname", "optstrict", "graphtype", "optstmtlist", "stmtlist",
-  "optsemi", "stmt", "compound", "simple", "rcompound", "$@1", "$@2",
-  "nodelist", "node", "attrstmt", "attrtype", "optmacroname", "optattr",
-  "attrlist", "optattrdefs", "attrdefs", "attritem", "attrassignment",
-  "attrmacro", "graphattrdefs", "subgraph", "$@3", "optsubghdr",
-  "optseparator", "atom", "qatom", YY_NULLPTR
+  "'['", "']'", "'+'", "$accept", "graph", "body", "hdr", "optgraphname",
+  "optstrict", "graphtype", "optstmtlist", "stmtlist", "optsemi", "stmt",
+  "compound", "simple", "rcompound", "$@1", "$@2", "nodelist", "node",
+  "attrstmt", "attrtype", "optmacroname", "optattr", "attrlist",
+  "optattrdefs", "attrdefs", "attrassignment", "graphattrdefs", "subgraph",
+  "$@3", "optsubghdr", "optseparator", "atom", "qatom", YY_NULLPTR
 };
 
 static const char *
@@ -661,7 +657,7 @@ yysymbol_name (yysymbol_kind_t yysymbol)
 #define yypact_value_is_default(Yyn) \
   ((Yyn) == YYPACT_NINF)
 
-#define YYTABLE_NINF (-56)
+#define YYTABLE_NINF (-53)
 
 #define yytable_value_is_error(Yyn) \
   0
@@ -670,14 +666,14 @@ yysymbol_name (yysymbol_kind_t yysymbol)
    STATE-NUM.  */
 static const yytype_int8 yypact[] =
 {
-      18,   -18,   -18,    20,     9,     3,   -18,    -2,   -18,   -18,
-     -18,     1,   -18,   -18,   -18,     1,   -18,   -18,    10,    -2,
-     -18,    19,    25,    21,   -18,    19,     1,   -18,   -18,   -18,
-     -18,    11,    17,   -18,   -18,   -18,   -18,   -18,   -18,   -18,
-     -18,   -18,     1,   -18,   -18,    22,     9,     1,     1,    29,
-      15,    23,   -18,   -18,    26,    23,    27,   -18,   -18,    28,
-     -18,   -18,   -18,   -18,     1,    25,    -5,   -18,   -18,   -18,
-       1,   -18,    16,   -18,   -18,    30,   -18,   -18,   -18,   -18
+      17,   -18,   -18,    19,     8,     3,   -18,    -2,   -18,   -18,
+     -18,     1,   -18,   -18,   -18,     1,   -18,   -18,     9,    -2,
+     -18,    18,    21,    23,   -18,    18,     1,   -18,   -18,   -18,
+     -18,    10,    13,   -18,   -18,   -18,   -18,   -18,   -18,   -18,
+     -18,   -18,     1,   -18,   -18,    22,     8,     1,     1,    25,
+      14,    24,   -18,   -18,    27,    24,    26,   -18,   -18,    29,
+     -18,   -18,   -18,   -18,     1,    21,    -5,   -18,   -18,   -18,
+     -18,    16,    30,   -18,   -18,   -18
 };
 
 /* YYDEFACT[STATE-NUM] -- Default reduction number in state STATE-NUM.
@@ -686,22 +682,22 @@ static const yytype_int8 yypact[] =
 static const yytype_int8 yydefact[] =
 {
        0,     3,     9,     0,     0,     0,     1,    14,     2,    11,
-      12,     8,    35,    36,    37,    54,    59,    61,     0,    13,
-      16,    18,    27,    22,    28,    18,    39,    50,    34,    23,
-      51,    30,    60,     6,     7,    53,     5,    15,    17,    20,
+      12,     8,    35,    36,    37,    51,    56,    58,     0,    13,
+      16,    18,    27,    22,    28,    18,    39,    47,    34,    23,
+      48,    30,    57,     6,     7,    50,     5,    15,    17,    20,
       24,    41,     0,    19,    41,     0,     0,     0,     0,     0,
-      55,    21,    40,    29,    30,     0,    33,    38,    52,    31,
-      48,    62,    25,    44,     0,    27,     0,    32,    26,    42,
-       0,    43,    58,    46,    47,     0,    49,    56,    57,    45
+      52,    21,    40,    29,    30,     0,    33,    38,    49,    31,
+      46,    59,    25,    44,     0,    27,     0,    32,    26,    42,
+      43,    55,     0,    53,    54,    45
 };
 
 /* YYPGOTO[NTERM-NUM].  */
 static const yytype_int8 yypgoto[] =
 {
-     -18,   -18,    -1,   -18,   -18,   -18,   -18,   -18,   -18,    31,
-      32,   -18,     0,   -17,   -18,   -18,   -18,    12,   -18,   -18,
-     -18,     8,    13,   -18,   -18,   -18,    -8,   -18,   -18,   -18,
-     -18,   -18,   -18,   -11,   -18
+     -18,   -18,    -4,   -18,   -18,   -18,   -18,   -18,   -18,    31,
+      32,   -18,    -7,   -17,   -18,   -18,   -18,    12,   -18,   -18,
+     -18,     6,    15,   -18,   -18,   -14,   -18,   -18,   -18,   -18,
+     -18,   -11,   -18
 };
 
 /* YYDEFGOTO[NTERM-NUM].  */
@@ -709,8 +705,8 @@ static const yytype_int8 yydefgoto[] =
 {
        0,     3,     8,     4,    33,     5,    11,    18,    19,    39,
       20,    21,    22,    41,    50,    65,    23,    24,    25,    26,
-      44,    51,    52,    66,    71,    72,    27,    74,    28,    29,
-      46,    30,    79,    31,    32
+      44,    51,    52,    66,    70,    27,    28,    29,    46,    30,
+      75,    31,    32
 };
 
 /* YYTABLE[YYPACT[STATE-NUM]] -- What to do in state STATE-NUM.  If
@@ -719,47 +715,46 @@ static const yytype_int8 yydefgoto[] =
 static const yytype_int8 yytable[] =
 {
       34,    12,    13,    14,    35,    15,     9,    16,    17,    10,
-      16,    17,   -55,    16,    17,    45,    69,    70,    -4,     1,
-       6,   -10,    15,     7,   -10,    36,     2,    16,    17,    47,
-      48,    54,    77,    78,    40,    38,    59,    60,    42,    54,
-      49,    57,    61,    63,    47,    58,    64,   -40,    68,    48,
-      62,    37,    55,    67,    53,    75,    43,    56,    73,    76
+      16,    17,   -52,    16,    17,    45,    69,    -4,     1,     6,
+     -10,    15,     7,   -10,    36,     2,    16,    17,    47,    48,
+      40,    54,    73,    74,    38,    49,    59,    60,    61,    54,
+      42,    57,    58,    62,    63,    47,   -40,    64,    68,    48,
+      55,    37,    71,    67,    53,    72,    43,     0,     0,    56
 };
 
 static const yytype_int8 yycheck[] =
 {
       11,     3,     4,     5,    15,     7,     3,    12,    13,     6,
-      12,    13,    14,    12,    13,    26,    21,    22,     0,     1,
-       0,     3,     7,    14,     6,    15,     8,    12,    13,    18,
-      19,    42,    16,    17,     9,    16,    47,    48,    17,    50,
-      23,    19,    13,    20,    18,    46,    18,    20,    65,    19,
-      50,    19,    44,    64,    42,    66,    25,    44,    66,    70
+      12,    13,    14,    12,    13,    26,    21,     0,     1,     0,
+       3,     7,    14,     6,    15,     8,    12,    13,    18,    19,
+       9,    42,    16,    17,    16,    22,    47,    48,    13,    50,
+      17,    19,    46,    50,    20,    18,    20,    18,    65,    19,
+      44,    19,    66,    64,    42,    66,    25,    -1,    -1,    44
 };
 
 /* YYSTOS[STATE-NUM] -- The symbol kind of the accessing symbol of
    state STATE-NUM.  */
 static const yytype_int8 yystos[] =
 {
-       0,     1,     8,    25,    27,    29,     0,    14,    26,     3,
-       6,    30,     3,     4,     5,     7,    12,    13,    31,    32,
-      34,    35,    36,    40,    41,    42,    43,    50,    52,    53,
-      55,    57,    58,    28,    57,    57,    15,    34,    16,    33,
-       9,    37,    17,    33,    44,    57,    54,    18,    19,    23,
-      38,    45,    46,    41,    57,    45,    46,    19,    26,    57,
-      57,    13,    36,    20,    18,    39,    47,    57,    37,    21,
-      22,    48,    49,    50,    51,    57,    57,    16,    17,    56
+       0,     1,     8,    24,    26,    28,     0,    14,    25,     3,
+       6,    29,     3,     4,     5,     7,    12,    13,    30,    31,
+      33,    34,    35,    39,    40,    41,    42,    48,    49,    50,
+      52,    54,    55,    27,    54,    54,    15,    33,    16,    32,
+       9,    36,    17,    32,    43,    54,    51,    18,    19,    22,
+      37,    44,    45,    40,    54,    44,    45,    19,    25,    54,
+      54,    13,    35,    20,    18,    38,    46,    54,    36,    21,
+      47,    48,    54,    16,    17,    53
 };
 
 /* YYR1[RULE-NUM] -- Symbol kind of the left-hand side of rule RULE-NUM.  */
 static const yytype_int8 yyr1[] =
 {
-       0,    24,    25,    25,    25,    26,    27,    28,    28,    29,
-      29,    30,    30,    31,    31,    32,    32,    33,    33,    34,
-      34,    35,    36,    36,    38,    39,    37,    37,    40,    40,
-      41,    41,    41,    42,    42,    43,    43,    43,    44,    44,
-      45,    45,    46,    47,    47,    48,    49,    49,    50,    51,
-      52,    54,    53,    55,    55,    55,    56,    56,    56,    57,
-      57,    58,    58
+       0,    23,    24,    24,    24,    25,    26,    27,    27,    28,
+      28,    29,    29,    30,    30,    31,    31,    32,    32,    33,
+      33,    34,    35,    35,    37,    38,    36,    36,    39,    39,
+      40,    40,    40,    41,    41,    42,    42,    42,    43,    43,
+      44,    44,    45,    46,    46,    47,    48,    49,    51,    50,
+      52,    52,    52,    53,    53,    53,    54,    54,    55,    55
 };
 
 /* YYR2[RULE-NUM] -- Number of symbols on the right-hand side of rule RULE-NUM.  */
@@ -769,9 +764,8 @@ static const yytype_int8 yyr2[] =
        0,     1,     1,     1,     0,     2,     1,     1,     0,     2,
        2,     3,     1,     1,     0,     0,     5,     0,     1,     3,
        1,     3,     5,     3,     1,     1,     1,     1,     2,     0,
-       1,     0,     4,     2,     0,     2,     1,     1,     3,     2,
-       1,     0,     3,     2,     1,     0,     1,     1,     0,     1,
-       1,     1,     3
+       1,     0,     4,     2,     0,     2,     3,     1,     0,     3,
+       2,     1,     0,     1,     1,     0,     1,     1,     1,     3
 };
 
 
@@ -1235,217 +1229,211 @@ yyreduce:
   switch (yyn)
     {
   case 2: /* graph: hdr body  */
-#line 103 "/cygdrive/d/a/scallywag/graphviz/graphviz-2.50.0-1.x86_64/src/graphviz-2.50.0/lib/cgraph/grammar.y"
+#line 104 "/cygdrive/d/a/scallywag/graphviz/graphviz-8.0.5-1.x86_64/src/graphviz-8.0.5/lib/cgraph/grammar.y"
                             {freestack(); endgraph();}
-#line 1241 "grammar.c"
+#line 1235 "grammar.c"
     break;
 
   case 3: /* graph: error  */
-#line 104 "/cygdrive/d/a/scallywag/graphviz/graphviz-2.50.0-1.x86_64/src/graphviz-2.50.0/lib/cgraph/grammar.y"
+#line 105 "/cygdrive/d/a/scallywag/graphviz/graphviz-8.0.5-1.x86_64/src/graphviz-8.0.5/lib/cgraph/grammar.y"
                                         {if (G) {freestack(); endgraph(); agclose(G); G = Ag_G_global = NULL;}}
-#line 1247 "grammar.c"
+#line 1241 "grammar.c"
     break;
 
   case 6: /* hdr: optstrict graphtype optgraphname  */
-#line 110 "/cygdrive/d/a/scallywag/graphviz/graphviz-2.50.0-1.x86_64/src/graphviz-2.50.0/lib/cgraph/grammar.y"
+#line 111 "/cygdrive/d/a/scallywag/graphviz/graphviz-8.0.5-1.x86_64/src/graphviz-8.0.5/lib/cgraph/grammar.y"
                                                                  {startgraph((yyvsp[0].str),(yyvsp[-1].i) != 0,(yyvsp[-2].i) != 0);}
-#line 1253 "grammar.c"
+#line 1247 "grammar.c"
     break;
 
   case 7: /* optgraphname: atom  */
-#line 113 "/cygdrive/d/a/scallywag/graphviz/graphviz-2.50.0-1.x86_64/src/graphviz-2.50.0/lib/cgraph/grammar.y"
+#line 114 "/cygdrive/d/a/scallywag/graphviz/graphviz-8.0.5-1.x86_64/src/graphviz-8.0.5/lib/cgraph/grammar.y"
                      {(yyval.str)=(yyvsp[0].str);}
-#line 1259 "grammar.c"
+#line 1253 "grammar.c"
     break;
 
   case 8: /* optgraphname: %empty  */
-#line 113 "/cygdrive/d/a/scallywag/graphviz/graphviz-2.50.0-1.x86_64/src/graphviz-2.50.0/lib/cgraph/grammar.y"
+#line 114 "/cygdrive/d/a/scallywag/graphviz/graphviz-8.0.5-1.x86_64/src/graphviz-8.0.5/lib/cgraph/grammar.y"
                                             {(yyval.str)=0;}
-#line 1265 "grammar.c"
+#line 1259 "grammar.c"
     break;
 
   case 9: /* optstrict: T_strict  */
-#line 115 "/cygdrive/d/a/scallywag/graphviz/graphviz-2.50.0-1.x86_64/src/graphviz-2.50.0/lib/cgraph/grammar.y"
+#line 116 "/cygdrive/d/a/scallywag/graphviz/graphviz-8.0.5-1.x86_64/src/graphviz-8.0.5/lib/cgraph/grammar.y"
                              {(yyval.i)=1;}
-#line 1271 "grammar.c"
+#line 1265 "grammar.c"
     break;
 
   case 10: /* optstrict: %empty  */
-#line 115 "/cygdrive/d/a/scallywag/graphviz/graphviz-2.50.0-1.x86_64/src/graphviz-2.50.0/lib/cgraph/grammar.y"
+#line 116 "/cygdrive/d/a/scallywag/graphviz/graphviz-8.0.5-1.x86_64/src/graphviz-8.0.5/lib/cgraph/grammar.y"
                                                     {(yyval.i)=0;}
-#line 1277 "grammar.c"
+#line 1271 "grammar.c"
     break;
 
   case 11: /* graphtype: T_graph  */
-#line 117 "/cygdrive/d/a/scallywag/graphviz/graphviz-2.50.0-1.x86_64/src/graphviz-2.50.0/lib/cgraph/grammar.y"
+#line 118 "/cygdrive/d/a/scallywag/graphviz/graphviz-8.0.5-1.x86_64/src/graphviz-8.0.5/lib/cgraph/grammar.y"
                                 {(yyval.i) = 0;}
-#line 1283 "grammar.c"
+#line 1277 "grammar.c"
     break;
 
   case 12: /* graphtype: T_digraph  */
-#line 117 "/cygdrive/d/a/scallywag/graphviz/graphviz-2.50.0-1.x86_64/src/graphviz-2.50.0/lib/cgraph/grammar.y"
+#line 118 "/cygdrive/d/a/scallywag/graphviz/graphviz-8.0.5-1.x86_64/src/graphviz-8.0.5/lib/cgraph/grammar.y"
                                                           {(yyval.i) = 1;}
-#line 1289 "grammar.c"
+#line 1283 "grammar.c"
     break;
 
   case 21: /* compound: simple rcompound optattr  */
-#line 130 "/cygdrive/d/a/scallywag/graphviz/graphviz-2.50.0-1.x86_64/src/graphviz-2.50.0/lib/cgraph/grammar.y"
+#line 131 "/cygdrive/d/a/scallywag/graphviz/graphviz-8.0.5-1.x86_64/src/graphviz-8.0.5/lib/cgraph/grammar.y"
                                         {if ((yyvsp[-1].i)) endedge(); else endnode();}
-#line 1295 "grammar.c"
+#line 1289 "grammar.c"
     break;
 
   case 24: /* $@1: %empty  */
-#line 135 "/cygdrive/d/a/scallywag/graphviz/graphviz-2.50.0-1.x86_64/src/graphviz-2.50.0/lib/cgraph/grammar.y"
+#line 136 "/cygdrive/d/a/scallywag/graphviz/graphviz-8.0.5-1.x86_64/src/graphviz-8.0.5/lib/cgraph/grammar.y"
                                  {getedgeitems();}
-#line 1301 "grammar.c"
+#line 1295 "grammar.c"
     break;
 
   case 25: /* $@2: %empty  */
-#line 135 "/cygdrive/d/a/scallywag/graphviz/graphviz-2.50.0-1.x86_64/src/graphviz-2.50.0/lib/cgraph/grammar.y"
+#line 136 "/cygdrive/d/a/scallywag/graphviz/graphviz-8.0.5-1.x86_64/src/graphviz-8.0.5/lib/cgraph/grammar.y"
                                                           {getedgeitems();}
-#line 1307 "grammar.c"
+#line 1301 "grammar.c"
     break;
 
   case 26: /* rcompound: T_edgeop $@1 simple $@2 rcompound  */
-#line 135 "/cygdrive/d/a/scallywag/graphviz/graphviz-2.50.0-1.x86_64/src/graphviz-2.50.0/lib/cgraph/grammar.y"
+#line 136 "/cygdrive/d/a/scallywag/graphviz/graphviz-8.0.5-1.x86_64/src/graphviz-8.0.5/lib/cgraph/grammar.y"
                                                                                       {(yyval.i) = 1;}
-#line 1313 "grammar.c"
+#line 1307 "grammar.c"
     break;
 
   case 27: /* rcompound: %empty  */
-#line 136 "/cygdrive/d/a/scallywag/graphviz/graphviz-2.50.0-1.x86_64/src/graphviz-2.50.0/lib/cgraph/grammar.y"
+#line 137 "/cygdrive/d/a/scallywag/graphviz/graphviz-8.0.5-1.x86_64/src/graphviz-8.0.5/lib/cgraph/grammar.y"
                                             {(yyval.i) = 0;}
-#line 1319 "grammar.c"
+#line 1313 "grammar.c"
     break;
 
   case 30: /* node: atom  */
-#line 142 "/cygdrive/d/a/scallywag/graphviz/graphviz-2.50.0-1.x86_64/src/graphviz-2.50.0/lib/cgraph/grammar.y"
+#line 143 "/cygdrive/d/a/scallywag/graphviz/graphviz-8.0.5-1.x86_64/src/graphviz-8.0.5/lib/cgraph/grammar.y"
                        {appendnode((yyvsp[0].str),NULL,NULL);}
-#line 1325 "grammar.c"
+#line 1319 "grammar.c"
     break;
 
   case 31: /* node: atom ':' atom  */
-#line 143 "/cygdrive/d/a/scallywag/graphviz/graphviz-2.50.0-1.x86_64/src/graphviz-2.50.0/lib/cgraph/grammar.y"
+#line 144 "/cygdrive/d/a/scallywag/graphviz/graphviz-8.0.5-1.x86_64/src/graphviz-8.0.5/lib/cgraph/grammar.y"
                             {appendnode((yyvsp[-2].str),(yyvsp[0].str),NULL);}
-#line 1331 "grammar.c"
+#line 1325 "grammar.c"
     break;
 
   case 32: /* node: atom ':' atom ':' atom  */
-#line 144 "/cygdrive/d/a/scallywag/graphviz/graphviz-2.50.0-1.x86_64/src/graphviz-2.50.0/lib/cgraph/grammar.y"
+#line 145 "/cygdrive/d/a/scallywag/graphviz/graphviz-8.0.5-1.x86_64/src/graphviz-8.0.5/lib/cgraph/grammar.y"
                                      {appendnode((yyvsp[-4].str),(yyvsp[-2].str),(yyvsp[0].str));}
-#line 1337 "grammar.c"
+#line 1331 "grammar.c"
     break;
 
   case 33: /* attrstmt: attrtype optmacroname attrlist  */
-#line 147 "/cygdrive/d/a/scallywag/graphviz/graphviz-2.50.0-1.x86_64/src/graphviz-2.50.0/lib/cgraph/grammar.y"
+#line 148 "/cygdrive/d/a/scallywag/graphviz/graphviz-8.0.5-1.x86_64/src/graphviz-8.0.5/lib/cgraph/grammar.y"
                                                   {attrstmt((yyvsp[-2].i),(yyvsp[-1].str));}
-#line 1343 "grammar.c"
+#line 1337 "grammar.c"
     break;
 
   case 34: /* attrstmt: graphattrdefs  */
-#line 148 "/cygdrive/d/a/scallywag/graphviz/graphviz-2.50.0-1.x86_64/src/graphviz-2.50.0/lib/cgraph/grammar.y"
+#line 149 "/cygdrive/d/a/scallywag/graphviz/graphviz-8.0.5-1.x86_64/src/graphviz-8.0.5/lib/cgraph/grammar.y"
                                          {attrstmt(T_graph,NULL);}
-#line 1349 "grammar.c"
+#line 1343 "grammar.c"
     break;
 
   case 35: /* attrtype: T_graph  */
-#line 151 "/cygdrive/d/a/scallywag/graphviz/graphviz-2.50.0-1.x86_64/src/graphviz-2.50.0/lib/cgraph/grammar.y"
+#line 152 "/cygdrive/d/a/scallywag/graphviz/graphviz-8.0.5-1.x86_64/src/graphviz-8.0.5/lib/cgraph/grammar.y"
                         {(yyval.i) = T_graph;}
-#line 1355 "grammar.c"
+#line 1349 "grammar.c"
     break;
 
   case 36: /* attrtype: T_node  */
-#line 152 "/cygdrive/d/a/scallywag/graphviz/graphviz-2.50.0-1.x86_64/src/graphviz-2.50.0/lib/cgraph/grammar.y"
+#line 153 "/cygdrive/d/a/scallywag/graphviz/graphviz-8.0.5-1.x86_64/src/graphviz-8.0.5/lib/cgraph/grammar.y"
                                  {(yyval.i) = T_node;}
-#line 1361 "grammar.c"
+#line 1355 "grammar.c"
     break;
 
   case 37: /* attrtype: T_edge  */
-#line 153 "/cygdrive/d/a/scallywag/graphviz/graphviz-2.50.0-1.x86_64/src/graphviz-2.50.0/lib/cgraph/grammar.y"
+#line 154 "/cygdrive/d/a/scallywag/graphviz/graphviz-8.0.5-1.x86_64/src/graphviz-8.0.5/lib/cgraph/grammar.y"
                                  {(yyval.i) = T_edge;}
-#line 1367 "grammar.c"
+#line 1361 "grammar.c"
     break;
 
   case 38: /* optmacroname: atom '='  */
-#line 156 "/cygdrive/d/a/scallywag/graphviz/graphviz-2.50.0-1.x86_64/src/graphviz-2.50.0/lib/cgraph/grammar.y"
+#line 157 "/cygdrive/d/a/scallywag/graphviz/graphviz-8.0.5-1.x86_64/src/graphviz-8.0.5/lib/cgraph/grammar.y"
                         {(yyval.str) = (yyvsp[-1].str);}
-#line 1373 "grammar.c"
+#line 1367 "grammar.c"
     break;
 
   case 39: /* optmacroname: %empty  */
-#line 157 "/cygdrive/d/a/scallywag/graphviz/graphviz-2.50.0-1.x86_64/src/graphviz-2.50.0/lib/cgraph/grammar.y"
+#line 158 "/cygdrive/d/a/scallywag/graphviz/graphviz-8.0.5-1.x86_64/src/graphviz-8.0.5/lib/cgraph/grammar.y"
                                       {(yyval.str) = NULL; }
+#line 1373 "grammar.c"
+    break;
+
+  case 46: /* attrassignment: atom '=' atom  */
+#line 171 "/cygdrive/d/a/scallywag/graphviz/graphviz-8.0.5-1.x86_64/src/graphviz-8.0.5/lib/cgraph/grammar.y"
+                                 {appendattr((yyvsp[-2].str),(yyvsp[0].str));}
 #line 1379 "grammar.c"
     break;
 
-  case 48: /* attrassignment: atom '=' atom  */
-#line 172 "/cygdrive/d/a/scallywag/graphviz/graphviz-2.50.0-1.x86_64/src/graphviz-2.50.0/lib/cgraph/grammar.y"
-                                 {appendattr((yyvsp[-2].str),(yyvsp[0].str));}
+  case 48: /* $@3: %empty  */
+#line 177 "/cygdrive/d/a/scallywag/graphviz/graphviz-8.0.5-1.x86_64/src/graphviz-8.0.5/lib/cgraph/grammar.y"
+                              {opensubg((yyvsp[0].str));}
 #line 1385 "grammar.c"
     break;
 
-  case 49: /* attrmacro: '@' atom  */
-#line 175 "/cygdrive/d/a/scallywag/graphviz/graphviz-2.50.0-1.x86_64/src/graphviz-2.50.0/lib/cgraph/grammar.y"
-                                 {appendattr((yyvsp[0].str),NULL);}
+  case 49: /* subgraph: optsubghdr $@3 body  */
+#line 177 "/cygdrive/d/a/scallywag/graphviz/graphviz-8.0.5-1.x86_64/src/graphviz-8.0.5/lib/cgraph/grammar.y"
+                                                   {closesubg();}
 #line 1391 "grammar.c"
     break;
 
-  case 51: /* $@3: %empty  */
-#line 181 "/cygdrive/d/a/scallywag/graphviz/graphviz-2.50.0-1.x86_64/src/graphviz-2.50.0/lib/cgraph/grammar.y"
-                              {opensubg((yyvsp[0].str));}
+  case 50: /* optsubghdr: T_subgraph atom  */
+#line 180 "/cygdrive/d/a/scallywag/graphviz/graphviz-8.0.5-1.x86_64/src/graphviz-8.0.5/lib/cgraph/grammar.y"
+                                  {(yyval.str)=(yyvsp[0].str);}
 #line 1397 "grammar.c"
     break;
 
-  case 52: /* subgraph: optsubghdr $@3 body  */
-#line 181 "/cygdrive/d/a/scallywag/graphviz/graphviz-2.50.0-1.x86_64/src/graphviz-2.50.0/lib/cgraph/grammar.y"
-                                                   {closesubg();}
+  case 51: /* optsubghdr: T_subgraph  */
+#line 181 "/cygdrive/d/a/scallywag/graphviz/graphviz-8.0.5-1.x86_64/src/graphviz-8.0.5/lib/cgraph/grammar.y"
+                                      {(yyval.str)=NULL;}
 #line 1403 "grammar.c"
     break;
 
-  case 53: /* optsubghdr: T_subgraph atom  */
-#line 184 "/cygdrive/d/a/scallywag/graphviz/graphviz-2.50.0-1.x86_64/src/graphviz-2.50.0/lib/cgraph/grammar.y"
-                                  {(yyval.str)=(yyvsp[0].str);}
+  case 52: /* optsubghdr: %empty  */
+#line 182 "/cygdrive/d/a/scallywag/graphviz/graphviz-8.0.5-1.x86_64/src/graphviz-8.0.5/lib/cgraph/grammar.y"
+                                      {(yyval.str)=NULL;}
 #line 1409 "grammar.c"
     break;
 
-  case 54: /* optsubghdr: T_subgraph  */
-#line 185 "/cygdrive/d/a/scallywag/graphviz/graphviz-2.50.0-1.x86_64/src/graphviz-2.50.0/lib/cgraph/grammar.y"
-                                      {(yyval.str)=NULL;}
+  case 56: /* atom: T_atom  */
+#line 187 "/cygdrive/d/a/scallywag/graphviz/graphviz-8.0.5-1.x86_64/src/graphviz-8.0.5/lib/cgraph/grammar.y"
+                  {(yyval.str) = (yyvsp[0].str);}
 #line 1415 "grammar.c"
     break;
 
-  case 55: /* optsubghdr: %empty  */
-#line 186 "/cygdrive/d/a/scallywag/graphviz/graphviz-2.50.0-1.x86_64/src/graphviz-2.50.0/lib/cgraph/grammar.y"
-                                      {(yyval.str)=NULL;}
+  case 57: /* atom: qatom  */
+#line 188 "/cygdrive/d/a/scallywag/graphviz/graphviz-8.0.5-1.x86_64/src/graphviz-8.0.5/lib/cgraph/grammar.y"
+                                 {(yyval.str) = (yyvsp[0].str);}
 #line 1421 "grammar.c"
     break;
 
-  case 59: /* atom: T_atom  */
-#line 191 "/cygdrive/d/a/scallywag/graphviz/graphviz-2.50.0-1.x86_64/src/graphviz-2.50.0/lib/cgraph/grammar.y"
-                  {(yyval.str) = (yyvsp[0].str);}
+  case 58: /* qatom: T_qatom  */
+#line 191 "/cygdrive/d/a/scallywag/graphviz/graphviz-8.0.5-1.x86_64/src/graphviz-8.0.5/lib/cgraph/grammar.y"
+                   {(yyval.str) = (yyvsp[0].str);}
 #line 1427 "grammar.c"
     break;
 
-  case 60: /* atom: qatom  */
-#line 192 "/cygdrive/d/a/scallywag/graphviz/graphviz-2.50.0-1.x86_64/src/graphviz-2.50.0/lib/cgraph/grammar.y"
-                                 {(yyval.str) = (yyvsp[0].str);}
+  case 59: /* qatom: qatom '+' T_qatom  */
+#line 192 "/cygdrive/d/a/scallywag/graphviz/graphviz-8.0.5-1.x86_64/src/graphviz-8.0.5/lib/cgraph/grammar.y"
+                                             {(yyval.str) = concat((yyvsp[-2].str),(yyvsp[0].str));}
 #line 1433 "grammar.c"
     break;
 
-  case 61: /* qatom: T_qatom  */
-#line 195 "/cygdrive/d/a/scallywag/graphviz/graphviz-2.50.0-1.x86_64/src/graphviz-2.50.0/lib/cgraph/grammar.y"
-                   {(yyval.str) = (yyvsp[0].str);}
-#line 1439 "grammar.c"
-    break;
 
-  case 62: /* qatom: qatom '+' T_qatom  */
-#line 196 "/cygdrive/d/a/scallywag/graphviz/graphviz-2.50.0-1.x86_64/src/graphviz-2.50.0/lib/cgraph/grammar.y"
-                                             {(yyval.str) = concat((yyvsp[-2].str),(yyvsp[0].str));}
-#line 1445 "grammar.c"
-    break;
-
-
-#line 1449 "grammar.c"
+#line 1437 "grammar.c"
 
       default: break;
     }
@@ -1638,7 +1626,7 @@ yyreturnlab:
   return yyresult;
 }
 
-#line 198 "/cygdrive/d/a/scallywag/graphviz/graphviz-2.50.0-1.x86_64/src/graphviz-2.50.0/lib/cgraph/grammar.y"
+#line 194 "/cygdrive/d/a/scallywag/graphviz/graphviz-8.0.5-1.x86_64/src/graphviz-8.0.5/lib/cgraph/grammar.y"
 
 
 static item *newitem(int tag, void *p0, char *p1)
@@ -1671,7 +1659,7 @@ static gstack_t *push(gstack_t *s, Agraph_t *subg) {
 static gstack_t *pop(gstack_t *s)
 {
 	gstack_t *rv;
-	rv = S->down;
+	rv = s->down;
 	agfree(G,s);
 	return rv;
 }
@@ -1682,10 +1670,8 @@ static void delete_items(item *ilist)
 
 	for (p = ilist; p; p = pn) {
 		pn = p->next;
-		switch(p->tag) {
-			case T_list: delete_items(p->u.list); break;
-			case T_atom: case T_attr: agstrfree(G,p->str); break;
-		}
+		if (p->tag == T_list) delete_items(p->u.list);
+		if (p->tag == T_atom) agstrfree(G,p->str);
 		agfree(G,p);
 	}
 }
@@ -1883,7 +1869,7 @@ concat (char* s1, char* s2)
   size_t len = strlen(s1) + strlen(s2) + 1;
 
   if (len <= BUFSIZ) sym = buf;
-  else sym = malloc(len);
+  else sym = gv_alloc(len);
   strcpy(sym,s1);
   strcat(sym,s2);
   s = agstrdup (G,sym);
@@ -1904,7 +1890,7 @@ concatPort (char* s1, char* s2)
   size_t len = strlen(s1) + strlen(s2) + 2;  /* one more for ':' */
 
   if (len <= BUFSIZ) sym = buf;
-  else sym = malloc(len);
+  else sym = gv_alloc(len);
   sprintf (sym, "%s:%s", s1, s2);
   s = agstrdup (G,sym);
   agstrfree (G,s1);
@@ -1965,13 +1951,9 @@ static void newedge(Agnode_t *t, char *tport, Agnode_t *h, char *hport, char *ke
 
 static void startgraph(char *name, bool directed, bool strict)
 {
-	static Agdesc_t	req;	/* get rid of warnings */
-
 	if (G == NULL) {
 		SubgraphDepth = 0;
-		req.directed = directed;
-		req.strict = strict;
-		req.maingraph = TRUE;
+		Agdesc_t req = {.directed = directed, .strict = strict, .maingraph = true};
 		Ag_G_global = G = agopen(name,req,Disc);
 	}
 	else {
