@@ -75,9 +75,10 @@
 #define yychar          htmlchar
 
 /* First part of user prologue.  */
-#line 19 "/cygdrive/d/a/scallywag/graphviz/graphviz-2.50.0-1.x86_64/src/graphviz-2.50.0/lib/common/htmlparse.y"
+#line 19 "/cygdrive/d/a/scallywag/graphviz/graphviz-8.0.5-1.x86_64/src/graphviz-8.0.5/lib/common/htmlparse.y"
 
 
+#include <cgraph/alloc.h>
 #include <common/render.h>
 #include <common/htmltable.h>
 #include <common/htmllex.h>
@@ -106,6 +107,9 @@ static struct {
 static void
 free_ritem(Dt_t* d, pitem* p,Dtdisc_t* ds)
 {
+  (void)d;
+  (void)ds;
+
   dtclose (p->u.rp);
   free (p);
 }
@@ -117,6 +121,9 @@ free_ritem(Dt_t* d, pitem* p,Dtdisc_t* ds)
 static void
 free_item(Dt_t* d, void* p,Dtdisc_t* ds)
 {
+  (void)d;
+  (void)ds;
+
   free (p);
 }
 
@@ -149,31 +156,24 @@ cleanCell (htmlcell_t* cp)
 static void
 free_citem(Dt_t* d, pitem* p,Dtdisc_t* ds)
 {
+  (void)d;
+  (void)ds;
+
   cleanCell (p->u.cp);
   free (p);
 }
 
 static Dtdisc_t rowDisc = {
-    offsetof(pitem,u),
-    sizeof(void*),
-    offsetof(pitem,link),
-    NIL(Dtmake_f),
-    (Dtfree_f)free_ritem,
-    NIL(Dtcompar_f),
-    NIL(Dthash_f),
-    NIL(Dtmemory_f),
-    NIL(Dtevent_f)
+    .key = offsetof(pitem, u),
+    .size = sizeof(void *),
+    .link = offsetof(pitem, link),
+    .freef = (Dtfree_f)free_ritem,
 };
 static Dtdisc_t cellDisc = {
-    offsetof(pitem,u),
-    sizeof(void*),
-    offsetof(pitem,link),
-    NIL(Dtmake_f),
-    (Dtfree_f)free_item,
-    NIL(Dtcompar_f),
-    NIL(Dthash_f),
-    NIL(Dtmemory_f),
-    NIL(Dtevent_f)
+    .key = offsetof(pitem, u),
+    .size = sizeof(void *),
+    .link = offsetof(pitem, link),
+    .freef = (Dtfree_f)free_item,
 };
 
 typedef struct {
@@ -189,6 +189,9 @@ typedef struct {
 static void 
 free_fitem(Dt_t* d, fitem* p, Dtdisc_t* ds)
 {
+    (void)d;
+    (void)ds;
+
     free (p->ti.str);
     free (p);
 }
@@ -196,12 +199,14 @@ free_fitem(Dt_t* d, fitem* p, Dtdisc_t* ds)
 static void 
 free_fspan(Dt_t* d, fspan* p, Dtdisc_t* ds)
 {
+    (void)d;
+    (void)ds;
+
     textspan_t* ti;
 
     if (p->lp.nitems) {
-	int i;
 	ti = p->lp.items;
-	for (i = 0; i < p->lp.nitems; i++) {
+	for (size_t i = 0; i < p->lp.nitems; i++) {
 	    free (ti->str);
 	    ti++;
 	}
@@ -211,28 +216,13 @@ free_fspan(Dt_t* d, fspan* p, Dtdisc_t* ds)
 }
 
 static Dtdisc_t fstrDisc = {
-    0,
-    0,
-    offsetof(fitem,link),
-    NIL(Dtmake_f),
-    (Dtfree_f)free_item,
-    NIL(Dtcompar_f),
-    NIL(Dthash_f),
-    NIL(Dtmemory_f),
-    NIL(Dtevent_f)
+    .link = offsetof(fitem, link),
+    .freef = (Dtfree_f)free_item,
 };
 
-
 static Dtdisc_t fspanDisc = {
-    0,
-    0,
-    offsetof(fspan,link),
-    NIL(Dtmake_f),
-    (Dtfree_f)free_item,
-    NIL(Dtcompar_f),
-    NIL(Dthash_f),
-    NIL(Dtmemory_f),
-    NIL(Dtevent_f)
+    .link = offsetof(fspan, link),
+    .freef = (Dtfree_f)free_item,
 };
 
 /* appendFItemList:
@@ -241,7 +231,7 @@ static Dtdisc_t fspanDisc = {
 static void
 appendFItemList (agxbuf *ag)
 {
-    fitem *fi = NEW(fitem);
+    fitem *fi = gv_alloc(sizeof(fitem));
 
     fi->ti.str = agxbdisown(ag);
     fi->ti.font = HTMLstate.fontstack->cfont;
@@ -253,12 +243,11 @@ appendFItemList (agxbuf *ag)
 static void 
 appendFLineList (int v)
 {
-    int cnt;
-    fspan *ln = NEW(fspan);
+    fspan *ln = gv_alloc(sizeof(fspan));
     fitem *fi;
     Dt_t *ilist = HTMLstate.fitemList;
 
-    cnt = dtsize(ilist);
+    size_t cnt = (size_t)dtsize(ilist);
     ln->lp.just = v;
     if (cnt) {
         int i = 0;
@@ -266,7 +255,7 @@ appendFLineList (int v)
 	ln->lp.items = N_NEW(cnt, textspan_t);
 
 	fi = (fitem*)dtflatten(ilist);
-	for (; fi; fi = (fitem*)dtlink(fitemList,(Dtlink_t*)fi)) {
+	for (; fi; fi = (fitem*)dtlink(fitemList, fi)) {
 		/* NOTE: When fitemList is closed, it uses free_item, which only frees the container,
 		 * not the contents, so this copy is safe.
 		 */
@@ -275,9 +264,9 @@ appendFLineList (int v)
 	}
     }
     else {
-	ln->lp.items = NEW(textspan_t);
+	ln->lp.items = gv_alloc(sizeof(textspan_t));
 	ln->lp.nitems = 1;
-	ln->lp.items[0].str = strdup("");
+	ln->lp.items[0].str = gv_strdup("");
 	ln->lp.items[0].font = HTMLstate.fontstack->cfont;
     }
 
@@ -289,15 +278,14 @@ appendFLineList (int v)
 static htmltxt_t*
 mkText(void)
 {
-    int cnt;
     Dt_t * ispan = HTMLstate.fspanList;
     fspan *fl ;
-    htmltxt_t *hft = NEW(htmltxt_t);
+    htmltxt_t *hft = gv_alloc(sizeof(htmltxt_t));
     
     if (dtsize (HTMLstate.fitemList)) 
 	appendFLineList (UNSET_ALIGN);
 
-    cnt = dtsize(ispan);
+    size_t cnt = (size_t)dtsize(ispan);
     hft->nspans = cnt;
     	
     if (cnt) {
@@ -328,9 +316,9 @@ static pitem* addRow (void)
 {
   Dt_t*      dp = dtopen(&cellDisc, Dtqueue);
   htmltbl_t* tbl = HTMLstate.tblstack;
-  pitem*     sp = NEW(pitem);
+  pitem*     sp = gv_alloc(sizeof(pitem));
   sp->u.rp = dp;
-  if (tbl->flags & HTML_HRULE)
+  if (tbl->hrule)
     sp->ruled = 1;
   dtinsert (tbl->u.p.rows, sp);
   return sp;
@@ -339,38 +327,36 @@ static pitem* addRow (void)
 /* setCell:
  * Set cell body and type and attach to row
  */
-static void setCell (htmlcell_t* cp, void* obj, int kind)
-{
-  pitem*     sp = NEW(pitem);
+static void setCell(htmlcell_t *cp, void *obj, char kind) {
+  pitem*     sp = gv_alloc(sizeof(pitem));
   htmltbl_t* tbl = HTMLstate.tblstack;
   pitem*     rp = dtlast (tbl->u.p.rows);
   Dt_t*      row = rp->u.rp;
   sp->u.cp = cp;
   dtinsert (row, sp);
   cp->child.kind = kind;
-  if (tbl->flags & HTML_VRULE)
+  if (tbl->vrule)
     cp->ruled = HTML_VRULE;
   
   if(kind == HTML_TEXT)
-  	cp->child.u.txt = (htmltxt_t*)obj;
+  	cp->child.u.txt = obj;
   else if (kind == HTML_IMAGE)
-    cp->child.u.img = (htmlimg_t*)obj;
+    cp->child.u.img = obj;
   else
-    cp->child.u.tbl = (htmltbl_t*)obj;
+    cp->child.u.tbl = obj;
 }
 
 /* mkLabel:
  * Create label, given body and type.
  */
-static htmllabel_t* mkLabel (void* obj, int kind)
-{
-  htmllabel_t* lp = NEW(htmllabel_t);
+static htmllabel_t *mkLabel(void *obj, char kind) {
+  htmllabel_t* lp = gv_alloc(sizeof(htmllabel_t));
 
   lp->kind = kind;
   if (kind == HTML_TEXT)
-    lp->u.txt = (htmltxt_t*)obj;
+    lp->u.txt = obj;
   else
-    lp->u.tbl = (htmltbl_t*)obj;
+    lp->u.tbl = obj;
   return lp;
 }
 
@@ -442,7 +428,7 @@ static int nonSpace (char* s)
 static void
 pushFont (textfont_t *fp)
 {
-    sfont_t *ft = NEW(sfont_t);
+    sfont_t *ft = gv_alloc(sizeof(sfont_t));
     textfont_t* curfont = HTMLstate.fontstack->cfont;
     textfont_t  f = *fp;
 
@@ -475,7 +461,7 @@ popFont (void)
 }
 
 
-#line 479 "htmlparse.c"
+#line 465 "htmlparse.c"
 
 # ifndef YY_CAST
 #  ifdef __cplusplus
@@ -972,13 +958,13 @@ static const yytype_int8 yytranslate[] =
 /* YYRLINE[YYN] -- Source line where rule number YYN was defined.  */
 static const yytype_int16 yyrline[] =
 {
-       0,   451,   451,   452,   453,   456,   459,   460,   463,   464,
-     465,   466,   467,   468,   469,   470,   471,   472,   475,   478,
-     481,   484,   487,   490,   493,   496,   499,   502,   505,   508,
-     511,   514,   517,   520,   523,   524,   527,   528,   531,   531,
-     552,   553,   554,   555,   556,   557,   560,   561,   564,   565,
-     566,   569,   569,   572,   573,   574,   577,   577,   578,   578,
-     579,   579,   580,   580,   583,   584,   587,   588,   591,   592
+       0,   437,   437,   438,   439,   442,   445,   446,   449,   450,
+     451,   452,   453,   454,   455,   456,   457,   458,   461,   464,
+     467,   470,   473,   476,   479,   482,   485,   488,   491,   494,
+     497,   500,   503,   506,   509,   510,   513,   514,   517,   517,
+     538,   539,   540,   541,   542,   543,   546,   547,   550,   551,
+     552,   555,   555,   558,   559,   560,   563,   563,   564,   564,
+     565,   565,   566,   566,   569,   570,   573,   574,   577,   578
 };
 #endif
 
@@ -1651,151 +1637,151 @@ yyreduce:
   switch (yyn)
     {
   case 2: /* html: T_html fonttext T_end_html  */
-#line 451 "/cygdrive/d/a/scallywag/graphviz/graphviz-2.50.0-1.x86_64/src/graphviz-2.50.0/lib/common/htmlparse.y"
+#line 437 "/cygdrive/d/a/scallywag/graphviz/graphviz-8.0.5-1.x86_64/src/graphviz-8.0.5/lib/common/htmlparse.y"
                                    { HTMLstate.lbl = mkLabel((yyvsp[-1].txt),HTML_TEXT); }
-#line 1657 "htmlparse.c"
+#line 1643 "htmlparse.c"
     break;
 
   case 3: /* html: T_html fonttable T_end_html  */
-#line 452 "/cygdrive/d/a/scallywag/graphviz/graphviz-2.50.0-1.x86_64/src/graphviz-2.50.0/lib/common/htmlparse.y"
+#line 438 "/cygdrive/d/a/scallywag/graphviz/graphviz-8.0.5-1.x86_64/src/graphviz-8.0.5/lib/common/htmlparse.y"
                                     { HTMLstate.lbl = mkLabel((yyvsp[-1].tbl),HTML_TBL); }
-#line 1663 "htmlparse.c"
+#line 1649 "htmlparse.c"
     break;
 
   case 4: /* html: error  */
-#line 453 "/cygdrive/d/a/scallywag/graphviz/graphviz-2.50.0-1.x86_64/src/graphviz-2.50.0/lib/common/htmlparse.y"
+#line 439 "/cygdrive/d/a/scallywag/graphviz/graphviz-8.0.5-1.x86_64/src/graphviz-8.0.5/lib/common/htmlparse.y"
               { cleanup(); YYABORT; }
-#line 1669 "htmlparse.c"
+#line 1655 "htmlparse.c"
     break;
 
   case 5: /* fonttext: text  */
-#line 456 "/cygdrive/d/a/scallywag/graphviz/graphviz-2.50.0-1.x86_64/src/graphviz-2.50.0/lib/common/htmlparse.y"
+#line 442 "/cygdrive/d/a/scallywag/graphviz/graphviz-8.0.5-1.x86_64/src/graphviz-8.0.5/lib/common/htmlparse.y"
                 { (yyval.txt) = mkText(); }
-#line 1675 "htmlparse.c"
+#line 1661 "htmlparse.c"
     break;
 
   case 8: /* textitem: string  */
-#line 463 "/cygdrive/d/a/scallywag/graphviz/graphviz-2.50.0-1.x86_64/src/graphviz-2.50.0/lib/common/htmlparse.y"
+#line 449 "/cygdrive/d/a/scallywag/graphviz/graphviz-8.0.5-1.x86_64/src/graphviz-8.0.5/lib/common/htmlparse.y"
                   { appendFItemList(HTMLstate.str);}
-#line 1681 "htmlparse.c"
+#line 1667 "htmlparse.c"
     break;
 
   case 9: /* textitem: br  */
-#line 464 "/cygdrive/d/a/scallywag/graphviz/graphviz-2.50.0-1.x86_64/src/graphviz-2.50.0/lib/common/htmlparse.y"
+#line 450 "/cygdrive/d/a/scallywag/graphviz/graphviz-8.0.5-1.x86_64/src/graphviz-8.0.5/lib/common/htmlparse.y"
               {appendFLineList((yyvsp[0].i));}
-#line 1687 "htmlparse.c"
+#line 1673 "htmlparse.c"
     break;
 
   case 18: /* font: T_font  */
-#line 475 "/cygdrive/d/a/scallywag/graphviz/graphviz-2.50.0-1.x86_64/src/graphviz-2.50.0/lib/common/htmlparse.y"
+#line 461 "/cygdrive/d/a/scallywag/graphviz/graphviz-8.0.5-1.x86_64/src/graphviz-8.0.5/lib/common/htmlparse.y"
               { pushFont ((yyvsp[0].font)); }
-#line 1693 "htmlparse.c"
+#line 1679 "htmlparse.c"
     break;
 
   case 19: /* n_font: T_end_font  */
-#line 478 "/cygdrive/d/a/scallywag/graphviz/graphviz-2.50.0-1.x86_64/src/graphviz-2.50.0/lib/common/htmlparse.y"
+#line 464 "/cygdrive/d/a/scallywag/graphviz/graphviz-8.0.5-1.x86_64/src/graphviz-8.0.5/lib/common/htmlparse.y"
                     { popFont (); }
-#line 1699 "htmlparse.c"
+#line 1685 "htmlparse.c"
     break;
 
   case 20: /* italic: T_italic  */
-#line 481 "/cygdrive/d/a/scallywag/graphviz/graphviz-2.50.0-1.x86_64/src/graphviz-2.50.0/lib/common/htmlparse.y"
+#line 467 "/cygdrive/d/a/scallywag/graphviz/graphviz-8.0.5-1.x86_64/src/graphviz-8.0.5/lib/common/htmlparse.y"
                   {pushFont((yyvsp[0].font));}
-#line 1705 "htmlparse.c"
+#line 1691 "htmlparse.c"
     break;
 
   case 21: /* n_italic: T_n_italic  */
-#line 484 "/cygdrive/d/a/scallywag/graphviz/graphviz-2.50.0-1.x86_64/src/graphviz-2.50.0/lib/common/htmlparse.y"
+#line 470 "/cygdrive/d/a/scallywag/graphviz/graphviz-8.0.5-1.x86_64/src/graphviz-8.0.5/lib/common/htmlparse.y"
                       {popFont();}
-#line 1711 "htmlparse.c"
+#line 1697 "htmlparse.c"
     break;
 
   case 22: /* bold: T_bold  */
-#line 487 "/cygdrive/d/a/scallywag/graphviz/graphviz-2.50.0-1.x86_64/src/graphviz-2.50.0/lib/common/htmlparse.y"
+#line 473 "/cygdrive/d/a/scallywag/graphviz/graphviz-8.0.5-1.x86_64/src/graphviz-8.0.5/lib/common/htmlparse.y"
               {pushFont((yyvsp[0].font));}
-#line 1717 "htmlparse.c"
+#line 1703 "htmlparse.c"
     break;
 
   case 23: /* n_bold: T_n_bold  */
-#line 490 "/cygdrive/d/a/scallywag/graphviz/graphviz-2.50.0-1.x86_64/src/graphviz-2.50.0/lib/common/htmlparse.y"
+#line 476 "/cygdrive/d/a/scallywag/graphviz/graphviz-8.0.5-1.x86_64/src/graphviz-8.0.5/lib/common/htmlparse.y"
                   {popFont();}
-#line 1723 "htmlparse.c"
+#line 1709 "htmlparse.c"
     break;
 
   case 24: /* strike: T_s  */
-#line 493 "/cygdrive/d/a/scallywag/graphviz/graphviz-2.50.0-1.x86_64/src/graphviz-2.50.0/lib/common/htmlparse.y"
+#line 479 "/cygdrive/d/a/scallywag/graphviz/graphviz-8.0.5-1.x86_64/src/graphviz-8.0.5/lib/common/htmlparse.y"
              {pushFont((yyvsp[0].font));}
-#line 1729 "htmlparse.c"
+#line 1715 "htmlparse.c"
     break;
 
   case 25: /* n_strike: T_n_s  */
-#line 496 "/cygdrive/d/a/scallywag/graphviz/graphviz-2.50.0-1.x86_64/src/graphviz-2.50.0/lib/common/htmlparse.y"
+#line 482 "/cygdrive/d/a/scallywag/graphviz/graphviz-8.0.5-1.x86_64/src/graphviz-8.0.5/lib/common/htmlparse.y"
                  {popFont();}
-#line 1735 "htmlparse.c"
+#line 1721 "htmlparse.c"
     break;
 
   case 26: /* underline: T_underline  */
-#line 499 "/cygdrive/d/a/scallywag/graphviz/graphviz-2.50.0-1.x86_64/src/graphviz-2.50.0/lib/common/htmlparse.y"
+#line 485 "/cygdrive/d/a/scallywag/graphviz/graphviz-8.0.5-1.x86_64/src/graphviz-8.0.5/lib/common/htmlparse.y"
                         {pushFont((yyvsp[0].font));}
-#line 1741 "htmlparse.c"
+#line 1727 "htmlparse.c"
     break;
 
   case 27: /* n_underline: T_n_underline  */
-#line 502 "/cygdrive/d/a/scallywag/graphviz/graphviz-2.50.0-1.x86_64/src/graphviz-2.50.0/lib/common/htmlparse.y"
+#line 488 "/cygdrive/d/a/scallywag/graphviz/graphviz-8.0.5-1.x86_64/src/graphviz-8.0.5/lib/common/htmlparse.y"
                             {popFont();}
-#line 1747 "htmlparse.c"
+#line 1733 "htmlparse.c"
     break;
 
   case 28: /* overline: T_overline  */
-#line 505 "/cygdrive/d/a/scallywag/graphviz/graphviz-2.50.0-1.x86_64/src/graphviz-2.50.0/lib/common/htmlparse.y"
+#line 491 "/cygdrive/d/a/scallywag/graphviz/graphviz-8.0.5-1.x86_64/src/graphviz-8.0.5/lib/common/htmlparse.y"
                       {pushFont((yyvsp[0].font));}
-#line 1753 "htmlparse.c"
+#line 1739 "htmlparse.c"
     break;
 
   case 29: /* n_overline: T_n_overline  */
-#line 508 "/cygdrive/d/a/scallywag/graphviz/graphviz-2.50.0-1.x86_64/src/graphviz-2.50.0/lib/common/htmlparse.y"
+#line 494 "/cygdrive/d/a/scallywag/graphviz/graphviz-8.0.5-1.x86_64/src/graphviz-8.0.5/lib/common/htmlparse.y"
                           {popFont();}
-#line 1759 "htmlparse.c"
+#line 1745 "htmlparse.c"
     break;
 
   case 30: /* sup: T_sup  */
-#line 511 "/cygdrive/d/a/scallywag/graphviz/graphviz-2.50.0-1.x86_64/src/graphviz-2.50.0/lib/common/htmlparse.y"
+#line 497 "/cygdrive/d/a/scallywag/graphviz/graphviz-8.0.5-1.x86_64/src/graphviz-8.0.5/lib/common/htmlparse.y"
             {pushFont((yyvsp[0].font));}
-#line 1765 "htmlparse.c"
+#line 1751 "htmlparse.c"
     break;
 
   case 31: /* n_sup: T_n_sup  */
-#line 514 "/cygdrive/d/a/scallywag/graphviz/graphviz-2.50.0-1.x86_64/src/graphviz-2.50.0/lib/common/htmlparse.y"
+#line 500 "/cygdrive/d/a/scallywag/graphviz/graphviz-8.0.5-1.x86_64/src/graphviz-8.0.5/lib/common/htmlparse.y"
                 {popFont();}
-#line 1771 "htmlparse.c"
+#line 1757 "htmlparse.c"
     break;
 
   case 32: /* sub: T_sub  */
-#line 517 "/cygdrive/d/a/scallywag/graphviz/graphviz-2.50.0-1.x86_64/src/graphviz-2.50.0/lib/common/htmlparse.y"
+#line 503 "/cygdrive/d/a/scallywag/graphviz/graphviz-8.0.5-1.x86_64/src/graphviz-8.0.5/lib/common/htmlparse.y"
             {pushFont((yyvsp[0].font));}
-#line 1777 "htmlparse.c"
+#line 1763 "htmlparse.c"
     break;
 
   case 33: /* n_sub: T_n_sub  */
-#line 520 "/cygdrive/d/a/scallywag/graphviz/graphviz-2.50.0-1.x86_64/src/graphviz-2.50.0/lib/common/htmlparse.y"
+#line 506 "/cygdrive/d/a/scallywag/graphviz/graphviz-8.0.5-1.x86_64/src/graphviz-8.0.5/lib/common/htmlparse.y"
                 {popFont();}
-#line 1783 "htmlparse.c"
+#line 1769 "htmlparse.c"
     break;
 
   case 34: /* br: T_br T_end_br  */
-#line 523 "/cygdrive/d/a/scallywag/graphviz/graphviz-2.50.0-1.x86_64/src/graphviz-2.50.0/lib/common/htmlparse.y"
+#line 509 "/cygdrive/d/a/scallywag/graphviz/graphviz-8.0.5-1.x86_64/src/graphviz-8.0.5/lib/common/htmlparse.y"
                        { (yyval.i) = (yyvsp[-1].i); }
-#line 1789 "htmlparse.c"
+#line 1775 "htmlparse.c"
     break;
 
   case 35: /* br: T_BR  */
-#line 524 "/cygdrive/d/a/scallywag/graphviz/graphviz-2.50.0-1.x86_64/src/graphviz-2.50.0/lib/common/htmlparse.y"
+#line 510 "/cygdrive/d/a/scallywag/graphviz/graphviz-8.0.5-1.x86_64/src/graphviz-8.0.5/lib/common/htmlparse.y"
               { (yyval.i) = (yyvsp[0].i); }
-#line 1795 "htmlparse.c"
+#line 1781 "htmlparse.c"
     break;
 
   case 38: /* @1: %empty  */
-#line 531 "/cygdrive/d/a/scallywag/graphviz/graphviz-2.50.0-1.x86_64/src/graphviz-2.50.0/lib/common/htmlparse.y"
+#line 517 "/cygdrive/d/a/scallywag/graphviz/graphviz-8.0.5-1.x86_64/src/graphviz-8.0.5/lib/common/htmlparse.y"
                           { 
           if (nonSpace(agxbuse(HTMLstate.str))) {
             htmlerror ("Syntax error: non-space string used before <TABLE>");
@@ -1807,11 +1793,11 @@ yyreduce:
           (yyvsp[0].tbl)->font = HTMLstate.fontstack->cfont;
           (yyval.tbl) = (yyvsp[0].tbl);
         }
-#line 1811 "htmlparse.c"
+#line 1797 "htmlparse.c"
     break;
 
   case 39: /* table: opt_space T_table @1 rows T_end_table opt_space  */
-#line 542 "/cygdrive/d/a/scallywag/graphviz/graphviz-2.50.0-1.x86_64/src/graphviz-2.50.0/lib/common/htmlparse.y"
+#line 528 "/cygdrive/d/a/scallywag/graphviz/graphviz-8.0.5-1.x86_64/src/graphviz-8.0.5/lib/common/htmlparse.y"
                                    {
           if (nonSpace(agxbuse(HTMLstate.str))) {
             htmlerror ("Syntax error: non-space string used after </TABLE>");
@@ -1820,155 +1806,155 @@ yyreduce:
           (yyval.tbl) = HTMLstate.tblstack;
           HTMLstate.tblstack = HTMLstate.tblstack->u.p.prev;
         }
-#line 1824 "htmlparse.c"
+#line 1810 "htmlparse.c"
     break;
 
   case 40: /* fonttable: table  */
-#line 552 "/cygdrive/d/a/scallywag/graphviz/graphviz-2.50.0-1.x86_64/src/graphviz-2.50.0/lib/common/htmlparse.y"
+#line 538 "/cygdrive/d/a/scallywag/graphviz/graphviz-8.0.5-1.x86_64/src/graphviz-8.0.5/lib/common/htmlparse.y"
                   { (yyval.tbl) = (yyvsp[0].tbl); }
-#line 1830 "htmlparse.c"
+#line 1816 "htmlparse.c"
     break;
 
   case 41: /* fonttable: font table n_font  */
-#line 553 "/cygdrive/d/a/scallywag/graphviz/graphviz-2.50.0-1.x86_64/src/graphviz-2.50.0/lib/common/htmlparse.y"
+#line 539 "/cygdrive/d/a/scallywag/graphviz/graphviz-8.0.5-1.x86_64/src/graphviz-8.0.5/lib/common/htmlparse.y"
                               { (yyval.tbl)=(yyvsp[-1].tbl); }
-#line 1836 "htmlparse.c"
+#line 1822 "htmlparse.c"
     break;
 
   case 42: /* fonttable: italic table n_italic  */
-#line 554 "/cygdrive/d/a/scallywag/graphviz/graphviz-2.50.0-1.x86_64/src/graphviz-2.50.0/lib/common/htmlparse.y"
+#line 540 "/cygdrive/d/a/scallywag/graphviz/graphviz-8.0.5-1.x86_64/src/graphviz-8.0.5/lib/common/htmlparse.y"
                                   { (yyval.tbl)=(yyvsp[-1].tbl); }
-#line 1842 "htmlparse.c"
+#line 1828 "htmlparse.c"
     break;
 
   case 43: /* fonttable: underline table n_underline  */
-#line 555 "/cygdrive/d/a/scallywag/graphviz/graphviz-2.50.0-1.x86_64/src/graphviz-2.50.0/lib/common/htmlparse.y"
+#line 541 "/cygdrive/d/a/scallywag/graphviz/graphviz-8.0.5-1.x86_64/src/graphviz-8.0.5/lib/common/htmlparse.y"
                                         { (yyval.tbl)=(yyvsp[-1].tbl); }
-#line 1848 "htmlparse.c"
+#line 1834 "htmlparse.c"
     break;
 
   case 44: /* fonttable: overline table n_overline  */
-#line 556 "/cygdrive/d/a/scallywag/graphviz/graphviz-2.50.0-1.x86_64/src/graphviz-2.50.0/lib/common/htmlparse.y"
+#line 542 "/cygdrive/d/a/scallywag/graphviz/graphviz-8.0.5-1.x86_64/src/graphviz-8.0.5/lib/common/htmlparse.y"
                                       { (yyval.tbl)=(yyvsp[-1].tbl); }
-#line 1854 "htmlparse.c"
+#line 1840 "htmlparse.c"
     break;
 
   case 45: /* fonttable: bold table n_bold  */
-#line 557 "/cygdrive/d/a/scallywag/graphviz/graphviz-2.50.0-1.x86_64/src/graphviz-2.50.0/lib/common/htmlparse.y"
+#line 543 "/cygdrive/d/a/scallywag/graphviz/graphviz-8.0.5-1.x86_64/src/graphviz-8.0.5/lib/common/htmlparse.y"
                               { (yyval.tbl)=(yyvsp[-1].tbl); }
-#line 1860 "htmlparse.c"
+#line 1846 "htmlparse.c"
     break;
 
   case 48: /* rows: row  */
-#line 564 "/cygdrive/d/a/scallywag/graphviz/graphviz-2.50.0-1.x86_64/src/graphviz-2.50.0/lib/common/htmlparse.y"
+#line 550 "/cygdrive/d/a/scallywag/graphviz/graphviz-8.0.5-1.x86_64/src/graphviz-8.0.5/lib/common/htmlparse.y"
            { (yyval.p) = (yyvsp[0].p); }
-#line 1866 "htmlparse.c"
+#line 1852 "htmlparse.c"
     break;
 
   case 49: /* rows: rows row  */
-#line 565 "/cygdrive/d/a/scallywag/graphviz/graphviz-2.50.0-1.x86_64/src/graphviz-2.50.0/lib/common/htmlparse.y"
+#line 551 "/cygdrive/d/a/scallywag/graphviz/graphviz-8.0.5-1.x86_64/src/graphviz-8.0.5/lib/common/htmlparse.y"
                 { (yyval.p) = (yyvsp[0].p); }
-#line 1872 "htmlparse.c"
+#line 1858 "htmlparse.c"
     break;
 
   case 50: /* rows: rows HR row  */
-#line 566 "/cygdrive/d/a/scallywag/graphviz/graphviz-2.50.0-1.x86_64/src/graphviz-2.50.0/lib/common/htmlparse.y"
+#line 552 "/cygdrive/d/a/scallywag/graphviz/graphviz-8.0.5-1.x86_64/src/graphviz-8.0.5/lib/common/htmlparse.y"
                    { (yyvsp[-2].p)->ruled = 1; (yyval.p) = (yyvsp[0].p); }
-#line 1878 "htmlparse.c"
+#line 1864 "htmlparse.c"
     break;
 
   case 51: /* $@2: %empty  */
-#line 569 "/cygdrive/d/a/scallywag/graphviz/graphviz-2.50.0-1.x86_64/src/graphviz-2.50.0/lib/common/htmlparse.y"
+#line 555 "/cygdrive/d/a/scallywag/graphviz/graphviz-8.0.5-1.x86_64/src/graphviz-8.0.5/lib/common/htmlparse.y"
             { addRow (); }
-#line 1884 "htmlparse.c"
+#line 1870 "htmlparse.c"
     break;
 
   case 52: /* row: T_row $@2 cells T_end_row  */
-#line 569 "/cygdrive/d/a/scallywag/graphviz/graphviz-2.50.0-1.x86_64/src/graphviz-2.50.0/lib/common/htmlparse.y"
+#line 555 "/cygdrive/d/a/scallywag/graphviz/graphviz-8.0.5-1.x86_64/src/graphviz-8.0.5/lib/common/htmlparse.y"
                                            { (yyval.p) = lastRow(); }
-#line 1890 "htmlparse.c"
+#line 1876 "htmlparse.c"
     break;
 
   case 53: /* cells: cell  */
-#line 572 "/cygdrive/d/a/scallywag/graphviz/graphviz-2.50.0-1.x86_64/src/graphviz-2.50.0/lib/common/htmlparse.y"
+#line 558 "/cygdrive/d/a/scallywag/graphviz/graphviz-8.0.5-1.x86_64/src/graphviz-8.0.5/lib/common/htmlparse.y"
              { (yyval.cell) = (yyvsp[0].cell); }
-#line 1896 "htmlparse.c"
+#line 1882 "htmlparse.c"
     break;
 
   case 54: /* cells: cells cell  */
-#line 573 "/cygdrive/d/a/scallywag/graphviz/graphviz-2.50.0-1.x86_64/src/graphviz-2.50.0/lib/common/htmlparse.y"
+#line 559 "/cygdrive/d/a/scallywag/graphviz/graphviz-8.0.5-1.x86_64/src/graphviz-8.0.5/lib/common/htmlparse.y"
                    { (yyval.cell) = (yyvsp[0].cell); }
-#line 1902 "htmlparse.c"
+#line 1888 "htmlparse.c"
     break;
 
   case 55: /* cells: cells VR cell  */
-#line 574 "/cygdrive/d/a/scallywag/graphviz/graphviz-2.50.0-1.x86_64/src/graphviz-2.50.0/lib/common/htmlparse.y"
+#line 560 "/cygdrive/d/a/scallywag/graphviz/graphviz-8.0.5-1.x86_64/src/graphviz-8.0.5/lib/common/htmlparse.y"
                       { (yyvsp[-2].cell)->ruled |= HTML_VRULE; (yyval.cell) = (yyvsp[0].cell); }
-#line 1908 "htmlparse.c"
+#line 1894 "htmlparse.c"
     break;
 
   case 56: /* $@3: %empty  */
-#line 577 "/cygdrive/d/a/scallywag/graphviz/graphviz-2.50.0-1.x86_64/src/graphviz-2.50.0/lib/common/htmlparse.y"
+#line 563 "/cygdrive/d/a/scallywag/graphviz/graphviz-8.0.5-1.x86_64/src/graphviz-8.0.5/lib/common/htmlparse.y"
                         { setCell((yyvsp[-1].cell),(yyvsp[0].tbl),HTML_TBL); }
-#line 1914 "htmlparse.c"
+#line 1900 "htmlparse.c"
     break;
 
   case 57: /* cell: T_cell fonttable $@3 T_end_cell  */
-#line 577 "/cygdrive/d/a/scallywag/graphviz/graphviz-2.50.0-1.x86_64/src/graphviz-2.50.0/lib/common/htmlparse.y"
+#line 563 "/cygdrive/d/a/scallywag/graphviz/graphviz-8.0.5-1.x86_64/src/graphviz-8.0.5/lib/common/htmlparse.y"
                                                                 { (yyval.cell) = (yyvsp[-3].cell); }
-#line 1920 "htmlparse.c"
+#line 1906 "htmlparse.c"
     break;
 
   case 58: /* $@4: %empty  */
-#line 578 "/cygdrive/d/a/scallywag/graphviz/graphviz-2.50.0-1.x86_64/src/graphviz-2.50.0/lib/common/htmlparse.y"
+#line 564 "/cygdrive/d/a/scallywag/graphviz/graphviz-8.0.5-1.x86_64/src/graphviz-8.0.5/lib/common/htmlparse.y"
                        { setCell((yyvsp[-1].cell),(yyvsp[0].txt),HTML_TEXT); }
-#line 1926 "htmlparse.c"
+#line 1912 "htmlparse.c"
     break;
 
   case 59: /* cell: T_cell fonttext $@4 T_end_cell  */
-#line 578 "/cygdrive/d/a/scallywag/graphviz/graphviz-2.50.0-1.x86_64/src/graphviz-2.50.0/lib/common/htmlparse.y"
+#line 564 "/cygdrive/d/a/scallywag/graphviz/graphviz-8.0.5-1.x86_64/src/graphviz-8.0.5/lib/common/htmlparse.y"
                                                                 { (yyval.cell) = (yyvsp[-3].cell); }
-#line 1932 "htmlparse.c"
+#line 1918 "htmlparse.c"
     break;
 
   case 60: /* $@5: %empty  */
-#line 579 "/cygdrive/d/a/scallywag/graphviz/graphviz-2.50.0-1.x86_64/src/graphviz-2.50.0/lib/common/htmlparse.y"
+#line 565 "/cygdrive/d/a/scallywag/graphviz/graphviz-8.0.5-1.x86_64/src/graphviz-8.0.5/lib/common/htmlparse.y"
                     { setCell((yyvsp[-1].cell),(yyvsp[0].img),HTML_IMAGE); }
-#line 1938 "htmlparse.c"
+#line 1924 "htmlparse.c"
     break;
 
   case 61: /* cell: T_cell image $@5 T_end_cell  */
-#line 579 "/cygdrive/d/a/scallywag/graphviz/graphviz-2.50.0-1.x86_64/src/graphviz-2.50.0/lib/common/htmlparse.y"
+#line 565 "/cygdrive/d/a/scallywag/graphviz/graphviz-8.0.5-1.x86_64/src/graphviz-8.0.5/lib/common/htmlparse.y"
                                                               { (yyval.cell) = (yyvsp[-3].cell); }
-#line 1944 "htmlparse.c"
+#line 1930 "htmlparse.c"
     break;
 
   case 62: /* $@6: %empty  */
-#line 580 "/cygdrive/d/a/scallywag/graphviz/graphviz-2.50.0-1.x86_64/src/graphviz-2.50.0/lib/common/htmlparse.y"
+#line 566 "/cygdrive/d/a/scallywag/graphviz/graphviz-8.0.5-1.x86_64/src/graphviz-8.0.5/lib/common/htmlparse.y"
               { setCell((yyvsp[0].cell),mkText(),HTML_TEXT); }
-#line 1950 "htmlparse.c"
+#line 1936 "htmlparse.c"
     break;
 
   case 63: /* cell: T_cell $@6 T_end_cell  */
-#line 580 "/cygdrive/d/a/scallywag/graphviz/graphviz-2.50.0-1.x86_64/src/graphviz-2.50.0/lib/common/htmlparse.y"
+#line 566 "/cygdrive/d/a/scallywag/graphviz/graphviz-8.0.5-1.x86_64/src/graphviz-8.0.5/lib/common/htmlparse.y"
                                                              { (yyval.cell) = (yyvsp[-2].cell); }
-#line 1956 "htmlparse.c"
+#line 1942 "htmlparse.c"
     break;
 
   case 64: /* image: T_img T_end_img  */
-#line 583 "/cygdrive/d/a/scallywag/graphviz/graphviz-2.50.0-1.x86_64/src/graphviz-2.50.0/lib/common/htmlparse.y"
+#line 569 "/cygdrive/d/a/scallywag/graphviz/graphviz-8.0.5-1.x86_64/src/graphviz-8.0.5/lib/common/htmlparse.y"
                          { (yyval.img) = (yyvsp[-1].img); }
-#line 1962 "htmlparse.c"
+#line 1948 "htmlparse.c"
     break;
 
   case 65: /* image: T_IMG  */
-#line 584 "/cygdrive/d/a/scallywag/graphviz/graphviz-2.50.0-1.x86_64/src/graphviz-2.50.0/lib/common/htmlparse.y"
+#line 570 "/cygdrive/d/a/scallywag/graphviz/graphviz-8.0.5-1.x86_64/src/graphviz-8.0.5/lib/common/htmlparse.y"
                { (yyval.img) = (yyvsp[0].img); }
-#line 1968 "htmlparse.c"
+#line 1954 "htmlparse.c"
     break;
 
 
-#line 1972 "htmlparse.c"
+#line 1958 "htmlparse.c"
 
       default: break;
     }
@@ -2161,18 +2147,18 @@ yyreturnlab:
   return yyresult;
 }
 
-#line 596 "/cygdrive/d/a/scallywag/graphviz/graphviz-2.50.0-1.x86_64/src/graphviz-2.50.0/lib/common/htmlparse.y"
+#line 582 "/cygdrive/d/a/scallywag/graphviz/graphviz-8.0.5-1.x86_64/src/graphviz-8.0.5/lib/common/htmlparse.y"
 
 
 /* parseHTML:
  * Return parsed label or NULL if failure.
- * Set warn to 0 on success; 1 for warning message; 2 if no expat.
+ * Set warn to 0 on success; 1 for warning message; 2 if no expat; 3 for error
+ * message.
  */
 htmllabel_t*
 parseHTML (char* txt, int* warn, htmlenv_t *env)
 {
-  unsigned char buf[SMALLBUF];
-  agxbuf        str;
+  agxbuf        str = {0};
   htmllabel_t*  l;
   sfont_t       dfltf;
 
@@ -2185,7 +2171,6 @@ parseHTML (char* txt, int* warn, htmlenv_t *env)
   HTMLstate.fitemList = dtopen(&fstrDisc, Dtqueue);
   HTMLstate.fspanList = dtopen(&fspanDisc, Dtqueue);
 
-  agxbinit (&str, SMALLBUF, buf);
   HTMLstate.str = &str;
   
   if (initHTMLlexer (txt, &str, env)) {/* failed: no libexpat - give up */
