@@ -32,12 +32,21 @@
 #include <common/render.h>
 #include <common/htmltable.h>
 #include <cgraph/agxbuf.h>
+#include <cgraph/prisize_t.h>
 #include <common/pointset.h>
 #include <common/intset.h>
 #include <cdt/cdt.h>
+#include <cgraph/alloc.h>
+#include <cgraph/exit.h>
+#include <cgraph/prisize_t.h>
 #include <cgraph/strcasecmp.h>
+#include <cgraph/unreachable.h>
+#include <float.h>
+#include <inttypes.h>
+#include <math.h>
 #include <stddef.h>
 #include <stdbool.h>
+#include <stdio.h>
 
 #define DEFAULT_BORDER    1
 #define DEFAULT_CELLPADDING  2
@@ -48,7 +57,7 @@ typedef struct {
     char *tooltip;
     char *target;
     char *id;
-    boolean explicit_tooltip;
+    bool explicit_tooltip;
     point LL;
     point UR;
 } htmlmap_data_t;
@@ -103,10 +112,9 @@ static void popFontInfo(htmlenv_t * env, textfont_t * savp)
 }
 
 static void
-emit_htextspans(GVJ_t * job, int nspans, htextspan_t * spans, pointf p,
+emit_htextspans(GVJ_t *job, size_t nspans, htextspan_t *spans, pointf p,
 		double halfwidth_x, textfont_t finfo, boxf b, int simple)
 {
-    int i, j;
     double center_x, left_x, right_x;
     textspan_t tl;
     textfont_t tf;
@@ -123,7 +131,7 @@ emit_htextspans(GVJ_t * job, int nspans, htextspan_t * spans, pointf p,
     p_.y = p.y + (b.UR.y - b.LL.y) / 2.0;
 
     gvrender_begin_label(job, LABEL_HTML);
-    for (i = 0; i < nspans; i++) {
+    for (size_t i = 0; i < nspans; i++) {
 	/* set p.x to leftmost point where the line of text begins */
 	switch (spans[i].just) {
 	case 'l':
@@ -140,8 +148,8 @@ emit_htextspans(GVJ_t * job, int nspans, htextspan_t * spans, pointf p,
 	p_.y -= spans[i].lfsize;	/* move to current base line */
 
 	ti = spans[i].items;
-	for (j = 0; j < spans[i].nitems; j++) {
-	    if (ti->font && (ti->font->size > 0))
+	for (size_t j = 0; j < spans[i].nitems; j++) {
+	    if (ti->font && ti->font->size > 0)
 		tf.size = ti->font->size;
 	    else
 		tf.size = finfo.size;
@@ -192,9 +200,9 @@ static void emit_html_txt(GVJ_t * job, htmltxt_t * tp, htmlenv_t * env)
     if (tp->nspans < 1)
 	return;
 
-    halfwidth_x = ((double) (tp->box.UR.x - tp->box.LL.x)) / 2.0;
-    p.x = env->pos.x + ((double) (tp->box.UR.x + tp->box.LL.x)) / 2.0;
-    p.y = env->pos.y + ((double) (tp->box.UR.y + tp->box.LL.y)) / 2.0;
+    halfwidth_x = (tp->box.UR.x - tp->box.LL.x) / 2.0;
+    p.x = env->pos.x + (tp->box.UR.x + tp->box.LL.x) / 2.0;
+    p.y = env->pos.y + (tp->box.UR.y + tp->box.LL.y) / 2.0;
 
     emit_htextspans(job, tp->nspans, tp->spans, p, halfwidth_x, env->finfo,
 		    tp->box, tp->simple);
@@ -221,7 +229,7 @@ static pointf *mkPts(pointf * AF, boxf b, int border)
     AF[0] = b.LL;
     AF[2] = b.UR;
     if (border > 1) {
-	double delta = ((double) border) / 2.0;
+	double delta = (double)border / 2.0;
 	AF[0].x += delta;
 	AF[0].y += delta;
 	AF[2].x -= delta;
@@ -245,11 +253,11 @@ static void doBorder(GVJ_t * job, htmldata_t * dp, boxf b)
 {
     pointf AF[7];
     char *sptr[2];
-    char *color = (dp->pencolor ? dp->pencolor : DEFAULT_COLOR);
+    char *color = dp->pencolor ? dp->pencolor : DEFAULT_COLOR;
     unsigned short sides;
 
     gvrender_set_pencolor(job, color);
-    if ((dp->style & (DASHED | DOTTED))) {
+    if (dp->style & (DASHED | DOTTED)) {
 	sptr[0] = sptr[1] = NULL;
 	if (dp->style & DASHED)
 	    sptr[0] = "dashed";
@@ -320,7 +328,7 @@ static void doBorder(GVJ_t * job, htmldata_t * dp, boxf b)
 	}
     } else {
 	if (dp->border > 1) {
-	    double delta = ((double) dp->border) / 2.0;
+	    double delta = (double)dp->border / 2.0;
 	    b.LL.x += delta;
 	    b.LL.y += delta;
 	    b.UR.x -= delta;
@@ -376,31 +384,26 @@ initAnchor(GVJ_t * job, htmlenv_t * env, htmldata_t * data, boxf b,
     int changed;
     char *id;
     static int anchorId;
-    int internalId = 0;
-    agxbuf xb;
-    unsigned char buf[SMALLBUF];
+    agxbuf xb = {0};
 
     save->url = obj->url;
     save->tooltip = obj->tooltip;
     save->target = obj->target;
     save->id = obj->id;
-    save->explicit_tooltip = obj->explicit_tooltip;
+    save->explicit_tooltip = obj->explicit_tooltip != 0;
     id = data->id;
     if (!id || !*id) {		/* no external id, so use the internal one */
-	agxbinit(&xb, SMALLBUF, buf);
 	if (!env->objid) {
-	    env->objid = strdup(getObjId(job, obj->u.n, &xb));
+	    env->objid = gv_strdup(getObjId(job, obj->u.n, &xb));
 	    env->objid_set = true;
 	}
 	agxbprint(&xb, "%s_%d", env->objid, anchorId++);
 	id = agxbuse(&xb);
-	internalId = 1;
     }
     changed =
 	initMapData(job, NULL, data->href, data->title, data->target, id,
 		    obj->u.g);
-    if (internalId)
-	agxbfree(&xb);
+    agxbfree(&xb);
 
     if (changed) {
 	if (obj->url || obj->explicit_tooltip) {
@@ -452,7 +455,7 @@ emit_html_rules(GVJ_t * job, htmlcell_t * cp, htmlenv_t * env, char *color, html
 {
     pointf rule_pt;
     double rule_length;
-    unsigned char base;
+    double base;
     boxf pts = cp->data.box;
     pointf pos = env->pos;
 
@@ -468,12 +471,12 @@ emit_html_rules(GVJ_t * job, htmlcell_t * cp, htmlenv_t * env, char *color, html
     pts.UR.y += pos.y;
 
     //Determine vertical line coordinate and length
-    if ((cp->ruled & HTML_VRULE) && (cp->col + cp->cspan < cp->parent->cc)) {
+    if ((cp->ruled & HTML_VRULE) && cp->col + cp->colspan < cp->parent->column_count) {
 	if (cp->row == 0) {	// first row
 	    // extend to center of table border and add half cell spacing
 	    base = cp->parent->data.border + cp->parent->data.space / 2;
 	    rule_pt.y = pts.LL.y - cp->parent->data.space / 2;
-	} else if (cp->row + cp->rspan == cp->parent->rc) {	// bottom row
+	} else if (cp->row + cp->rowspan == cp->parent->row_count) { // bottom row
 	    // extend to center of table border and add half cell spacing
 	    base = cp->parent->data.border + cp->parent->data.space / 2;
 	    rule_pt.y = pts.LL.y - cp->parent->data.space / 2 - base;
@@ -486,18 +489,18 @@ emit_html_rules(GVJ_t * job, htmlcell_t * cp, htmlenv_t * env, char *color, html
 	doSide(job, rule_pt, 0, rule_length);
     }
     //Determine the horizontal coordinate and length
-    if ((cp->ruled & HTML_HRULE) && (cp->row + cp->rspan < cp->parent->rc)) {
+    if ((cp->ruled & HTML_HRULE) && cp->row + cp->rowspan < cp->parent->row_count) {
 	if (cp->col == 0) {	// first column 
 	    // extend to center of table border and add half cell spacing
 	    base = cp->parent->data.border + cp->parent->data.space / 2;
 	    rule_pt.x = pts.LL.x - base - cp->parent->data.space / 2;
-	    if (cp->col + cp->cspan == cp->parent->cc)	// also last column
+	    if (cp->col + cp->colspan == cp->parent->column_count)	// also last column
 		base *= 2;
 	    /* incomplete row of cells; extend line to end */
-	    else if (nextc && (nextc->row != cp->row)) {
-		base += (cp->parent->data.box.UR.x + pos.x) - (pts.UR.x + cp->parent->data.space / 2);
+	    else if (nextc && nextc->row != cp->row) {
+		base += cp->parent->data.box.UR.x + pos.x - (pts.UR.x + cp->parent->data.space / 2);
 	    }
-	} else if (cp->col + cp->cspan == cp->parent->cc) {	// last column
+	} else if (cp->col + cp->colspan == cp->parent->column_count) {	// last column
 	    // extend to center of table border and add half cell spacing
 	    base = cp->parent->data.border + cp->parent->data.space / 2;
 	    rule_pt.x = pts.LL.x - cp->parent->data.space / 2;
@@ -505,8 +508,8 @@ emit_html_rules(GVJ_t * job, htmlcell_t * cp, htmlenv_t * env, char *color, html
 	    base = 0;
 	    rule_pt.x = pts.LL.x - cp->parent->data.space / 2;
 	    /* incomplete row of cells; extend line to end */
-	    if (nextc && (nextc->row != cp->row)) {
-		base += (cp->parent->data.box.UR.x + pos.x) - (pts.UR.x + cp->parent->data.space / 2);
+	    if (nextc && nextc->row != cp->row) {
+		base += cp->parent->data.box.UR.x + pos.x - (pts.UR.x + cp->parent->data.space / 2);
 	    }
 	}
 	rule_pt.y = pts.LL.y - cp->parent->data.space / 2;
@@ -619,7 +622,7 @@ static void emit_html_img(GVJ_t * job, htmlimg_t * cp, htmlenv_t * env)
 	scale = env->imgscale;
     assert(cp->src);
     assert(cp->src[0]);
-    gvrender_usershape(job, cp->src, A, 4, TRUE, scale, "mc");
+    gvrender_usershape(job, cp->src, A, 4, true, scale, "mc");
 }
 
 static void emit_html_cell(GVJ_t * job, htmlcell_t * cp, htmlenv_t * env)
@@ -735,6 +738,8 @@ heightOfLbl (htmllabel_t * lp)
     case HTML_TEXT:
 	sz  = lp->u.txt->box.UR.y - lp->u.txt->box.LL.y;
 	break;
+    default:
+	UNREACHABLE();
     }
     return sz;
 }
@@ -767,7 +772,7 @@ void emit_html_label(GVJ_t * job, htmllabel_t * lp, textlabel_t * tp)
     env.imgscale = agget(job->obj->u.n, "imagescale");
     env.objid = job->obj->id;
     env.objid_set = false;
-    if ((env.imgscale == NULL) || (env.imgscale[0] == '\0'))
+    if (env.imgscale == NULL || env.imgscale[0] == '\0')
 	env.imgscale = "false";
     if (lp->kind == HTML_TBL) {
 	htmltbl_t *tbl = lp->u.tbl;
@@ -803,15 +808,14 @@ void free_html_text(htmltxt_t * t)
 {
     htextspan_t *tl;
     textspan_t *ti;
-    int i, j;
 
     if (!t)
 	return;
 
     tl = t->spans;
-    for (i = 0; i < t->nspans; i++) {
+    for (size_t i = 0; i < t->nspans; i++) {
 	ti = tl->items;
-	for (j = 0; j < tl->nitems; j++) {
+	for (size_t j = 0; j < tl->nitems; j++) {
 	    free(ti->str);
 	    if (ti->layout && ti->free_layout)
 		ti->free_layout(ti->layout);
@@ -845,7 +849,7 @@ static void free_html_tbl(htmltbl_t * tbl)
 {
     htmlcell_t **cells;
 
-    if (tbl->rc == -1) {
+    if (tbl->row_count == SIZE_MAX) { // raw, parsed table
 	dtclose(tbl->u.p.rows);
     } else {
 	cells = tbl->u.n.cells;
@@ -880,7 +884,7 @@ static htmldata_t *portToCell(htmlcell_t * cp, char *id)
 {
     htmldata_t *rv;
 
-    if (cp->data.port && (strcasecmp(cp->data.port, id) == 0))
+    if (cp->data.port && strcasecmp(cp->data.port, id) == 0)
 	rv = &cp->data;
     else if (cp->child.kind == HTML_TBL)
 	rv = portToTbl(cp->child.u.tbl, id);
@@ -900,7 +904,7 @@ static htmldata_t *portToTbl(htmltbl_t * tp, char *id)
     htmlcell_t **cells;
     htmlcell_t *cp;
 
-    if (tp->data.port && (strcasecmp(tp->data.port, id) == 0))
+    if (tp->data.port && strcasecmp(tp->data.port, id) == 0)
 	rv = &tp->data;
     else {
 	rv = NULL;
@@ -920,8 +924,7 @@ static htmldata_t *portToTbl(htmltbl_t * tp, char *id)
  * If successful, return pointer to port's box.
  * Else return NULL.
  */
-boxf *html_port(node_t * n, char *pname, int *sides)
-{
+boxf *html_port(node_t *n, char *pname, unsigned char *sides){
     htmldata_t *tp;
     htmllabel_t *lbl = ND_label(n)->u.html;
     boxf *rv = NULL;
@@ -938,64 +941,6 @@ boxf *html_port(node_t * n, char *pname, int *sides)
 
 }
 
-/* html_path:
- * Return a box in a table containing the given endpoint.
- * If the top flow is text (no internal structure), return 
- * the box of the flow
- * Else return the box of the subtable containing the point.
- * Because of spacing, the point might not be in any subtable.
- * In that case, return the top flow's box.
- * Note that box[0] must contain the edge point. Additional boxes
- * move out to the boundary.
- *
- * At present, unimplemented, since the label may be inside a
- * non-box node and we need to figure out what this means.
- */
-int html_path(node_t * n, port * p, int side, boxf * rv, int *k)
-{
-#ifdef UNIMPL
-    point p;
-    tbl_t *info;
-    tbl_t *t;
-    boxf b;
-    int i;
-
-    info = (tbl_t *) ND_shape_info(n);
-    assert(info->tbls);
-    info = info->tbls[0];	/* top-level flow */
-    assert(IS_FLOW(info));
-
-    b = info->box;
-    if (info->tbl) {
-	info = info->tbl;
-	if (pt == 1)
-	    p = ED_tail_port(e).p;
-	else
-	    p = ED_head_port(e).p;
-	p = flip_pt(p, GD_rankdir(n->graph));	/* move p to node's coordinate system */
-	for (i = 0; (t = info->tbls[i]) != 0; i++)
-	    if (INSIDE(p, t->box)) {
-		b = t->box;
-		break;
-	    }
-    }
-
-    /* move box into layout coordinate system */
-    if (GD_flip(n->graph))
-	b = flip_trans_box(b, ND_coord_i(n));
-    else
-	b = move_box(b, ND_coord_i(n));
-
-    *k = 1;
-    *rv = b;
-    if (pt == 1)
-	return BOTTOM;
-    else
-	return TOP;
-#endif
-    return 0;
-}
-
 static int size_html_txt(GVC_t *gvc, htmltxt_t * ftxt, htmlenv_t * env)
 {
     double xsize = 0.0;		/* width of text block */
@@ -1004,23 +949,22 @@ static int size_html_txt(GVC_t *gvc, htmltxt_t * ftxt, htmlenv_t * env)
     double mxfsize = 0.0;	/* max. font size for the current line */
     double curbline = 0.0;	/* dist. of current base line from top */
     pointf sz;
-    int i, j;
     double width;
     textspan_t lp;
     textfont_t tf = {NULL,NULL,NULL,0.0,0,0};
-    double maxoffset, mxysize;
-    int simple = 1;              /* one item per span, same font size/face, no flags */
+    double maxoffset, mxysize = 0.0;
+    bool simple = true; // one item per span, same font size/face, no flags
     double prev_fsize = -1;
     char* prev_fname = NULL;
 
-    for (i = 0; i < ftxt->nspans; i++) {
+    for (size_t i = 0; i < ftxt->nspans; i++) {
 	if (ftxt->spans[i].nitems > 1) {
-	    simple = 0;
+	    simple = false;
 	    break;
 	}
 	if (ftxt->spans[i].items[0].font) {
 	    if (ftxt->spans[i].items[0].font->flags) {
-		simple = 0;
+		simple = false;
 		break;
 	    }
 	    if (ftxt->spans[i].items[0].font->size > 0)
@@ -1036,25 +980,25 @@ static int size_html_txt(GVC_t *gvc, htmltxt_t * ftxt, htmlenv_t * env)
 	    tf.size = env->finfo.size;
 	    tf.name = env->finfo.name;
 	}
-	if (prev_fsize == -1)
+	if (i == 0)
 	    prev_fsize = tf.size;
 	else if (tf.size != prev_fsize) {
-	    simple = 0;
+	    simple = false;
 	    break;
 	}
 	if (prev_fname == NULL)
 	    prev_fname = tf.name;
 	else if (strcmp(tf.name,prev_fname)) {
-	    simple = 0;
+	    simple = false;
 	    break;
 	}
     }
     ftxt->simple = simple;
 
-    for (i = 0; i < ftxt->nspans; i++) {
+    for (size_t i = 0; i < ftxt->nspans; i++) {
 	width = 0;
 	mxysize = maxoffset = mxfsize = 0;
-	for (j = 0; j < ftxt->spans[i].nitems; j++) {
+	for (size_t j = 0; j < ftxt->spans[i].nitems; j++) {
 	    lp.str =
 		strdup_and_subst_obj(ftxt->spans[i].items[j].str,
 				     env->obj);
@@ -1146,13 +1090,13 @@ static int size_html_img(htmlimg_t * img, htmlenv_t * env)
 
     b.LL.x = b.LL.y = 0;
     b.UR = gvusershape_size(env->g, img->src);
-    if ((b.UR.x == -1) && (b.UR.y == -1)) {
+    if (b.UR.x == -1 && b.UR.y == -1) {
 	rv = 1;
 	b.UR.x = b.UR.y = 0;
 	agerr(AGERR, "No or improper image file=\"%s\"\n", img->src);
     } else {
 	rv = 0;
-	GD_has_images(env->g) = TRUE;
+	GD_has_images(env->g) = true;
     }
 
     B2BF(b, img->box);
@@ -1177,8 +1121,8 @@ size_html_cell(graph_t * g, htmlcell_t * cp, htmltbl_t * parent,
 	    cp->data.pad = DEFAULT_CELLPADDING;
     }
     if (!(cp->data.flags & BORDER_SET)) {
-	if (parent->cb >= 0)
-	    cp->data.border = parent->cb;
+	if (parent->cellborder >= 0)
+	    cp->data.border = (unsigned char)parent->cellborder;
 	else if (parent->data.flags & BORDER_SET)
 	    cp->data.border = parent->data.border;
 	else
@@ -1202,7 +1146,7 @@ size_html_cell(graph_t * g, htmlcell_t * cp, htmltbl_t * parent,
 
     if (cp->data.flags & FIXED_FLAG) {
 	if (cp->data.width && cp->data.height) {
-	    if (((cp->data.width < sz.x) || (cp->data.height < sz.y)) && (cp->child.kind != HTML_IMAGE)) {
+	    if ((cp->data.width < sz.x || cp->data.height < sz.y) && cp->child.kind != HTML_IMAGE) {
 		agerr(AGWARN, "cell size too small for content\n");
 		rv = 1;
 	    }
@@ -1224,7 +1168,7 @@ static int findCol(PointSet * ps, int row, int col, htmlcell_t * cellp)
     int notFound = 1;
     int lastc;
     int i, j, c;
-    int end = cellp->cspan - 1;
+    int end = cellp->colspan - 1;
 
     while (notFound) {
 	lastc = col + end;
@@ -1237,8 +1181,8 @@ static int findCol(PointSet * ps, int row, int col, htmlcell_t * cellp)
 	else
 	    notFound = 0;
     }
-    for (j = col; j < col + cellp->cspan; j++) {
-	for (i = row; i < row + cellp->rspan; i++) {
+    for (j = col; j < col + cellp->colspan; j++) {
+	for (i = row; i < row + cellp->rowspan; i++) {
 	    addPS(ps, j, i);
 	}
     }
@@ -1256,31 +1200,30 @@ static int processTbl(graph_t * g, htmltbl_t * tbl, htmlenv_t * env)
     pitem *rp;
     pitem *cp;
     Dt_t *cdict;
-    int r, c;
     htmlcell_t *cellp;
     htmlcell_t **cells;
     Dt_t *rows = tbl->u.p.rows;
     int rv = 0;
-    int n_rows = 0;
-    int n_cols = 0;
+    size_t n_rows = 0;
+    size_t n_cols = 0;
     PointSet *ps = newPS();
     Dt_t *is = openIntSet();
 
     rp = (pitem *) dtflatten(rows);
     size_t cnt = 0;
-    r = 0;
+    uint16_t r = 0;
     while (rp) {
 	cdict = rp->u.rp;
 	cp = (pitem *) dtflatten(cdict);
 	while (cp) {
 	    cellp = cp->u.cp;
 	    cnt++;
-	    cp = (pitem *) dtlink(cdict, (Dtlink_t *) cp);
+	    cp = (pitem *)dtlink(cdict, cp);
 	}
 	if (rp->ruled) {
 	    addIntSet(is, r + 1);
 	}
-	rp = (pitem *) dtlink(rows, (Dtlink_t *) rp);
+	rp = (pitem *)dtlink(rows, rp);
 	r++;
     }
 
@@ -1290,7 +1233,7 @@ static int processTbl(graph_t * g, htmltbl_t * tbl, htmlenv_t * env)
     while (rp) {
 	cdict = rp->u.rp;
 	cp = (pitem *) dtflatten(cdict);
-	c = 0;
+	uint16_t c = 0;
 	while (cp) {
 	    cellp = cp->u.cp;
 	    *cells++ = cellp;
@@ -1298,18 +1241,18 @@ static int processTbl(graph_t * g, htmltbl_t * tbl, htmlenv_t * env)
 	    c = findCol(ps, r, c, cellp);
 	    cellp->row = r;
 	    cellp->col = c;
-	    c += cellp->cspan;
+	    c += cellp->colspan;
 	    n_cols = MAX(c, n_cols);
-	    n_rows = MAX(r + cellp->rspan, n_rows);
-	    if (inIntSet(is, r + cellp->rspan))
+	    n_rows = MAX(r + cellp->rowspan, n_rows);
+	    if (inIntSet(is, r + cellp->rowspan))
 		cellp->ruled |= HTML_HRULE;
-	    cp = (pitem *) dtlink(cdict, (Dtlink_t *) cp);
+	    cp = (pitem *)dtlink(cdict, cp);
 	}
-	rp = (pitem *) dtlink(rows, (Dtlink_t *) rp);
+	rp = (pitem *)dtlink(rows, rp);
 	r++;
     }
-    tbl->rc = n_rows;
-    tbl->cc = n_cols;
+    tbl->row_count = n_rows;
+    tbl->column_count = n_cols;
     dtclose(rows);
     dtclose(is);
     freePS(ps);
@@ -1332,55 +1275,36 @@ static void sizeLinearArray(htmltbl_t * tbl)
 {
     htmlcell_t *cp;
     htmlcell_t **cells;
-    int wd, ht, i, x, y;
+    int i;
 
-    tbl->heights = N_NEW(tbl->rc + 1, int);
-    tbl->widths = N_NEW(tbl->cc + 1, int);
+    tbl->heights = N_NEW(tbl->row_count + 1, double);
+    tbl->widths = N_NEW(tbl->column_count + 1, double);
 
     for (cells = tbl->u.n.cells; *cells; cells++) {
 	cp = *cells;
-	if (cp->rspan == 1)
+	double ht;
+	if (cp->rowspan == 1)
 	    ht = cp->data.box.UR.y;
 	else {
-	    ht = SPLIT(cp->data.box.UR.y, cp->rspan, tbl->data.space);
-	    ht = MAX(ht, 1);
+	    ht = SPLIT(cp->data.box.UR.y, cp->rowspan, tbl->data.space);
+	    ht = fmax(ht, 1);
 	}
-	if (cp->cspan == 1)
+	double wd;
+	if (cp->colspan == 1)
 	    wd = cp->data.box.UR.x;
 	else {
-	    wd = SPLIT(cp->data.box.UR.x, cp->cspan, tbl->data.space);
-	    wd = MAX(wd, 1);
+	    wd = SPLIT(cp->data.box.UR.x, cp->colspan, tbl->data.space);
+	    wd = fmax(wd, 1);
 	}
-	for (i = cp->row; i < cp->row + cp->rspan; i++) {
-	    y = tbl->heights[i];
-	    tbl->heights[i] = MAX(y, ht);
+	for (i = cp->row; i < cp->row + cp->rowspan; i++) {
+	    tbl->heights[i] = fmax(tbl->heights[i], ht);
 	}
-	for (i = cp->col; i < cp->col + cp->cspan; i++) {
-	    x = tbl->widths[i];
-	    tbl->widths[i] = MAX(x, wd);
+	for (i = cp->col; i < cp->col + cp->colspan; i++) {
+	    tbl->widths[i] = fmax(tbl->widths[i], wd);
 	}
     }
 }
 
-static char *nnames[] = {
-    "0", "1", "2", "3", "4", "5", "6", "7", "8", "9",
-    "10", "11", "12", "13", "14", "15", "16", "17", "18", "19", "20",
-};
-
-/* nToName:
- * Convert int to its decimal string representation.
- */
-static char *nToName(int c)
-{
-    static char name[100];
-
-    if (c < sizeof(nnames) / sizeof(char *))
-	return nnames[c];
-
-    snprintf(name, sizeof(name), "%d", c);
-    return name;
-}
-
 /* closeGraphs:
  * Clean up graphs made for setting column and row widths.
  */
@@ -1409,7 +1333,7 @@ static void checkChain(graph_t * g)
     for (h = ND_next(t); h; h = ND_next(h)) {
 	if (!agfindedge(g, t, h)) {
 	    e = agedge(g, t, h, NULL, 1);
-	    agbindrec(e, "Agedgeinfo_t", sizeof(Agedgeinfo_t), TRUE);
+	    agbindrec(e, "Agedgeinfo_t", sizeof(Agedgeinfo_t), true);
 	    ED_minlen(e) = 0;
 	    elist_append(e, ND_out(t));
 	    elist_append(e, ND_in(h));
@@ -1432,7 +1356,7 @@ checkEdge (graph_t* g, node_t* t, node_t* h, int sz)
 	ED_minlen(e) = MAX(ED_minlen(e), sz);
     else {
 	e = agedge(g, t, h, NULL, 1);
-	agbindrec(e, "Agedgeinfo_t", sizeof(Agedgeinfo_t), TRUE);
+	agbindrec(e, "Agedgeinfo_t", sizeof(Agedgeinfo_t), true);
 	ED_minlen(e) = sz;
 	elist_append(e, ND_out(t));
 	elist_append(e, ND_in(h));
@@ -1441,10 +1365,10 @@ checkEdge (graph_t* g, node_t* t, node_t* h, int sz)
 
 /* makeGraphs:
  * Generate dags modeling the row and column constraints.
- * If the table has cc columns, we create the graph
- *  0 -> 1 -> 2 -> ... -> cc
- * and if a cell starts in column c with span cspan, with
- * width w, we add the edge c -> c+cspan [minlen = w].
+ * If the table has column_count columns, we create the graph
+ *  0 -> 1 -> 2 -> ... -> column_count
+ * and if a cell starts in column c with span colspan, with
+ * width w, we add the edge c -> c+colspan [minlen = w].
  * Ditto for rows.
  *
  */
@@ -1455,14 +1379,15 @@ static void makeGraphs(htmltbl_t * tbl, graph_t * rowg, graph_t * colg)
     node_t *t;
     node_t *lastn;
     node_t *h;
-    int i;
+    agxbuf value_buffer = {0};
 
     lastn = NULL;
-    for (i = 0; i <= tbl->cc; i++) {
-	t = agnode(colg, nToName(i), 1);
-	agbindrec(t, "Agnodeinfo_t", sizeof(Agnodeinfo_t), TRUE);
-	alloc_elist(tbl->rc, ND_in(t));
-	alloc_elist(tbl->rc, ND_out(t));
+    for (size_t i = 0; i <= tbl->column_count; i++) {
+	agxbprint(&value_buffer, "%" PRISIZE_T, i);
+	t = agnode(colg, agxbuse(&value_buffer), 1);
+	agbindrec(t, "Agnodeinfo_t", sizeof(Agnodeinfo_t), true);
+	alloc_elist(tbl->row_count, ND_in(t));
+	alloc_elist(tbl->row_count, ND_out(t));
 	if (lastn) {
 	    ND_next(lastn) = t;
 	    lastn = t;
@@ -1471,11 +1396,12 @@ static void makeGraphs(htmltbl_t * tbl, graph_t * rowg, graph_t * colg)
 	}
     }
     lastn = NULL;
-    for (i = 0; i <= tbl->rc; i++) {
-	t = agnode(rowg, nToName(i), 1);
-	agbindrec(t, "Agnodeinfo_t", sizeof(Agnodeinfo_t), TRUE);
-	alloc_elist(tbl->cc, ND_in(t));
-	alloc_elist(tbl->cc, ND_out(t));
+    for (size_t i = 0; i <= tbl->row_count; i++) {
+	agxbprint(&value_buffer, "%" PRISIZE_T, i);
+	t = agnode(rowg, agxbuse(&value_buffer), 1);
+	agbindrec(t, "Agnodeinfo_t", sizeof(Agnodeinfo_t), true);
+	alloc_elist(tbl->column_count, ND_in(t));
+	alloc_elist(tbl->column_count, ND_out(t));
 	if (lastn) {
 	    ND_next(lastn) = t;
 	    lastn = t;
@@ -1486,15 +1412,21 @@ static void makeGraphs(htmltbl_t * tbl, graph_t * rowg, graph_t * colg)
 
     for (cells = tbl->u.n.cells; *cells; cells++) {
 	cp = *cells;
-	t = agfindnode(colg, nToName(cp->col));
-	h = agfindnode(colg, nToName(cp->col + cp->cspan));
+	agxbprint(&value_buffer, "%" PRIu16, cp->col);
+	t = agfindnode(colg, agxbuse(&value_buffer));
+	agxbprint(&value_buffer, "%d", cp->col + cp->colspan);
+	h = agfindnode(colg, agxbuse(&value_buffer));
 	checkEdge (colg, t, h, cp->data.box.UR.x);
 
-	t = agfindnode(rowg, nToName(cp->row));
-	h = agfindnode(rowg, nToName(cp->row + cp->rspan));
+	agxbprint(&value_buffer, "%" PRIu16, cp->row);
+	t = agfindnode(rowg, agxbuse(&value_buffer));
+	agxbprint(&value_buffer, "%d", cp->row + cp->rowspan);
+	h = agfindnode(rowg, agxbuse(&value_buffer));
 	checkEdge (rowg, t, h, cp->data.box.UR.y);
     }
 
+    agxbfree(&value_buffer);
+
     /* Make sure that 0 <= 1 <= 2 ...k. This implies graph connected. */
     checkChain(colg);
     checkChain(rowg);
@@ -1537,26 +1469,22 @@ static void sizeArray(htmltbl_t * tbl)
 {
     graph_t *rowg;
     graph_t *colg;
-#ifdef _WIN32
-    Agdesc_t dir = { 1, 1, 0, 1 };
-#else
     Agdesc_t dir = Agstrictdirected;
-#endif
 
     /* Do the 1D cases by hand */
-    if ((tbl->rc == 1) || (tbl->cc == 1)) {
+    if (tbl->row_count == 1 || tbl->column_count == 1) {
 	sizeLinearArray(tbl);
 	return;
     }
 
-    tbl->heights = N_NEW(tbl->rc + 1, int);
-    tbl->widths = N_NEW(tbl->cc + 1, int);
+    tbl->heights = N_NEW(tbl->row_count + 1, double);
+    tbl->widths = N_NEW(tbl->column_count + 1, double);
 
     rowg = agopen("rowg", dir, NULL);
     colg = agopen("colg", dir, NULL);
     /* Only need GD_nlist */
-    agbindrec(rowg, "Agraphinfo_t", sizeof(Agraphinfo_t), TRUE);	// graph custom data
-    agbindrec(colg, "Agraphinfo_t", sizeof(Agraphinfo_t), TRUE);	// graph custom data
+    agbindrec(rowg, "Agraphinfo_t", sizeof(Agraphinfo_t), true);	// graph custom data
+    agbindrec(colg, "Agraphinfo_t", sizeof(Agraphinfo_t), true);	// graph custom data
     makeGraphs(tbl, rowg, colg);
     rank(rowg, 2, INT_MAX);
     rank(colg, 2, INT_MAX);
@@ -1581,9 +1509,7 @@ static void pos_html_img(htmlimg_t * cp, boxf pos)
  */
 static void pos_html_txt(htmltxt_t * ftxt, char c)
 {
-    int i;
-
-    for (i = 0; i < ftxt->nspans; i++) {
+    for (size_t i = 0; i < ftxt->nspans; i++) {
 	if (ftxt->spans[i].just == UNSET_ALIGN)	/* unset */
 	    ftxt->spans[i].just = c;
     }
@@ -1598,12 +1524,12 @@ static void pos_html_cell(htmlcell_t * cp, boxf pos, int sides)
     boxf cbox;
 
     if (!cp->data.pencolor && cp->parent->data.pencolor)
-	cp->data.pencolor = strdup(cp->parent->data.pencolor);
+	cp->data.pencolor = gv_strdup(cp->parent->data.pencolor);
 
     /* If fixed, align cell */
     if (cp->data.flags & FIXED_FLAG) {
 	oldsz = cp->data.box.UR;
-	delx = (pos.UR.x - pos.LL.x) - oldsz.x;
+	delx = pos.UR.x - pos.LL.x - oldsz.x;
 	if (delx > 0) {
 	    switch (cp->data.flags & HALIGN_MASK) {
 	    case HALIGN_LEFT:
@@ -1619,7 +1545,7 @@ static void pos_html_cell(htmlcell_t * cp, boxf pos, int sides)
 		break;
 	    }
 	}
-	dely = (pos.UR.y - pos.LL.y) - oldsz.y;
+	dely = pos.UR.y - pos.LL.y - oldsz.y;
 	if (dely > 0) {
 	    switch (cp->data.flags & VALIGN_MASK) {
 	    case VALIGN_BOTTOM:
@@ -1650,7 +1576,7 @@ static void pos_html_cell(htmlcell_t * cp, boxf pos, int sides)
     } else if (cp->child.kind == HTML_IMAGE) {
 	/* Note that alignment trumps scaling */
 	oldsz = cp->child.u.img->box.UR;
-	delx = (cbox.UR.x - cbox.LL.x) - oldsz.x;
+	delx = cbox.UR.x - cbox.LL.x - oldsz.x;
 	if (delx > 0) {
 	    switch (cp->data.flags & HALIGN_MASK) {
 	    case HALIGN_LEFT:
@@ -1662,7 +1588,7 @@ static void pos_html_cell(htmlcell_t * cp, boxf pos, int sides)
 	    }
 	}
 
-	dely = (cbox.UR.y - cbox.LL.y) - oldsz.y;
+	dely = cbox.UR.y - cbox.LL.y - oldsz.y;
 	if (dely > 0) {
 	    switch (cp->data.flags & VALIGN_MASK) {
 	    case VALIGN_BOTTOM:
@@ -1679,12 +1605,11 @@ static void pos_html_cell(htmlcell_t * cp, boxf pos, int sides)
 	int af;
 
 	oldsz = cp->child.u.txt->box.UR;
-	delx = (cbox.UR.x - cbox.LL.x) - oldsz.x;
+	delx = cbox.UR.x - cbox.LL.x - oldsz.x;
 	/* If the cell is larger than the text block and alignment is 
 	 * done at textblock level, the text box is shrunk accordingly. 
 	 */
-	if ((delx > 0)
-	    && ((af = (cp->data.flags & HALIGN_MASK)) != HALIGN_TEXT)) {
+	if (delx > 0 && (af = (cp->data.flags & HALIGN_MASK)) != HALIGN_TEXT) {
 	    switch (af) {
 	    case HALIGN_LEFT:
 		cbox.UR.x -= delx;
@@ -1699,7 +1624,7 @@ static void pos_html_cell(htmlcell_t * cp, boxf pos, int sides)
 	    }
 	}
 
-	dely = (cbox.UR.y - cbox.LL.y) - oldsz.y;
+	dely = cbox.UR.y - cbox.LL.y - oldsz.y;
 	if (dely > 0) {
 	    switch (cp->data.flags & VALIGN_MASK) {
 	    case VALIGN_BOTTOM:
@@ -1741,22 +1666,19 @@ static void pos_html_cell(htmlcell_t * cp, boxf pos, int sides)
  */
 static void pos_html_tbl(htmltbl_t * tbl, boxf pos, int sides)
 {
-    int x, y, delx, dely, oldsz;
-    int i, extra, plus;
+    int plus;
     htmlcell_t **cells = tbl->u.n.cells;
     htmlcell_t *cp;
     boxf cbox;
 
     if (tbl->u.n.parent && tbl->u.n.parent->data.pencolor
 	&& !tbl->data.pencolor)
-	tbl->data.pencolor = strdup(tbl->u.n.parent->data.pencolor);
+	tbl->data.pencolor = gv_strdup(tbl->u.n.parent->data.pencolor);
 
-    oldsz = tbl->data.box.UR.x;
-    delx = (pos.UR.x - pos.LL.x) - oldsz;
-    assert(delx >= 0);
+    double oldsz = tbl->data.box.UR.x;
+    double delx = fmax(pos.UR.x - pos.LL.x - oldsz, 0);
     oldsz = tbl->data.box.UR.y;
-    dely = (pos.UR.y - pos.LL.y) - oldsz;
-    assert(dely >= 0);
+    double dely = fmax(pos.UR.y - pos.LL.y - oldsz, 0);
 
     /* If fixed, align box */
     if (tbl->data.flags & FIXED_FLAG) {
@@ -1795,19 +1717,21 @@ static void pos_html_tbl(htmltbl_t * tbl, boxf pos, int sides)
     }
 
     /* change sizes to start positions and distribute extra space */
-    x = pos.LL.x + tbl->data.border + tbl->data.space;
-    extra = delx / (tbl->cc);
-    plus = ROUND(delx - extra * (tbl->cc));
-    for (i = 0; i <= tbl->cc; i++) {
-	delx = tbl->widths[i] + extra + (i < plus ? 1 : 0);
+    double x = pos.LL.x + tbl->data.border + tbl->data.space;
+    assert(tbl->column_count <= DBL_MAX);
+    double extra = delx / (double)tbl->column_count;
+    plus = ROUND(delx - extra * (double)tbl->column_count);
+    for (size_t i = 0; i <= tbl->column_count; i++) {
+	delx = tbl->widths[i] + extra + ((i <= INT_MAX && (int)i < plus) ? 1 : 0);
 	tbl->widths[i] = x;
 	x += delx + tbl->data.space;
     }
-    y = pos.UR.y - tbl->data.border - tbl->data.space;
-    extra = dely / (tbl->rc);
-    plus = ROUND(dely - extra * (tbl->rc));
-    for (i = 0; i <= tbl->rc; i++) {
-	dely = tbl->heights[i] + extra + (i < plus ? 1 : 0);
+    double y = pos.UR.y - tbl->data.border - tbl->data.space;
+    assert(tbl->row_count <= DBL_MAX);
+    extra = dely / (double)tbl->row_count;
+    plus = ROUND(dely - extra * (double)tbl->row_count);
+    for (size_t i = 0; i <= tbl->row_count; i++) {
+	dely = tbl->heights[i] + extra + ((i <= INT_MAX && (int)i < plus) ? 1 : 0);
 	tbl->heights[i] = y;
 	y -= dely + tbl->data.space;
     }
@@ -1819,15 +1743,15 @@ static void pos_html_tbl(htmltbl_t * tbl, boxf pos, int sides)
 		mask |= LEFT;
 	    if (cp->row == 0)
 		mask |= TOP;
-	    if (cp->col + cp->cspan == tbl->cc)
+	    if (cp->col + cp->colspan == tbl->column_count)
 		mask |= RIGHT;
-	    if (cp->row + cp->rspan == tbl->rc)
+	    if (cp->row + cp->rowspan == tbl->row_count)
 		mask |= BOTTOM;
 	}
 	cbox.LL.x = tbl->widths[cp->col];
-	cbox.UR.x = tbl->widths[cp->col + cp->cspan] - tbl->data.space;
+	cbox.UR.x = tbl->widths[cp->col + cp->colspan] - tbl->data.space;
 	cbox.UR.y = tbl->heights[cp->row];
-	cbox.LL.y = tbl->heights[cp->row + cp->rspan] + tbl->data.space;
+	cbox.LL.y = tbl->heights[cp->row + cp->rowspan] + tbl->data.space;
 	pos_html_cell(cp, cbox, sides & mask);
     }
 
@@ -1843,7 +1767,6 @@ static int
 size_html_tbl(graph_t * g, htmltbl_t * tbl, htmlcell_t * parent,
 	      htmlenv_t * env)
 {
-    int i, wd, ht;
     int rv = 0;
     static textfont_t savef;
 
@@ -1862,28 +1785,31 @@ size_html_tbl(graph_t * g, htmltbl_t * tbl, htmlcell_t * parent,
 
     sizeArray(tbl);
 
-    wd = (tbl->cc + 1) * tbl->data.space + 2 * tbl->data.border;
-    ht = (tbl->rc + 1) * tbl->data.space + 2 * tbl->data.border;
-    for (i = 0; i < tbl->cc; i++)
+    assert(tbl->column_count <= DBL_MAX);
+    double wd = ((double)tbl->column_count + 1) * tbl->data.space + 2 * tbl->data.border;
+    assert(tbl->row_count <= DBL_MAX);
+    double ht = ((double)tbl->row_count + 1) * tbl->data.space + 2 * tbl->data.border;
+    for (size_t i = 0; i < tbl->column_count; i++)
 	wd += tbl->widths[i];
-    for (i = 0; i < tbl->rc; i++)
+    for (size_t i = 0; i < tbl->row_count; i++)
 	ht += tbl->heights[i];
 
     if (tbl->data.flags & FIXED_FLAG) {
 	if (tbl->data.width && tbl->data.height) {
-	    if ((tbl->data.width < wd) || (tbl->data.height < ht)) {
+	    if (tbl->data.width < wd || tbl->data.height < ht) {
 		agerr(AGWARN, "table size too small for content\n");
 		rv = 1;
 	    }
-	    wd = ht = 0;
+	    wd = 0;
+	    ht = 0;
 	} else {
 	    agerr(AGWARN,
 		  "fixed table size with unspecified width or height\n");
 	    rv = 1;
 	}
     }
-    tbl->data.box.UR.x = MAX(wd, tbl->data.width);
-    tbl->data.box.UR.y = MAX(ht, tbl->data.height);
+    tbl->data.box.UR.x = fmax(wd, tbl->data.width);
+    tbl->data.box.UR.y = fmax(ht, tbl->data.height);
 
     if (tbl->font)
 	popFontInfo(env, &savef);
@@ -1901,7 +1827,7 @@ static char *nameOf(void *obj, agxbuf * xb)
 	agxbput(xb, agnameof(obj));
 	break;
     case AGEDGE:
-	ep = (Agedge_t *) obj;
+	ep = obj;
 	agxbput(xb, agnameof(agtail(ep)));
 	agxbput(xb, agnameof(aghead(ep)));
 	if (agisdirected(agraphof(aghead(ep))))
@@ -1933,16 +1859,15 @@ void printImage(htmlimg_t * ip, int ind)
 
 void printTxt(htmltxt_t * txt, int ind)
 {
-    int i, j;
-
     indent(ind);
-    fprintf(stderr, "txt spans = %d \n", txt->nspans);
-    for (i = 0; i < txt->nspans; i++) {
+    fprintf(stderr, "txt spans = %" PRISIZE_T " \n", txt->nspans);
+    for (size_t i = 0; i < txt->nspans; i++) {
 	indent(ind + 1);
-	fprintf(stderr, "[%d] %d items\n", i, txt->spans[i].nitems);
-	for (j = 0; j < txt->spans[i].nitems; j++) {
+	fprintf(stderr, "[%" PRISIZE_T "] %" PRISIZE_T " items\n", i,
+	        txt->spans[i].nitems);
+	for (size_t j = 0; j < txt->spans[i].nitems; j++) {
 	    indent(ind + 2);
-	    fprintf(stderr, "[%d] (%f,%f) \"%s\" ",
+	    fprintf(stderr, "[%" PRISIZE_T "] (%f,%f) \"%s\" ",
 		    j, txt->spans[i].items[j].size.x,
             txt->spans[i].items[j].size.y,
 		    txt->spans[i].items[j].str);
@@ -1996,7 +1921,7 @@ void printTbl(htmltbl_t * tbl, int ind)
 {
     htmlcell_t **cells = tbl->u.n.cells;
     indent(ind);
-    fprintf(stderr, "tbl (%p) %d %d ", tbl, tbl->cc, tbl->rc);
+    fprintf(stderr, "tbl (%p) %" PRISIZE_T " %" PRISIZE_T " ", tbl, tbl->column_count, tbl->row_count);
     printData(&tbl->data);
     fputs("\n", stderr);
     while (*cells)
@@ -2006,8 +1931,8 @@ void printTbl(htmltbl_t * tbl, int ind)
 static void printCell(htmlcell_t * cp, int ind)
 {
     indent(ind);
-    fprintf(stderr, "cell %d %d %d %d ", cp->cspan, cp->rspan, cp->col,
-	    cp->row);
+    fprintf(stderr, "cell %" PRIu16 " %" PRIu16 " %" PRIu16 " %" PRIu16 " ", cp->colspan,
+            cp->colspan, cp->rowspan, cp->col, cp->row);
     printData(&cp->data);
     fputs("\n", stderr);
     switch (cp->child.kind) {
@@ -2038,9 +1963,9 @@ static char *getPenColor(void *obj)
 {
     char *str;
 
-    if (((str = agget(obj, "pencolor")) != 0) && str[0])
+    if ((str = agget(obj, "pencolor")) != 0 && str[0])
 	return str;
-    else if (((str = agget(obj, "color")) != 0) && str[0])
+    else if ((str = agget(obj, "color")) != 0 && str[0])
 	return str;
     else
 	return NULL;
@@ -2053,7 +1978,6 @@ int make_html_label(void *obj, textlabel_t * lp)
 {
     int rv;
     double wd2, ht2;
-    boxf box;
     graph_t *g;
     htmllabel_t *lbl;
     htmlenv_t env;
@@ -2079,12 +2003,14 @@ int make_html_label(void *obj, textlabel_t * lp)
     env.finfo.flags = 0;
     lbl = parseHTML(lp->text, &rv, &env);
     if (!lbl) {
+	if (rv == 3) {
+	    // fatal error
+	    graphviz_exit(EXIT_FAILURE);
+	}
 	/* Parse of label failed; revert to simple text label */
-	agxbuf xb;
-	unsigned char buf[SMALLBUF];
-	agxbinit(&xb, SMALLBUF, buf);
-	lp->html = FALSE;
-	lp->text = strdup(nameOf(obj, &xb));
+	agxbuf xb = {0};
+	lp->html = false;
+	lp->text = gv_strdup(nameOf(obj, &xb));
 	switch (lp->charset) {
 	case CHAR_LATIN1:
 	    s = latin1ToUTF8(lp->text);
@@ -2102,22 +2028,22 @@ int make_html_label(void *obj, textlabel_t * lp)
 
     if (lbl->kind == HTML_TBL) {
 	if (!lbl->u.tbl->data.pencolor && getPenColor(obj))
-	    lbl->u.tbl->data.pencolor = strdup(getPenColor(obj));
+	    lbl->u.tbl->data.pencolor = gv_strdup(getPenColor(obj));
 	rv |= size_html_tbl(g, lbl->u.tbl, NULL, &env);
-	wd2 = (lbl->u.tbl->data.box.UR.x) / 2;
-	ht2 = (lbl->u.tbl->data.box.UR.y) / 2;
-	box = boxfof(-wd2, -ht2, wd2, ht2);
-	pos_html_tbl(lbl->u.tbl, box, BOTTOM | RIGHT | TOP | LEFT);
-	lp->dimen.x = box.UR.x - box.LL.x;
-	lp->dimen.y = box.UR.y - box.LL.y;
+	wd2 = lbl->u.tbl->data.box.UR.x / 2;
+	ht2 = lbl->u.tbl->data.box.UR.y / 2;
+	boxf b = {{-wd2, -ht2}, {wd2, ht2}};
+	pos_html_tbl(lbl->u.tbl, b, BOTTOM | RIGHT | TOP | LEFT);
+	lp->dimen.x = b.UR.x - b.LL.x;
+	lp->dimen.y = b.UR.y - b.LL.y;
     } else {
 	rv |= size_html_txt(GD_gvc(g), lbl->u.txt, &env);
 	wd2 = lbl->u.txt->box.UR.x  / 2;
 	ht2 = lbl->u.txt->box.UR.y  / 2;
-	box = boxfof(-wd2, -ht2, wd2, ht2);
-	lbl->u.txt->box = box;
-	lp->dimen.x = box.UR.x - box.LL.x;
-	lp->dimen.y = box.UR.y - box.LL.y;
+	boxf b = {{-wd2, -ht2}, {wd2, ht2}};
+	lbl->u.txt->box = b;
+	lp->dimen.x = b.UR.x - b.LL.x;
+	lp->dimen.y = b.UR.y - b.LL.y;
     }
 
     lp->u.html = lbl;
@@ -2127,7 +2053,7 @@ int make_html_label(void *obj, textlabel_t * lp)
      */
     if (lbl->kind == HTML_TBL) {
 	free(lp->text);
-	lp->text = strdup("<TABLE>");
+	lp->text = gv_strdup("<TABLE>");
     }
 
     return rv;
