@@ -9,47 +9,41 @@
  *************************************************************************/
 
 #include "config.h"
-
+#include <assert.h>
+#include <cgraph/agxbuf.h>
+#include <cgraph/alloc.h>
+#include <common/geomprocs.h>
 #include <common/render.h>
+#include <limits.h>
 #include <math.h>
 #include <pathplan/pathplan.h>
-#include <setjmp.h>
 #include <stdbool.h>
 #include <stdlib.h>
-
-#define PINC 300
+#include <string.h>
 
 static int nedges, nboxes; /* total no. of edges and boxes used in routing */
 
 static int routeinit;
 /* static data used across multiple edges */
-static pointf *ps;             /* final spline points */
-static int maxpn;             /* size of ps[] */
 static Ppoint_t *polypoints;  /* vertices of polygon defined by boxes */
 static int polypointn;        /* size of polypoints[] */
 static Pedge_t *edges;        /* polygon edges passed to Proutespline */
 static int edgen;             /* size of edges[] */
 
 static int checkpath(int, boxf*, path*);
-static int mkspacep(int size);
 static void printpath(path * pp);
 #ifdef DEBUG
 static void printboxes(int boxn, boxf* boxes)
 {
     pointf ll, ur;
     int bi;
-    char buf[BUFSIZ];
-    int newcnt = Show_cnt + boxn;
 
-    Show_boxes = ALLOC(newcnt+2,Show_boxes,char*);
     for (bi = 0; bi < boxn; bi++) {
 	ll = boxes[bi].LL, ur = boxes[bi].UR;
-	snprintf(buf, sizeof(buf), "%d %d %d %d pathbox", (int)ll.x, (int)ll.y,
-	         (int)ur.x, (int)ur.y);
-	Show_boxes[bi+1+Show_cnt] = strdup (buf);
+	agxbuf buf = {0};
+	agxbprint(&buf, "%.0f %.0f %.0f %.0f pathbox", ll.x, ll.y, ur.x, ur.y);
+	show_boxes_append(&Show_boxes, agxbdisown(&buf));
     }
-    Show_cnt = newcnt;
-    Show_boxes[Show_cnt+1] = NULL;
 }
 
 #if DEBUG > 1
@@ -90,58 +84,44 @@ static void psprintpointf(pointf p)
 
 static void psprintspline(Ppolyline_t spl)
 {
-    char buf[BUFSIZ];
-    int newcnt = Show_cnt + spl.pn + 4;
-    int li, i;
+    int i;
 
-    Show_boxes = ALLOC(newcnt+2,Show_boxes,char*);
-    li = Show_cnt+1;
-    Show_boxes[li++] = strdup ("%%!");
-    Show_boxes[li++] = strdup ("%% spline");
-    Show_boxes[li++] = strdup ("gsave 1 0 0 setrgbcolor newpath");
+    show_boxes_append(&Show_boxes, gv_strdup("%%!"));
+    show_boxes_append(&Show_boxes, gv_strdup("%% spline"));
+    show_boxes_append(&Show_boxes, gv_strdup("gsave 1 0 0 setrgbcolor newpath"));
     for (i = 0; i < spl.pn; i++) {
-	snprintf(buf, sizeof(buf), "%f %f %s", spl.ps[i].x, spl.ps[i].y,
+	agxbuf buf = {0};
+	agxbprint(&buf, "%f %f %s", spl.ps[i].x, spl.ps[i].y,
 	  i == 0 ?  "moveto" : (i % 3 == 0 ? "curveto" : ""));
-	Show_boxes[li++] = strdup (buf);
+	show_boxes_append(&Show_boxes, agxbdisown(&buf));
     }
-    Show_boxes[li++] = strdup ("stroke grestore");
-    Show_cnt = newcnt;
-    Show_boxes[Show_cnt+1] = NULL;
+    show_boxes_append(&Show_boxes, gv_strdup("stroke grestore"));
 }
 
 static void psprintline(Ppolyline_t pl)
 {
-    char buf[BUFSIZ];
-    int newcnt = Show_cnt + pl.pn + 4;
-    int i, li;
+    int i;
 
-    Show_boxes = ALLOC(newcnt+2,Show_boxes,char*);
-    li = Show_cnt+1;
-    Show_boxes[li++] = strdup ("%%!");
-    Show_boxes[li++] = strdup ("%% line");
-    Show_boxes[li++] = strdup ("gsave 0 0 1 setrgbcolor newpath");
+    show_boxes_append(&Show_boxes, gv_strdup("%%!"));
+    show_boxes_append(&Show_boxes, gv_strdup("%% line"));
+    show_boxes_append(&Show_boxes, gv_strdup("gsave 0 0 1 setrgbcolor newpath"));
     for (i = 0; i < pl.pn; i++) {
-	snprintf(buf, sizeof(buf), "%f %f %s", pl.ps[i].x, pl.ps[i].y,
+	agxbuf buf = {0};
+	agxbprint(&buf, "%f %f %s", pl.ps[i].x, pl.ps[i].y,
 		i == 0 ? "moveto" : "lineto");
-	Show_boxes[li++] = strdup (buf);
+	show_boxes_append(&Show_boxes, agxbdisown(&buf));
     }
-    Show_boxes[li++] = strdup ("stroke grestore");
-    Show_cnt = newcnt;
-    Show_boxes[Show_cnt+1] = NULL;
+    show_boxes_append(&Show_boxes, gv_strdup("stroke grestore"));
 }
 
 static void psprintpoly(Ppoly_t p)
 {
-    char buf[BUFSIZ];
-    int newcnt = Show_cnt + p.pn + 3;
     point tl, hd;
-    int bi, li;
+    int bi;
     char*  pfx;
 
-    Show_boxes = ALLOC(newcnt+2,Show_boxes,char*);
-    li = Show_cnt+1;
-    Show_boxes[li++] = strdup ("%% poly list");
-    Show_boxes[li++] = strdup ("gsave 0 1 0 setrgbcolor");
+    show_boxes_append(&Show_boxes, gv_strdup("%% poly list"));
+    show_boxes_append(&Show_boxes, gv_strdup("gsave 0 1 0 setrgbcolor"));
     for (bi = 0; bi < p.pn; bi++) {
 	tl.x = (int)p.ps[bi].x;
 	tl.y = (int)p.ps[bi].y;
@@ -149,56 +129,42 @@ static void psprintpoly(Ppoly_t p)
 	hd.y = (int)p.ps[(bi+1) % p.pn].y;
 	if (tl.x == hd.x && tl.y == hd.y) pfx = "%%";
 	else pfx ="";
-	snprintf(buf, sizeof(buf), "%s%d %d %d %d makevec", pfx, tl.x, tl.y, hd.x,
-	         hd.y);
-	Show_boxes[li++] = strdup (buf);
+	agxbuf buf = {0};
+	agxbprint(&buf, "%s%d %d %d %d makevec", pfx, tl.x, tl.y, hd.x, hd.y);
+	show_boxes_append(&Show_boxes, agxbdisown(&buf));
     }
-    Show_boxes[li++] = strdup ("grestore");
-
-    Show_cnt = newcnt;
-    Show_boxes[Show_cnt+1] = NULL;
+    show_boxes_append(&Show_boxes, gv_strdup("grestore"));
 }
 
 static void psprintboxes(int boxn, boxf* boxes)
 {
-    char buf[BUFSIZ];
-    int newcnt = Show_cnt + 5*boxn + 3;
     pointf ll, ur;
-    int bi, li;
+    int bi;
 
-    Show_boxes = ALLOC(newcnt+2,Show_boxes,char*);
-    li = Show_cnt+1;
-    Show_boxes[li++] = strdup ("%% box list");
-    Show_boxes[li++] = strdup ("gsave 0 1 0 setrgbcolor");
+    show_boxes_append(&Show_boxes, gv_strdup("%% box list"));
+    show_boxes_append(&Show_boxes, gv_strdup("gsave 0 1 0 setrgbcolor"));
     for (bi = 0; bi < boxn; bi++) {
 	ll = boxes[bi].LL, ur = boxes[bi].UR;
-	snprintf(buf, sizeof(buf), "newpath\n%d %d moveto", (int)ll.x, (int)ll.y);
-	Show_boxes[li++] = strdup (buf);
-	snprintf(buf, sizeof(buf), "%d %d lineto", (int)ll.x, (int)ur.y);
-	Show_boxes[li++] = strdup (buf);
-	snprintf(buf, sizeof(buf), "%d %d lineto", (int)ur.x, (int)ur.y);
-	Show_boxes[li++] = strdup (buf);
-	snprintf(buf, sizeof(buf), "%d %d lineto", (int)ur.x, (int)ll.y);
-	Show_boxes[li++] = strdup (buf);
-	Show_boxes[li++] = strdup ("closepath stroke");
+	agxbuf buf = {0};
+	agxbprint(&buf, "newpath\n%.0f %.0f moveto", ll.x, ll.y);
+	show_boxes_append(&Show_boxes, agxbdisown(&buf));
+	agxbprint(&buf, "%.0f %.0f lineto", ll.x, ur.y);
+	show_boxes_append(&Show_boxes, agxbdisown(&buf));
+	agxbprint(&buf, "%.0f %.0f lineto", ur.x, ur.y);
+	show_boxes_append(&Show_boxes, agxbdisown(&buf));
+	agxbprint(&buf, "%.0f %.0f lineto", ur.x, ll.y);
+	show_boxes_append(&Show_boxes, agxbdisown(&buf));
+	show_boxes_append(&Show_boxes, gv_strdup("closepath stroke"));
     }
-    Show_boxes[li++] = strdup ("grestore");
-
-    Show_cnt = newcnt;
-    Show_boxes[Show_cnt+1] = NULL;
+    show_boxes_append(&Show_boxes, gv_strdup("grestore"));
 }
 
 static void psprintinit (int begin)
 {
-    int newcnt = Show_cnt + 1;
-
-    Show_boxes = ALLOC(newcnt+2,Show_boxes,char*);
     if (begin)
-	Show_boxes[1+Show_cnt] = strdup ("dbgstart");
+	show_boxes_append(&Show_boxes, gv_strdup("dbgstart"));
     else
-	Show_boxes[1+Show_cnt] = strdup ("grestore");
-    Show_cnt = newcnt;
-    Show_boxes[Show_cnt+1] = NULL;
+	show_boxes_append(&Show_boxes, gv_strdup("grestore"));
 }
 
 static bool debugleveln(edge_t* realedge, int i)
@@ -249,8 +215,11 @@ simpleSplineRoute (pointf tp, pointf hp, Ppoly_t poly, int* n_spl_pts,
             return NULL;
     }
 
-    if (mkspacep(spl.pn))
+    pointf *ps = calloc(spl.pn, sizeof(ps[0]));
+    if (ps == NULL) {
+	agerr(AGERR, "cannot allocate ps\n");
 	return NULL;
+    }
     for (i = 0; i < spl.pn; i++) {
         ps[i] = spl.ps[i];
     }
@@ -266,19 +235,8 @@ int
 routesplinesinit()
 {
     if (++routeinit > 1) return 0;
-    if (!(ps = calloc(PINC, sizeof(pointf)))) {
-	agerr(AGERR, "routesplinesinit: cannot allocate ps\n");
-	return 1;
-    }
-    maxpn = PINC;
 #ifdef DEBUG
-    if (Show_boxes) {
-        for (int i = 0; Show_boxes[i]; i++)
-	    free (Show_boxes[i]);
-	free (Show_boxes);
-	Show_boxes = NULL;
-	Show_cnt = 0;
-    }
+    show_boxes_free(&Show_boxes);
 #endif
     nedges = 0;
     nboxes = 0;
@@ -290,7 +248,6 @@ routesplinesinit()
 void routesplinesterm()
 {
     if (--routeinit > 0) return;
-    free(ps);
     if (Verbose)
 	fprintf(stderr,
 		"routesplines: %d edges, %d boxes %.2f sec\n",
@@ -312,18 +269,18 @@ limitBoxes (boxf* boxes, int boxn, const pointf *pps, int pn, int delta)
 	    sp[1] = pps[splinepi + 1];
 	    sp[2] = pps[splinepi + 2];
 	    sp[3] = pps[splinepi + 3];
-	    sp[0].x = sp[0].x + t * (sp[1].x - sp[0].x);
-	    sp[0].y = sp[0].y + t * (sp[1].y - sp[0].y);
-	    sp[1].x = sp[1].x + t * (sp[2].x - sp[1].x);
-	    sp[1].y = sp[1].y + t * (sp[2].y - sp[1].y);
-	    sp[2].x = sp[2].x + t * (sp[3].x - sp[2].x);
-	    sp[2].y = sp[2].y + t * (sp[3].y - sp[2].y);
- 	    sp[0].x = sp[0].x + t * (sp[1].x - sp[0].x);
-	    sp[0].y = sp[0].y + t * (sp[1].y - sp[0].y);
-	    sp[1].x = sp[1].x + t * (sp[2].x - sp[1].x);
-	    sp[1].y = sp[1].y + t * (sp[2].y - sp[1].y);
-	    sp[0].x = sp[0].x + t * (sp[1].x - sp[0].x);
-	    sp[0].y = sp[0].y + t * (sp[1].y - sp[0].y);
+	    sp[0].x += t * (sp[1].x - sp[0].x);
+	    sp[0].y += t * (sp[1].y - sp[0].y);
+	    sp[1].x += t * (sp[2].x - sp[1].x);
+	    sp[1].y += t * (sp[2].y - sp[1].y);
+	    sp[2].x += t * (sp[3].x - sp[2].x);
+	    sp[2].y += t * (sp[3].y - sp[2].y);
+	    sp[0].x += t * (sp[1].x - sp[0].x);
+	    sp[0].y += t * (sp[1].y - sp[0].y);
+	    sp[1].x += t * (sp[2].x - sp[1].x);
+	    sp[1].y += t * (sp[2].y - sp[1].y);
+	    sp[0].x += t * (sp[1].x - sp[0].x);
+	    sp[0].y += t * (sp[1].y - sp[0].y);
 	    for (bi = 0; bi < boxn; bi++) {
 /* this tested ok on 64bit machines, but on 32bit we need this FUDGE
  *     or graphs/directed/records.gv fails */
@@ -509,8 +466,12 @@ static pointf *_routesplines(path * pp, int *npoints, int polyline)
 	    polypoints[i].y *= -1;
     }
 
-    for (bi = 0; bi < boxn; bi++)
-	boxes[bi].LL.x = INT_MAX, boxes[bi].UR.x = INT_MIN;
+    static const double INITIAL_LLX = INT_MAX;
+    static const double INITIAL_URX = INT_MIN;
+    for (bi = 0; bi < boxn; bi++) {
+	boxes[bi].LL.x = INITIAL_LLX;
+	boxes[bi].UR.x = INITIAL_URX;
+    }
     poly.ps = polypoints, poly.pn = pi;
     eps[0].x = pp->start.p.x, eps[0].y = pp->start.p.y;
     eps[1].x = pp->end.p.x, eps[1].y = pp->end.p.y;
@@ -559,13 +520,12 @@ static pointf *_routesplines(path * pp, int *npoints, int polyline)
 	}
 #endif
     }
-    if (mkspacep(spl.pn))
+    pointf *ps = calloc(spl.pn, sizeof(ps[0]));
+    if (ps == NULL) {
+	agerr(AGERR, "cannot allocate ps\n");
 	return NULL;  /* Bailout if no memory left */
-
-    for (bi = 0; bi < boxn; bi++) {
-	boxes[bi].LL.x = INT_MAX;
-	boxes[bi].UR.x = INT_MIN;
     }
+
     unbounded = true;
     for (splinepi = 0; splinepi < spl.pn; splinepi++) {
 	ps[splinepi] = spl.ps[splinepi];
@@ -582,7 +542,8 @@ static pointf *_routesplines(path * pp, int *npoints, int polyline)
 	for (bi = 0; bi < boxn; bi++) {
 	/* these fp equality tests are used only to detect if the
 	 * values have been changed since initialization - ok */
-	    if (boxes[bi].LL.x == INT_MAX || boxes[bi].UR.x == INT_MIN) {
+	    if (memcmp(&boxes[bi].LL.x, &INITIAL_LLX, sizeof(boxes[bi].LL.x)) == 0 ||
+	        memcmp(&boxes[bi].UR.x, &INITIAL_URX, sizeof(boxes[bi].UR.x)) == 0) {
 		delta *= 2; /* try again with a finer interval */
 		if (delta > INT_MAX/boxn) /* in limitBoxes, boxn*delta must fit in an int, so give up */
 		    loopcnt = LOOP_TRIES;
@@ -628,18 +589,23 @@ pointf *routepolylines(path * pp, int *npoints)
     return _routesplines (pp, npoints, 1);
 }
 
-static int overlap(int i0, int i1, int j0, int j1)
-{
-    /* i'll bet there's an elegant way to do this */
-    if (i1 <= j0)
-	return 0;
-    if (i0 >= j1)
-	return 0;
-    if (j0 <= i0 && i0 <= j1)
-	return j1 - i0;
-    if (j0 <= i1 && i1 <= j1)
-	return i1 - j0;
-    return MIN(i1 - i0, j1 - j0);
+static double overlap(double i0, double i1, double j0, double j1) {
+  if (i1 <= j0)
+    return 0;
+  if (i0 >= j1)
+    return 0;
+
+  // does the first interval subsume the second?
+  if (i0 <= j0 && i1 >= j1)
+    return i1 - i0;
+  // does the second interval subsume the first?
+  if (j0 <= i0 && j1 >= i1)
+    return j1 - j0;
+
+  if (j0 <= i0 && i0 <= j1)
+    return j1 - i0;
+  assert(j0 <= i1 && i1 <= j1);
+  return i1 - j0;
 }
 
 
@@ -656,7 +622,6 @@ static int checkpath(int boxn, boxf* boxes, path* thepath)
 {
     boxf *ba, *bb;
     int bi, i, errs, l, r, d, u;
-    int xoverlap, yoverlap;
 
     /* remove degenerate boxes. */
     i = 0;
@@ -694,7 +659,7 @@ static int checkpath(int boxn, boxf* boxes, path* thepath)
 	    printpath(thepath);
 	}
 	if (errs > 0) {
-	    int xy;
+	    double xy;
 
 	    if (l == 1)
 		xy = ba->UR.x, ba->UR.x = bb->LL.x, bb->LL.x = xy, l = 0;
@@ -720,9 +685,9 @@ static int checkpath(int boxn, boxf* boxes, path* thepath)
 	    }
 	}
 	/* check for overlapping boxes */
-	xoverlap = overlap(ba->LL.x, ba->UR.x, bb->LL.x, bb->UR.x);
-	yoverlap = overlap(ba->LL.y, ba->UR.y, bb->LL.y, bb->UR.y);
-	if (xoverlap && yoverlap) {
+	double xoverlap = overlap(ba->LL.x, ba->UR.x, bb->LL.x, bb->UR.x);
+	double yoverlap = overlap(ba->LL.y, ba->UR.y, bb->LL.y, bb->UR.y);
+	if (xoverlap > 0 && yoverlap > 0) {
 	    if (xoverlap < yoverlap) {
 		if (ba->UR.x - ba->LL.x > bb->UR.x - bb->LL.x) {
 		    /* take space from ba */
@@ -784,20 +749,6 @@ static int checkpath(int boxn, boxf* boxes, path* thepath)
     return 0;
 }
 
-static int mkspacep(int size)
-{
-    if (size > maxpn) {
-	int newmax = maxpn + (size / PINC + 1) * PINC;
-	ps = realloc(ps, newmax * sizeof(pointf));
-	if (!ps) {
-	    agerr(AGERR, "cannot re-allocate ps\n");
-	    return 1;
-	}
-	maxpn = newmax;
-    }
-    return 0;
-}
-
 static void printpath(path * pp)
 {
     int bi;
@@ -854,13 +805,18 @@ static void* vec_get(vec* pvec, size_t index)
 
 static void vec_delete(vec* pvec)
 {
-    for (size_t i = 0; i < vec_length(pvec); ++i) {
-      vec_delete(vec_get(pvec, i));
-    }
     free(pvec->_mem);
     free(pvec);
 }
 
+// cycles is assumed to be a vec of vec of nodes.
+static void cycles_delete(vec* cycles) {
+  for (size_t i = 0; i < vec_length(cycles); ++i) {
+    vec_delete(vec_get(cycles, i));
+  }
+  vec_delete(cycles);
+}
+
 static void vec_push_back(vec* pvec, void* data)
 {
     if (pvec->_elems == pvec->_capelems) {
@@ -980,6 +936,7 @@ static void dfs(graph_t *g, node_t* search, vec* visited, node_t* end, vec* cycl
 	}
 }
 
+// Returns a vec of vec of nodes (aka a vector of cycles), which must be freed using cycles_delete.
 static vec* find_all_cycles(graph_t *g)
 {
     node_t *n;
@@ -995,7 +952,7 @@ static vec* find_all_cycles(graph_t *g)
 		dfs(g, n, cycle, n, cycles);
 	}
 	
-	vec_delete(alloced_cycles); //cycles contains copied vecs
+	cycles_delete(alloced_cycles); //cycles contains copied vecs
     return cycles;
 }
 
@@ -1025,32 +982,25 @@ static vec* find_shortest_cycle_with_edge(vec* cycles, edge_t* edge, size_t min_
 
 static pointf get_cycle_centroid(graph_t *g, edge_t* edge)
 {
-	static vec* cycles = 0;
-	static graph_t* ref_g = 0;
-
-	if (cycles == 0 || ref_g != g) {
-		//free the memory we're using to hold the previous cycles
-		if (cycles != 0) {
-			vec_delete(cycles);
-		}
-		cycles = find_all_cycles(g);
-		ref_g = g;
-	}
+	vec* cycles = find_all_cycles(g);
 
 	//find the center of the shortest cycle containing this edge
 	//cycles of length 2 do their own thing, we want 3 or
 	vec* cycle = find_shortest_cycle_with_edge(cycles, edge, 3);
 	size_t cycle_len;
-	size_t cnt = 0;
     pointf sum = {0.0, 0.0};
 	size_t idx; //edge index
 	node_t *n;
 
-	if (!cycle)
+	if (cycle == NULL) {
+		if (cycles != NULL)
+			cycles_delete(cycles);
 		return get_centroid(g);
+	}
 
 	cycle_len = vec_length(cycle);
 
+	double cnt = 0;
 	for (idx=0; idx < cycle_len; ++idx) {
 		n = vec_get(cycle, idx);
 		sum.x += ND_coord(n).x;
@@ -1058,6 +1008,9 @@ static pointf get_cycle_centroid(graph_t *g, edge_t* edge)
         cnt++;
 	}
 
+	if (cycles != NULL)
+		cycles_delete(cycles);
+
 	sum.x /= cnt;
     sum.y /= cnt;
     return sum;
@@ -1065,15 +1018,11 @@ static pointf get_cycle_centroid(graph_t *g, edge_t* edge)
 
 static void bend(pointf spl[4], pointf centroid)
 {
-    pointf  midpt,a;
+    pointf  a;
     double  r;
-    double  dist,dx,dy;
 
-    midpt.x = (spl[0].x + spl[3].x)/2.0;
-    midpt.y = (spl[0].y + spl[3].y)/2.0;
-    dx = (spl[3].x - spl[0].x);
-    dy = (spl[3].y - spl[0].y);
-    dist = hypot(dx, dy);
+    pointf midpt = mid_pointf(spl[0], spl[3]);
+    double dist = DIST(spl[3], spl[0]);
     r = dist/5.0;
     {
         double vX = centroid.x - midpt.x;
@@ -1092,62 +1041,46 @@ static void bend(pointf spl[4], pointf centroid)
  *
  * FIX: handle ports on boundary?
  */
-#define MAX_EDGE 20
 void 
 makeStraightEdge(graph_t * g, edge_t * e, int et, splineInfo* sinfo)
 {
     edge_t *e0;
-    edge_t** edges;
-    edge_t* elist[MAX_EDGE];
-    int i, e_cnt;
 
-    e_cnt = 1;
+    size_t e_cnt = 1;
     e0 = e;
     while (e0 != ED_to_virt(e0) && (e0 = ED_to_virt(e0))) e_cnt++;
 
-    if (e_cnt <= MAX_EDGE)
-	edges = elist;
-    else
-	edges = N_NEW(e_cnt,edge_t*);
+    edge_t **edge_list = N_NEW(e_cnt, edge_t*);
     e0 = e;
-    for (i = 0; i < e_cnt; i++) {
-	edges[i] = e0;
+    for (size_t i = 0; i < e_cnt; i++) {
+	edge_list[i] = e0;
 	e0 = ED_to_virt(e0);
     }
-    makeStraightEdges (g, edges, e_cnt, et, sinfo);
-    if (e_cnt > MAX_EDGE) free (edges);
-
+    assert(e_cnt <= INT_MAX);
+    makeStraightEdges(g, edge_list, (int)e_cnt, et, sinfo);
+    free(edge_list);
 }
 
 void 
-makeStraightEdges(graph_t * g, edge_t** edges, int e_cnt, int et, splineInfo* sinfo)
-{
+makeStraightEdges(graph_t *g, edge_t **edge_list, int e_cnt, int et,
+                  splineInfo *sinfo) {
     pointf dumb[4];
-    node_t *n;
-    node_t *head;
     bool curved = et == EDGETYPE_CURVED;
-    pointf perp;
     pointf del;
-    edge_t *e0;
-    edge_t *e;
-    int i, j, xstep, dx;
-    double l_perp;
-    pointf dumber[4];
-    pointf p, q;
 
-    e = edges[0];
-    n = agtail(e);
-    head = aghead(e);
-    p = dumb[1] = dumb[0] = add_pointf(ND_coord(n), ED_tail_port(e).p);
-    q = dumb[2] = dumb[3] = add_pointf(ND_coord(head), ED_head_port(e).p);
+    edge_t *e = edge_list[0];
+    node_t *n = agtail(e);
+    node_t *head = aghead(e);
+    dumb[1] = dumb[0] = add_pointf(ND_coord(n), ED_tail_port(e).p);
+    dumb[2] = dumb[3] = add_pointf(ND_coord(head), ED_head_port(e).p);
     if (e_cnt == 1 || Concentrate) {
-	if (curved) bend(dumb,get_cycle_centroid(g, edges[0]));
+	if (curved) bend(dumb,get_cycle_centroid(g, edge_list[0]));
 	clip_and_install(e, aghead(e), dumb, 4, sinfo);
-	addEdgeLabels(e, p, q);
+	addEdgeLabels(e);
 	return;
     }
 
-    e0 = e;
+    edge_t *e0 = e;
     if (APPROXEQPT(dumb[0], dumb[3], MILLIPOINT)) {
 	/* degenerate case */
 	dumb[1] = dumb[0];
@@ -1156,11 +1089,13 @@ makeStraightEdges(graph_t * g, edge_t** edges, int e_cnt, int et, splineInfo* si
 	del.y = 0;
     }
     else {
-        perp.x = dumb[0].y - dumb[3].y;
-        perp.y = dumb[3].x - dumb[0].x;
-	l_perp = LEN(perp.x, perp.y);
-	xstep = GD_nodesep(g->root);
-	dx = xstep * (e_cnt - 1) / 2;
+        pointf perp = {
+          .x = dumb[0].y - dumb[3].y,
+          .y = dumb[3].x - dumb[0].x
+        };
+	double l_perp = hypot(perp.x, perp.y);
+	int xstep = GD_nodesep(g->root);
+	int dx = xstep * (e_cnt - 1) / 2;
 	dumb[1].x = dumb[0].x + dx * perp.x / l_perp;
 	dumb[1].y = dumb[0].y + dx * perp.y / l_perp;
 	dumb[2].x = dumb[3].x + dx * perp.x / l_perp;
@@ -1169,40 +1104,29 @@ makeStraightEdges(graph_t * g, edge_t** edges, int e_cnt, int et, splineInfo* si
 	del.y = -xstep * perp.y / l_perp;
     }
 
-    for (i = 0; i < e_cnt; i++) {
-	e0 = edges[i];
+    for (int i = 0; i < e_cnt; i++) {
+	e0 = edge_list[i];
+	pointf dumber[4];
 	if (aghead(e0) == head) {
-	    p = dumb[0];
-	    q = dumb[3];
-	    for (j = 0; j < 4; j++) {
+	    for (size_t j = 0; j < 4; j++) {
 		dumber[j] = dumb[j];
 	    }
 	} else {
-	    p = dumb[3];
-	    q = dumb[0];
-	    for (j = 0; j < 4; j++) {
+	    for (size_t j = 0; j < 4; j++) {
 		dumber[3 - j] = dumb[j];
 	    }
 	}
 	if (et == EDGETYPE_PLINE) {
-	    Ppoint_t pts[4];
-	    Ppolyline_t spl, line;
-
-	    line.pn = 4;
-	    line.ps = pts;
-	    for (j=0; j < 4; j++) {
-		pts[j] = dumber[j];
-	    }
+	    Ppoint_t pts[] = {dumber[0], dumber[1], dumber[2], dumber[3]};
+	    Ppolyline_t spl, line = {.pn = 4, .ps = pts};
 	    make_polyline (line, &spl);
 	    clip_and_install(e0, aghead(e0), spl.ps, spl.pn, sinfo);
 	}
 	else
 	    clip_and_install(e0, aghead(e0), dumber, 4, sinfo);
 
-	addEdgeLabels(e0, p, q);
-	dumb[1].x += del.x;
-	dumb[1].y += del.y;
-	dumb[2].x += del.x;
-	dumb[2].y += del.y;
+	addEdgeLabels(e0);
+	dumb[1] = add_pointf(dumb[1], del);
+	dumb[2] = add_pointf(dumb[2], del);
     }
 }
