@@ -18,6 +18,7 @@
 #include <gvc/gvplugin_textlayout.h>
 #include <gvc/gvcint.h>
 #include <gvc/gvcproc.h>
+#include <stdbool.h>
 
 int gvtextlayout_select(GVC_t * gvc)
 {
@@ -27,17 +28,17 @@ int gvtextlayout_select(GVC_t * gvc)
     plugin = gvplugin_load(gvc, API_textlayout, "textlayout");
     if (plugin) {
 	typeptr = plugin->typeptr;
-	gvc->textlayout.engine = (gvtextlayout_engine_t *) (typeptr->engine);
+	gvc->textlayout.engine = typeptr->engine;
 	return GVRENDER_PLUGIN;  /* FIXME - need more suitable success code */
     }
     return NO_SUPPORT;
 }
 
-boolean gvtextlayout(GVC_t *gvc, textspan_t *span, char **fontpath)
+bool gvtextlayout(GVC_t *gvc, textspan_t *span, char **fontpath)
 {
     gvtextlayout_engine_t *gvte = gvc->textlayout.engine;
 
     if (gvte && gvte->textlayout)
 	return gvte->textlayout(span, fontpath);
-    return FALSE;
+    return false;
 }
