@@ -10,8 +10,9 @@
 
 
 #include "config.h"
-
+#include <cgraph/alloc.h>
 #include <cgraph/unreachable.h>
+#include <math.h>
 #include <neatogen/neato.h>
 #include <neatogen/adjust.h>
 #include <pathplan/pathplan.h>
@@ -23,19 +24,17 @@
 #include <ortho/ortho.h>
 #endif
 
-extern int in_poly(Ppoly_t argpoly, Ppoint_t q);
 
-
-static boolean spline_merge(node_t * n)
+static bool spline_merge(node_t * n)
 {
     (void)n;
-    return FALSE;
+    return false;
 }
 
-static boolean swap_ends_p(edge_t * e)
+static bool swap_ends_p(edge_t * e)
 {
     (void)e;
-    return FALSE;
+    return false;
 }
 
 static splineInfo sinfo = {.swapEnds = swap_ends_p,
@@ -46,7 +45,6 @@ make_barriers(Ppoly_t ** poly, int npoly, int pp, int qp,
 	      Pedge_t ** barriers, int *n_barriers)
 {
     int i, j, k, n, b;
-    Pedge_t *bar;
 
     n = 0;
     for (i = 0; i < npoly; i++) {
@@ -56,7 +54,7 @@ make_barriers(Ppoly_t ** poly, int npoly, int pp, int qp,
 	    continue;
 	n += poly[i]->pn;
     }
-    bar = N_GNEW(n, Pedge_t);
+    Pedge_t *bar = gv_calloc(n, sizeof(Pedge_t));
     b = 0;
     for (i = 0; i < npoly; i++) {
 	if (i == pp)
@@ -77,8 +75,6 @@ make_barriers(Ppoly_t ** poly, int npoly, int pp, int qp,
     *n_barriers = n;
 }
 
-/* genPt:
- */
 static Ppoint_t genPt(double x, double y, pointf c)
 {
     Ppoint_t p;
@@ -88,9 +84,6 @@ static Ppoint_t genPt(double x, double y, pointf c)
     return p;
 }
 
-
-/* recPt:
- */
 static Ppoint_t recPt(double x, double y, pointf c, expand_t* m)
 {
     Ppoint_t p;
@@ -116,9 +109,9 @@ static void *newitem(Dt_t * d, edgeitem * obj, Dtdisc_t * disc)
 {
     edgeitem *newp;
 
-    NOTUSED(d);
-    NOTUSED(disc);
-    newp = NEW(edgeitem);
+    (void)d;
+    (void)disc;
+    newp = gv_alloc(sizeof(edgeitem));
     newp->id = obj->id;
     newp->e = obj->e;
     ED_count(newp->e) = 1;
@@ -128,16 +121,16 @@ static void *newitem(Dt_t * d, edgeitem * obj, Dtdisc_t * disc)
 
 static void freeitem(Dt_t * d, edgeitem * obj, Dtdisc_t * disc)
 {
-    NOTUSED(d);
-    NOTUSED(disc);
+    (void)d;
+    (void)disc;
     free(obj);
 }
 
 static int
 cmpitems(Dt_t * d, edgeinfo * key1, edgeinfo * key2, Dtdisc_t * disc)
 {
-    NOTUSED(d);
-    NOTUSED(disc);
+    (void)d;
+    (void)disc;
 
     if (key1->n1 > key2->n1)
 	return 1;
@@ -248,7 +241,7 @@ void makeSelfArcs(edge_t * e, int stepx)
 	makePortLabels(e);
     } else {
 	int i;
-	edge_t **edges = N_GNEW(cnt, edge_t *);
+	edge_t **edges = gv_calloc(cnt, sizeof(edge_t*));
 	for (i = 0; i < cnt; i++) {
 	    edges[i] = e;
 	    e = ED_to_virt(e);
@@ -276,7 +269,7 @@ void makeSelfArcs(edge_t * e, int stepx)
  * The polygon has its vertices in CW order.
  * 
  */
-Ppoly_t *makeObstacle(node_t * n, expand_t* pmargin, boolean isOrtho)
+Ppoly_t *makeObstacle(node_t * n, expand_t* pmargin, bool isOrtho)
 {
     Ppoly_t *obs;
     polygon_t *poly;
@@ -286,22 +279,21 @@ Ppoly_t *makeObstacle(node_t * n, expand_t* pmargin, boolean isOrtho)
     boxf b;
     pointf pt;
     field_t *fld;
-    int isPoly;
+    bool isPoly;
     pointf* verts = NULL;
     pointf vs[4];
     pointf p;
-    pointf margin;
+    pointf margin = {0};
 
     switch (shapeOf(n)) {
     case SH_POLY:
     case SH_POINT:
-	obs = NEW(Ppoly_t);
-	poly = (polygon_t *) ND_shape_info(n);
+	obs = gv_alloc(sizeof(Ppoly_t));
+	poly = ND_shape_info(n);
 	if (isOrtho) {
-	    isPoly = 1;
+	    isPoly = true;
 	    sides = 4;
 	    verts = vs;
-	    margin.x = margin.y = 0;
 		/* For fixedshape, we can't use the width and height, as this includes
 		 * the label. We only want to use the actual node shape.
 		 */
@@ -326,18 +318,18 @@ Ppoly_t *makeObstacle(node_t * n, expand_t* pmargin, boolean isOrtho)
 	    }
 	}
 	else if (poly->sides >= 3) {
-	    isPoly = 1;
+	    isPoly = true;
 	    sides = poly->sides;
 	    verts = poly->vertices;
 	    margin.x = pmargin->x;
 	    margin.y = pmargin->y;
 	} else {		/* ellipse */
-	    isPoly = 0;
+	    isPoly = false;
 	    sides = 8;
 	    adj = drand48() * .01;
 	}
 	obs->pn = sides;
-	obs->ps = N_NEW(sides, Ppoint_t);
+	obs->ps = gv_calloc(sides, sizeof(Ppoint_t));
 	/* assuming polys are in CCW order, and pathplan needs CW */
 	for (j = 0; j < sides; j++) {
 	    double xmargin = 0.0, ymargin = 0.0;
@@ -395,11 +387,11 @@ Ppoly_t *makeObstacle(node_t * n, expand_t* pmargin, boolean isOrtho)
 	}
 	break;
     case SH_RECORD:
-	fld = (field_t *) ND_shape_info(n);
+	fld = ND_shape_info(n);
 	b = fld->b;
-	obs = NEW(Ppoly_t);
+	obs = gv_alloc(sizeof(Ppoly_t));
 	obs->pn = 4;
-	obs->ps = N_NEW(4, Ppoint_t);
+	obs->ps = gv_calloc(4, sizeof(Ppoint_t));
 	/* CW order */
 	pt = ND_coord(n);
 	if (pmargin->doAdd) {
@@ -416,9 +408,9 @@ Ppoly_t *makeObstacle(node_t * n, expand_t* pmargin, boolean isOrtho)
 	}
 	break;
     case SH_EPSF:
-	obs = NEW(Ppoly_t);
+	obs = gv_alloc(sizeof(Ppoly_t));
 	obs->pn = 4;
-	obs->ps = N_NEW(4, Ppoint_t);
+	obs->ps = gv_calloc(4, sizeof(Ppoint_t));
 	/* CW order */
 	pt = ND_coord(n);
 	if (pmargin->doAdd) {
@@ -443,17 +435,13 @@ Ppoly_t *makeObstacle(node_t * n, expand_t* pmargin, boolean isOrtho)
 
 /* getPath
  * Construct the shortest path from one endpoint of e to the other.
- * The obstacles and their number are given by obs and npoly.
  * vconfig is a precomputed data structure to help in the computation.
  * If chkPts is true, the function finds the polygons, if any, containing
  * the endpoints and tells the shortest path computation to ignore them. 
  * Assumes this info is set in ND_lim, usually from _spline_edges.
  * Returns the shortest path.
  */
-Ppolyline_t
-getPath(edge_t * e, vconfig_t * vconfig, int chkPts, Ppoly_t ** obs,
-	int npoly)
-{
+Ppolyline_t getPath(edge_t *e, vconfig_t *vconfig, bool chkPts) {
     Ppolyline_t line;
     int pp, qp;
     Ppoint_t p, q;
@@ -471,19 +459,14 @@ getPath(edge_t * e, vconfig_t * vconfig, int chkPts, Ppoly_t ** obs,
     return line;
 }
 
-/* makePolyline:
- */
 static void makePolyline(edge_t * e) {
     Ppolyline_t spl, line = ED_path(e);
-    Ppoint_t p0, q0;
 
-    p0 = line.ps[0];
-    q0 = line.ps[line.pn - 1];
     make_polyline (line, &spl);
     if (Verbose > 1)
 	fprintf(stderr, "polyline %s %s\n", agnameof(agtail(e)), agnameof(aghead(e)));
     clip_and_install(e, aghead(e), spl.ps, spl.pn, &sinfo);
-    addEdgeLabels(e, p0, q0);
+    addEdgeLabels(e);
 }
 
 /* makeSpline:
@@ -496,8 +479,7 @@ static void makePolyline(edge_t * e) {
  * is on or inside one of the obstacles and, if so, tells the shortest path
  * computation to ignore them. 
  */
-void makeSpline(edge_t * e, Ppoly_t ** obs, int npoly, boolean chkPts)
-{
+void makeSpline(edge_t *e, Ppoly_t **obs, int npoly, bool chkPts) {
     Ppolyline_t line, spline;
     Pvector_t slopes[2];
     int i, n_barriers;
@@ -531,7 +513,7 @@ void makeSpline(edge_t * e, Ppoly_t ** obs, int npoly, boolean chkPts)
 	fprintf(stderr, "spline %s %s\n", agnameof(agtail(e)), agnameof(aghead(e)));
     clip_and_install(e, aghead(e), spline.ps, spline.pn, &sinfo);
     free(barriers);
-    addEdgeLabels(e, p, q);
+    addEdgeLabels(e);
 }
 
   /* True if either head or tail has a port on its boundary */
@@ -556,7 +538,6 @@ static int _spline_edges(graph_t * g, expand_t* pmargin, int edgetype)
     Ppoly_t *obp;
     int cnt, i = 0, npoly;
     vconfig_t *vconfig = 0;
-    path *P = NULL;
     int useEdges = Nop > 1;
     int legal = 0;
 
@@ -566,7 +547,7 @@ static int _spline_edges(graph_t * g, expand_t* pmargin, int edgetype)
     
     /* build configuration */
     if (edgetype >= EDGETYPE_PLINE) {
-	obs = N_NEW(agnnodes(g), Ppoly_t *);
+	obs = gv_calloc(agnnodes(g), sizeof(Ppoly_t*));
 	for (n = agfstnode(g); n; n = agnxtnode(g, n)) {
 	    obp = makeObstacle(n, pmargin, edgetype == EDGETYPE_ORTHO);
 	    if (obp) {
@@ -602,7 +583,7 @@ static int _spline_edges(graph_t * g, expand_t* pmargin, int edgetype)
 	/* path-finding pass */
 	for (n = agfstnode(g); n; n = agnxtnode(g, n)) {
 	    for (e = agfstout(g, n); e; e = agnxtout(g, e)) {
-		ED_path(e) = getPath(e, vconfig, TRUE, obs, npoly);
+		ED_path(e) = getPath(e, vconfig, true);
 	    }
 	}
     }
@@ -619,16 +600,12 @@ static int _spline_edges(graph_t * g, expand_t* pmargin, int edgetype)
 /* fprintf (stderr, "%s -- %s %d\n", agnameof(agtail(e)), agnameof(aghead(e)), ED_count(e)); */
 	    node_t *head = aghead(e);
 	    if (useEdges && ED_spl(e)) {
-		addEdgeLabels(e,
-			      add_pointf(ND_coord(n), ED_tail_port(e).p),
-			      add_pointf(ND_coord(head), ED_head_port(e).p));
+		add_pointf(ND_coord(n), ED_tail_port(e).p);
+		add_pointf(ND_coord(head), ED_head_port(e).p);
+		addEdgeLabels(e);
 	    } 
 	    else if (ED_count(e) == 0) continue;  /* only do representative */
 	    else if (n == head) {    /* self arc */
-		if (!P) {
-		    P = NEW(path);
-		    P->boxes = N_NEW(agnnodes(g) + 20 * 2 * 9, boxf);
-		}
 		makeSelfArcs(e, GD_nodesep(g->root));
 	    } else if (vconfig) { /* EDGETYPE_SPLINE or EDGETYPE_PLINE */
 #ifdef HAVE_GTS
@@ -653,7 +630,7 @@ static int _spline_edges(graph_t * g, expand_t* pmargin, int edgetype)
 		e0 = e;
 		for (i = 0; i < cnt; i++) {
 		    if (edgetype == EDGETYPE_SPLINE)
-			makeSpline(e0, obs, npoly, TRUE);
+			makeSpline(e0, obs, npoly, true);
 		    else
 			makePolyline(e0);
 		    e0 = ED_to_virt(e0);
@@ -671,10 +648,6 @@ static int _spline_edges(graph_t * g, expand_t* pmargin, int edgetype)
 
     if (vconfig)
 	Pobsclose (vconfig);
-    if (P) {
-	free(P->boxes);
-	free(P);
-    }
     if (obs) {
 	for (i=0; i < npoly; i++) {
 	    free (obs[i]->ps);
@@ -766,8 +739,7 @@ int spline_edges1(graph_t * g, int edgetype)
  * when output in dot or plain format.
  *
  */
-void spline_edges0(graph_t * g, boolean set_aspect)
-{
+void spline_edges0(graph_t *g, bool set_aspect) {
     int et = EDGE_TYPE (g);
     if (set_aspect) neato_set_aspect(g);
     if (et == EDGETYPE_NONE) return;
@@ -782,8 +754,6 @@ void spline_edges0(graph_t * g, boolean set_aspect)
     spline_edges1(g, et);
 }
 
-/* shiftClusters:
- */
 static void
 shiftClusters (graph_t * g, pointf offset)
 {
@@ -817,7 +787,7 @@ void spline_edges(graph_t * g)
     }
 	
     shiftClusters (g, GD_bb(g).LL);
-    spline_edges0(g, TRUE);
+    spline_edges0(g, true);
 }
 
 /* scaleEdge:
@@ -947,8 +917,6 @@ static void translateE(edge_t * e, pointf offset)
     }
 }
 
-/* translateG:
- */
 static void translateG(Agraph_t * g, pointf offset)
 {
     int i;
@@ -967,8 +935,6 @@ static void translateG(Agraph_t * g, pointf offset)
 	translateG(GD_clust(g)[i], offset);
 }
 
-/* neato_translate:
- */
 void neato_translate(Agraph_t * g)
 {
     node_t *n;
@@ -1017,9 +983,7 @@ static bool _neato_set_aspect(graph_t * g)
 	}
 	/* normalize */
 	if (GD_flip(g)) {
-	    double t = GD_bb(g).UR.x;
-	    GD_bb(g).UR.x = GD_bb(g).UR.y;
-	    GD_bb(g).UR.y = t;
+	    GD_bb(g).UR = exch_xyf(GD_bb(g).UR);
 	}
 	if (GD_drawing(g)->ratio_kind == R_FILL) {
 	    /* fill is weird because both X and Y can stretch */
@@ -1043,7 +1007,7 @@ static bool _neato_set_aspect(graph_t * g)
 	    xf = (double) GD_drawing(g)->size.x / GD_bb(g).UR.x;
 	    yf = (double) GD_drawing(g)->size.y / GD_bb(g).UR.y;
 	    if (xf > 1.0 && yf > 1.0) {
-		double scale = MIN(xf, yf);
+		double scale = fmin(xf, yf);
 		xf = yf = scale;
 	    } else
 		return translated;
