@@ -1,6 +1,3 @@
-/* $Id$ $Revision$ */
-/* vim:set shiftwidth=4 ts=8: */
-
 /*************************************************************************
  * Copyright (c) 2011 AT&T Intellectual Property 
  * All rights reserved. This program and the accompanying materials
@@ -8,11 +5,11 @@
  * which accompanies this distribution, and is available at
  * http://www.eclipse.org/legal/epl-v10.html
  *
- * Contributors: See CVS logs. Details at http://www.graphviz.org/
+ * Contributors: Details at https://graphviz.org
  *************************************************************************/
 
 #include <stdio.h>
-#include <vis.h>
+#include <pathplan/vis.h>
 typedef Ppoint_t point;
 
 
@@ -30,8 +27,7 @@ void printvis(vconfig_t * cp)
 
     printf("this next prev point\n");
     for (i = 0; i < cp->N; i++)
-	printf("%3d  %3d  %3d    (%f,%f)\n", i, next[i], prev[i],
-	       (double) pts[i].x, (double) pts[i].y);
+	printf("%3d  %3d  %3d    (%f,%f)\n", i, next[i], prev[i], pts[i].x, pts[i].y);
 
     printf("\n\n");
 
