@@ -10,11 +10,10 @@
 
 
 #include <assert.h>
+#include <cgraph/alloc.h>
 #include <stdlib.h>
 #include <pathplan/pathutil.h>
 
-#define ALLOC(size,ptr,type) realloc(ptr,(size)*sizeof(type))
-
 void freePath(Ppolyline_t* p)
 {
     free(p->ps);
@@ -26,13 +25,12 @@ int Ppolybarriers(Ppoly_t ** polys, int npolys, Pedge_t ** barriers,
 {
     Ppoly_t pp;
     int i, j, k, n, b;
-    Pedge_t *bar;
 
     n = 0;
     for (i = 0; i < npolys; i++)
 	n = n + polys[i]->pn;
 
-    bar = malloc(n * sizeof(Pedge_t));
+    Pedge_t *bar = gv_calloc(n, sizeof(Pedge_t));
 
     b = 0;
     for (i = 0; i < npolys; i++) {
@@ -63,7 +61,7 @@ make_polyline(Ppolyline_t line, Ppolyline_t* sline)
     int npts = 4 + 3*(line.pn-2);
 
     if (npts > isz) {
-	ispline = ALLOC(npts, ispline, Ppoint_t); 
+	ispline = gv_recalloc(ispline, (size_t)isz, (size_t)npts, sizeof(Ppoint_t));
 	isz = npts;
     }
 
@@ -81,3 +79,7 @@ make_polyline(Ppolyline_t line, Ppolyline_t* sline)
     sline->ps = ispline;
 }
 
+/**
+ * @dir lib/pathplan
+ * @brief finds and smooths shortest paths, API pathplan.h
+ */
