@@ -8,13 +8,19 @@
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
 
-#include "config.h"
-
 #include <iostream>
 #include <fstream>
 #include <stdexcept>
 #include <LASi.h>
 
+// Confusingly, Freetype (included transitively by LASi), ship their Autotools
+// config.h. As a result, we get macros like `HAVE_FCNTL_H` injected from their
+// headers, overriding ours. Suppress this and force ours into scope.
+#ifdef HAVE_FCNTL_H
+#undef HAVE_FCNTL_H
+#endif
+#include "config.h"
+
 #include <gvc/gvplugin_render.h>
 #include <gvc/gvplugin_device.h>
 #include <gvc/gvio.h>
@@ -316,7 +322,6 @@ static void ps_set_color(GVJ_t *job, gvcolor_t *color)
 
 static void lasi_textspan(GVJ_t * job, pointf p, textspan_t * span)
 {
-    char *str;
     const char *font;
     const PangoFontDescription *pango_font;
     FontStretch stretch; 
@@ -350,15 +355,37 @@ static void lasi_textspan(GVJ_t * job, pointf p, textspan_t * span)
 	switch (pango_font_description_get_variant(pango_font)) {
 	    case PANGO_VARIANT_NORMAL: variant = NORMAL_VARIANT; break;
 	    case PANGO_VARIANT_SMALL_CAPS: variant = SMALLCAPS; break;
+#if PANGO_VERSION_CHECK(1, 50, 0)
+	    case PANGO_VARIANT_ALL_SMALL_CAPS: variant = SMALLCAPS; break;
+	    case PANGO_VARIANT_PETITE_CAPS: variant = SMALLCAPS; break;
+	    case PANGO_VARIANT_ALL_PETITE_CAPS: variant = SMALLCAPS; break;
+	    case PANGO_VARIANT_UNICASE: variant = SMALLCAPS; break;
+	    case PANGO_VARIANT_TITLE_CAPS: variant = SMALLCAPS; break;
+#endif
 	}
 	switch (pango_font_description_get_weight(pango_font)) {
+#if PANGO_VERSION_CHECK(1, 24, 0)
+	    case PANGO_WEIGHT_THIN: weight = ULTRALIGHT; break; // no exact match in LASi
+#endif
 	    case PANGO_WEIGHT_ULTRALIGHT: weight = ULTRALIGHT; break;
 	    case PANGO_WEIGHT_LIGHT: weight = LIGHT; break;
+#if PANGO_VERSION_CHECK(1, 36, 7)
+	    case PANGO_WEIGHT_SEMILIGHT: weight = LIGHT; break; // no exact match in LASi
+#endif
+#if PANGO_VERSION_CHECK(1, 24, 0)
+	    case PANGO_WEIGHT_BOOK: weight = NORMAL_WEIGHT; break; // no exact match in LASi
+#endif
 	    case PANGO_WEIGHT_NORMAL: weight = NORMAL_WEIGHT; break;
+#if PANGO_VERSION_CHECK(1, 24, 0)
+	    case PANGO_WEIGHT_MEDIUM: weight = NORMAL_WEIGHT; break; // no exact match in LASi
+#endif
 	    case PANGO_WEIGHT_SEMIBOLD: weight = BOLD; break; /* no exact match in lasi */
 	    case PANGO_WEIGHT_BOLD: weight = BOLD; break;
 	    case PANGO_WEIGHT_ULTRABOLD: weight = ULTRABOLD; break;
 	    case PANGO_WEIGHT_HEAVY: weight = HEAVY; break;
+#if PANGO_VERSION_CHECK(1, 24, 0)
+	    case PANGO_WEIGHT_ULTRAHEAVY: weight = HEAVY; break; // no exact match in LASi
+#endif
 	}
     }
     else {
@@ -422,8 +449,7 @@ static void lasi_ellipse(GVJ_t * job, pointf * A, int filled)
     }
 }
 
-static void lasi_bezier(GVJ_t * job, pointf * A, int n, int, int, int filled)
-{
+static void lasi_bezier(GVJ_t *job, pointf *A, int n, int filled) {
     int j;
 
     if (filled && job->obj->fillcolor.u.HSVA[3] > .5) {
@@ -507,10 +533,11 @@ static void lasi_comment(GVJ_t * job, char *str)
 
 static void lasi_library_shape(GVJ_t * job, char *name, pointf * A, int n, int filled)
 {
+    assert(n >= 0);
     if (filled && job->obj->fillcolor.u.HSVA[3] > .5) {
 	ps_set_color(job, &(job->obj->fillcolor));
 	gvputs(job, "[ ");
-	gvprintpointflist(job, A, n);
+	gvprintpointflist(job, A, (size_t)n);
 	gvputs(job, " ");
 	gvprintpointf(job, A[0]);
 	gvprintf(job, " ]  %d true %s\n", n, name);
