@@ -10,6 +10,8 @@
 
 #include "config.h"
 
+#include <assert.h>
+#include <limits.h>
 #include <stdbool.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -36,6 +38,8 @@
 #include <fcntl.h>
 #endif
 
+#include <cgraph/agxbuf.h>
+#include <cgraph/exit.h>
 #include <gvc/gvplugin_device.h>
 
 #include <cairo.h>
@@ -63,14 +67,18 @@ static void handle_configure_notify(GVJ_t * job, XConfigureEvent * cev)
 /*	plugin/xlib/gvdevice_xlib.c */
 /*	lib/gvc/gvevent.c */
 
+    assert(cev->width >= 0 && "Xlib returned an event with negative width");
+    assert(cev->height >= 0 && "Xlib returned an event with negative height");
+
     job->zoom *= 1 + MIN(
 	((double) cev->width - (double) job->width) / (double) job->width,
 	((double) cev->height - (double) job->height) / (double) job->height);
-    if (cev->width > job->width || cev->height > job->height)
-        job->has_grown = 1;
-    job->width = cev->width;
-    job->height = cev->height;
-    job->needs_refresh = 1;
+    if ((unsigned)cev->width > job->width ||
+        (unsigned)cev->height > job->height)
+        job->has_grown = true;
+    job->width = (unsigned)cev->width;
+    job->height = (unsigned)cev->height;
+    job->needs_refresh = true;
 }
 
 static void handle_expose(GVJ_t * job, XExposeEvent * eev)
@@ -78,8 +86,13 @@ static void handle_expose(GVJ_t * job, XExposeEvent * eev)
     window_t *window;
 
     window = job->window;
+    assert(eev->width >= 0 &&
+           "Xlib returned an expose event with negative width");
+    assert(eev->height >= 0 &&
+           "Xlib returned an expose event with negative height");
     XCopyArea(eev->display, window->pix, eev->window, window->gc,
-              eev->x, eev->y, eev->width, eev->height, eev->x, eev->y);
+              eev->x, eev->y, (unsigned)eev->width, (unsigned)eev->height,
+              eev->x, eev->y);
 }
 
 static void handle_client_message(GVJ_t * job, XClientMessageEvent * cmev)
@@ -89,10 +102,10 @@ static void handle_client_message(GVJ_t * job, XClientMessageEvent * cmev)
     window = job->window;
     if (cmev->format == 32
         && (Atom) cmev->data.l[0] == window->wm_delete_window_atom)
-        exit(0);
+        graphviz_exit(0);
 }
 
-static boolean handle_keypress(GVJ_t *job, XKeyEvent *kev)
+static bool handle_keypress(GVJ_t *job, XKeyEvent *kev)
 {
     
     int i;
@@ -101,9 +114,9 @@ static boolean handle_keypress(GVJ_t *job, XKeyEvent *kev)
     keycodes = job->keycodes;
     for (i=0; i < job->numkeys; i++) {
 	if (kev->keycode == keycodes[i])
-	    return (job->keybindings[i].callback)(job);
+	    return job->keybindings[i].callback(job) != 0;
     }
-    return FALSE;
+    return false;
 }
 
 static Visual *find_argb_visual(Display * dpy, int scr)
@@ -139,10 +152,9 @@ static Visual *find_argb_visual(Display * dpy, int scr)
 
 static void browser_show(GVJ_t *job)
 {
-#if defined HAVE_SYS_TYPES_H && defined HAVE_UNISTD_H
+#ifdef HAVE_SYS_TYPES_H
    char *exec_argv[3] = {BROWSER, NULL, NULL};
    pid_t pid;
-   int err;
 
    exec_argv[1] = job->selected_href;
 
@@ -151,7 +163,7 @@ static void browser_show(GVJ_t *job)
        fprintf(stderr,"fork failed: %s\n", strerror(errno));
    }
    else if (pid == 0) {
-       err = execvp(exec_argv[0], exec_argv);
+       execvp(exec_argv[0], exec_argv);
        fprintf(stderr,"error starting %s: %s\n", exec_argv[0], strerror(errno));
    }
 #else
@@ -177,21 +189,27 @@ static int handle_xlib_events (GVJ_t *firstjob, Display *dpy)
                 case ButtonPress:
 		    pointer.x = (double)xev.xbutton.x;
 		    pointer.y = (double)xev.xbutton.y;
-                    (job->callbacks->button_press)(job, xev.xbutton.button, pointer);
+                    assert(xev.xbutton.button <= (unsigned)INT_MAX &&
+                           "Xlib returned invalid button event");
+                    job->callbacks->button_press(job, (int)xev.xbutton.button,
+                                                 pointer);
 		    rc++;
                     break;
                 case MotionNotify:
 		    if (job->button) { /* only interested while a button is pressed */
 		        pointer.x = (double)xev.xbutton.x;
 		        pointer.y = (double)xev.xbutton.y;
-                        (job->callbacks->motion)(job, pointer);
+                        job->callbacks->motion(job, pointer);
 		        rc++;
 		    }
                     break;
                 case ButtonRelease:
 		    pointer.x = (double)xev.xbutton.x;
 		    pointer.y = (double)xev.xbutton.y;
-                    (job->callbacks->button_release)(job, xev.xbutton.button, pointer);
+                    assert(xev.xbutton.button <= (unsigned)INT_MAX &&
+                           "Xlib returned invalid button event");
+                    job->callbacks->button_release(job, (int)xev.xbutton.button,
+                                                   pointer);
 		    if (job->selected_href && job->selected_href[0] && xev.xbutton.button == 1)
 		        browser_show(job);
 		    rc++;
@@ -213,6 +231,8 @@ static int handle_xlib_events (GVJ_t *firstjob, Display *dpy)
                     handle_client_message(job, &xev.xclient);
 		    rc++;
                     break;
+                default:
+                    break;
                 }
 	        break;
 	    }
@@ -228,26 +248,32 @@ static void update_display(GVJ_t *job, Display *dpy)
 
     window = job->window;
 
+    // window geometry is set to fixed values
+    assert(job->width <= (unsigned)INT_MAX && "out of range width");
+    assert(job->height <= (unsigned)INT_MAX && "out of range height");
+
     if (job->has_grown) {
 	XFreePixmap(dpy, window->pix);
+	assert(window->depth >= 0 && "Xlib returned invalid window depth");
 	window->pix = XCreatePixmap(dpy, window->win,
-			job->width, job->height, window->depth);
-	job->has_grown = 0;
-	job->needs_refresh = 1;
+	                            job->width, job->height,
+	                            (unsigned)window->depth);
+	job->has_grown = false;
+	job->needs_refresh = true;
     }
     if (job->needs_refresh) {
 	XFillRectangle(dpy, window->pix, window->gc, 0, 0,
                 	job->width, job->height);
 	surface = cairo_xlib_surface_create(dpy,
 			window->pix, window->visual,
-			job->width, job->height);
-    	job->context = (void *)cairo_create(surface);
-	job->external_context = TRUE;
-        (job->callbacks->refresh)(job);
+			(int)job->width, (int)job->height);
+    	job->context = cairo_create(surface);
+	job->external_context = true;
+        job->callbacks->refresh(job);
 	cairo_surface_destroy(surface);
 	XCopyArea(dpy, window->pix, window->win, window->gc,
 			0, 0, job->width, job->height, 0, 0);
-        job->needs_refresh = 0;
+        job->needs_refresh = false;
     }
 }
 
@@ -261,9 +287,7 @@ static void init_window(GVJ_t *job, Display *dpy, int scr)
     XSizeHints *normalhints;
     XClassHint *classhint;
     uint64_t attributemask = 0;
-    char *name;
     window_t *window;
-    int w, h;
     double zoom_to_fit;
 
     window = malloc(sizeof(window_t));
@@ -272,8 +296,8 @@ static void init_window(GVJ_t *job, Display *dpy, int scr)
 	return;
     }
 
-    w = 480;    /* FIXME - w,h should be set by a --geometry commandline option */
-    h = 325;
+    unsigned w = 480;    /* FIXME - w,h should be set by a --geometry commandline option */
+    unsigned h = 325;
     
     zoom_to_fit = MIN((double) w / (double) job->width,
 			(double) h / (double) job->height);
@@ -284,8 +308,8 @@ static void init_window(GVJ_t *job, Display *dpy, int scr)
     job->height = h;
 
     job->window = window;
-    job->fit_mode = 0;
-    job->needs_refresh = 1;
+    job->fit_mode = false;
+    job->needs_refresh = true;
 
     if (argb && (window->visual = find_argb_visual(dpy, scr))) {
         window->cmap = XCreateColormap(dpy, RootWindow(dpy, scr),
@@ -313,16 +337,17 @@ static void init_window(GVJ_t *job, Display *dpy, int scr)
                              InputOutput, window->visual,
                              attributemask, &attributes);
 
-    name = malloc(strlen("graphviz: ") + strlen(base) + 1);
-    strcpy(name, "graphviz: ");
-    strcat(name, base);
+    agxbuf name = {0};
+    agxbprint(&name, "graphviz: %s", base);
 
     normalhints = XAllocSizeHints();
     normalhints->flags = 0;
     normalhints->x = 0;
     normalhints->y = 0;
-    normalhints->width = job->width;
-    normalhints->height = job->height;
+    assert(job->width <= (unsigned)INT_MAX && "out of range width");
+    normalhints->width = (int)job->width;
+    assert(job->height <= (unsigned)INT_MAX && "out of range height");
+    normalhints->height = (int)job->height;
 
     classhint = XAllocClassHint();
     classhint->res_name = "graphviz";
@@ -332,15 +357,16 @@ static void init_window(GVJ_t *job, Display *dpy, int scr)
     wmhints->flags = InputHint;
     wmhints->input = True;
 
-    Xutf8SetWMProperties(dpy, window->win, name, base, 0, 0,
+    Xutf8SetWMProperties(dpy, window->win, agxbuse(&name), base, 0, 0,
                          normalhints, wmhints, classhint);
     XFree(wmhints);
     XFree(classhint);
     XFree(normalhints);
-    free(name);
+    agxbfree(&name);
 
+    assert(window->depth >= 0 && "Xlib returned invalid window depth");
     window->pix = XCreatePixmap(dpy, window->win, job->width, job->height,
-		window->depth);
+                                (unsigned)window->depth);
     if (argb)
         gcv.foreground = 0;
     else
@@ -355,20 +381,20 @@ static void init_window(GVJ_t *job, Display *dpy, int scr)
         | KeyPressMask
         | StructureNotifyMask
         | ExposureMask);
-    XSelectInput(dpy, window->win, window->event_mask);
+    XSelectInput(dpy, window->win, (long)window->event_mask);
     window->wm_delete_window_atom =
         XInternAtom(dpy, "WM_DELETE_WINDOW", False);
     XSetWMProtocols(dpy, window->win, &window->wm_delete_window_atom, 1);
     XMapWindow(dpy, window->win);
 }
 
-static int handle_stdin_events(GVJ_t *job, int stdin_fd)
+static int handle_stdin_events(GVJ_t *job)
 {
     int rc=0;
 
     if (feof(stdin))
 	return -1;
-    (job->callbacks->read)(job, job->input_filename, job->layout_type);
+    job->callbacks->read(job, job->input_filename, job->layout_type);
     
     rc++;
     return rc;
@@ -377,8 +403,7 @@ static int handle_stdin_events(GVJ_t *job, int stdin_fd)
 #ifdef HAVE_SYS_INOTIFY_H
 static int handle_file_events(GVJ_t *job, int inotify_fd)
 {
-    int avail, ret, len, ln, rc = 0;
-    static char *buf;
+    int avail, ret, len, rc = 0;
     char *bf, *p;
     struct inotify_event *event;
 
@@ -389,54 +414,39 @@ static int handle_file_events(GVJ_t *job, int inotify_fd)
     }
 
     if (avail) {
-        buf = realloc(buf, avail);
+        assert(avail > 0 && "invalid value from FIONREAD");
+        void *buf = malloc((size_t)avail);
         if (!buf) {
-            fprintf(stderr,"problem with realloc(%d)\n", avail);
+            fprintf(stderr, "out of memory (could not allocate %d bytes)\n",
+                    avail);
             return -1;
         }
-        len = read(inotify_fd, buf, avail);
+        len = (int)read(inotify_fd, buf, (size_t)avail);
         if (len != avail) {
-            fprintf(stderr,"avail = %u, len = %u\n", avail, len);
+            fprintf(stderr,"avail = %d, len = %d\n", avail, len);
+            free(buf);
             return -1;
         }
         bf = buf;
         while (len > 0) {
     	    event = (struct inotify_event *)bf;
-	    switch (event->mask) {
-	    case IN_MODIFY:
+	    if (event->mask == IN_MODIFY) {
 		p = strrchr(job->input_filename, '/');
 		if (p)
 		    p++;
 		else 
 		    p = job->input_filename;
-		if (strcmp((char*)(&(event->name)), p) == 0) {
-		    (job->callbacks->read)(job, job->input_filename, job->layout_type);
+		if (strcmp(event->name, p) == 0) {
+		    job->callbacks->read(job, job->input_filename, job->layout_type);
 		    rc++;
 		}
-		break;
-
-            case IN_ACCESS:
-            case IN_ATTRIB:
-            case IN_CLOSE_WRITE:
-            case IN_CLOSE_NOWRITE:
-            case IN_OPEN:
-            case IN_MOVED_FROM:
-            case IN_MOVED_TO:
-            case IN_CREATE:
-            case IN_DELETE:
-            case IN_DELETE_SELF:
-            case IN_MOVE_SELF:
-            case IN_UNMOUNT:
-            case IN_Q_OVERFLOW:
-            case IN_IGNORED:
-            case IN_ISDIR:
-            case IN_ONESHOT:
-		break;
     	    }
-	    ln = event->len + sizeof(struct inotify_event);
+	    size_t ln = event->len + sizeof(struct inotify_event);
+            assert(ln <= (size_t)len);
             bf += ln;
-            len -= ln;
+            len -= (int)ln;
         }
+        free(buf);
         if (len != 0) {
             fprintf(stderr,"length miscalculation, len = %d\n", len);
             return -1;
@@ -467,7 +477,8 @@ static void xlib_initialize(GVJ_t *firstjob)
     firstjob->display = dpy;
     firstjob->screen = scr;
 
-    keycodes = malloc(firstjob->numkeys * sizeof(KeyCode));
+    assert(firstjob->numkeys >= 0);
+    keycodes = malloc((size_t)firstjob->numkeys * sizeof(KeyCode));
     if (keycodes == NULL) {
         fprintf(stderr, "Failed to malloc %d*KeyCode\n", firstjob->numkeys);
         return;
@@ -484,7 +495,7 @@ static void xlib_initialize(GVJ_t *firstjob)
 
     firstjob->device_dpi.x = DisplayWidth(dpy, scr) * 25.4 / DisplayWidthMM(dpy, scr);
     firstjob->device_dpi.y = DisplayHeight(dpy, scr) * 25.4 / DisplayHeightMM(dpy, scr);
-    firstjob->device_sets_dpi = TRUE;
+    firstjob->device_sets_dpi = true;
 
     initialized = true;
 }
@@ -502,7 +513,6 @@ static void xlib_finalize(GVJ_t *firstjob)
     int wd=0;
     int inotify_fd=0;
     bool watching_file_p = false;
-    static char *dir;
     char *p, *cwd = NULL;
 
     inotify_fd = inotify_init();
@@ -524,22 +534,21 @@ static void xlib_finalize(GVJ_t *firstjob)
 #ifdef HAVE_SYS_INOTIFY_H
 	    watching_file_p = true;
 
+	    agxbuf dir = {0};
 	    if (firstjob->input_filename[0] != '/') {
     	        cwd = getcwd(NULL, 0);
-	        dir = realloc(dir, strlen(cwd) + 1 + strlen(firstjob->input_filename) + 1);
-	        strcpy(dir, cwd);
-	        strcat(dir, "/");
-	        strcat(dir, firstjob->input_filename);
+	        agxbprint(&dir, "%s/%s", cwd, firstjob->input_filename);
 	        free(cwd);
 	    }
 	    else {
-	        dir = realloc(dir, strlen(firstjob->input_filename) + 1);
-	        strcpy(dir, firstjob->input_filename);
+	        agxbput(&dir, firstjob->input_filename);
 	    }
-	    p = strrchr(dir,'/');
+	    char *dirstr = agxbuse(&dir);
+	    p = strrchr(dirstr,'/');
 	    *p = '\0';
     
-    	    wd = inotify_add_watch(inotify_fd, dir, IN_MODIFY );
+    	    wd = inotify_add_watch(inotify_fd, dirstr, IN_MODIFY);
+	    agxbfree(&dir);
 
             numfds = MAX(inotify_fd, numfds);
 #endif
@@ -573,7 +582,7 @@ static void xlib_finalize(GVJ_t *firstjob)
 
 	if (watching_stdin_p) {
 	    if (FD_ISSET(stdin_fd, &rfds)) {
-                ret = handle_stdin_events(firstjob, stdin_fd);
+                ret = handle_stdin_events(firstjob);
 	        if (ret < 0) {
 	            watching_stdin_p = false;
                     FD_CLR(stdin_fd, &rfds);
